/*======================================================================

 Project Name : S3C2443

 Copyright 2006 by Samsung Electronics, Inc.
 All rights reserved.

 Project Description :
 This software is only for verifying functions of the S3C2443. 
 Anybody can use this code without our permission.

 File Name    : iis.c
 Description  : S3C2443 IIS module code
 Author       : Junon Jeon
 Dept         : AP
 Created Date : 2006.06.02
 Version      : 0.0
 History
   R0.0 (2006.06.02): Junon draft
		- S3C2443 IIS test code is derived from S3C2413A IIS test code.
		
=======================================================================*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "2443addr.h"
#include "option.h"
#include "System.h"
#include "console.h"
#include "iis.h"	
#include "PLL.h"
#if 1//jijeon 2008 05 07 
#include "wm8960.h"
#endif

#define IIS_BUFFER 0x31000000

#define REC_LEN_IIS 0xfffff*4	//Max value of DMA Transfer Counter (20bit) = 0xfffff, Date Size=Word (4bytes)
#define PLAY_IIS_SPEAK    0
#define PLAY_IIS_HP    1
#define RECORD_MIC_IN  2

 

// Global variables
char which_Buf = 1;
volatile char Rec_Done = 0;
char IIS_MasterClk_Sel = 0; 
unsigned char test_wave[]=
{
#include "Youve Got Mail.hex"
};
	
unsigned char  *Buf,*_temp;
unsigned char *rec_buf = (unsigned char *)(test_wave+0x28);
//unsigned char *rec_buf1 = (unsigned char *)(IIS_BUFFER + REC_LEN_IIS); 
unsigned char *rec_buf1 = (unsigned char *)(IIS_BUFFER + REC_LEN_IIS); 
unsigned char *Mic_rec_buf0 = (unsigned char *)(IIS_BUFFER + 0x28); 

unsigned int size, fs; 
unsigned int save_GPE_CON, save_GPG_CON;
unsigned int save_GPE_DAT, save_GPG_DAT;

float IIS_Codec_CLK;

// Functions
// GPIO port setting
void IIS_Port_Init(void);
void IIS_Port_Return(void);


void InitWM8960(int mode);

// IIS library
unsigned int IIS_Select_SamplingRate(void);
void Select_IIS_Master_CLK(void);
void Select_PCLK(void);
void Select_EXTCLK(void);
void Select_EPLL(void);
void Select_EPLL_Ref_CLK(void);
void down_wav(void);

void __irq Rx_IIS_Int(void);
void __irq DMA1_Rec_Done(void);
void __irq DMA2_Done(void);
void __irq Play_Rec_Simul(void);


void Play_Iis_speak(void);
void Play_Iis_HP(void);
void IIS_RecSound_DMA1( int mode, unsigned int rec_size);
void IIS_RecSound_DMA1_Repeat(unsigned char *start_addr, unsigned int rec_size);
void IIS_PlayWave_DMA2(unsigned char *start_addr, unsigned int play_size);



void Data_init(void);
void IIS_PlayWave_DMA2_MASTER(unsigned char *start_addr, unsigned int play_size);
void RecordSound_ViaLineIn_MASTER(void);
void IIS_RecSound_DMA1_MASTER(int mode, unsigned int rec_size);

void IIS_PlayWave_DMA2_SLAVE(unsigned char *start_addr, unsigned int play_size);
void Play_Iis_EXTCLK(void);
void IIS_PlayWave_DMA2_EXTCLK(unsigned char *start_addr, unsigned int play_size);

void * func_iis_test[][2]=
{	
	//IIS Function Test Item
	//  									"123456789012345678901"
   	(void *)Play_Iis_speak,              			       "Play Wave    speak        ", 
   	(void *)Play_Iis_HP,              			       "Play Wave    HP       ",    	
   	(void *)RecordSound_ViaMICIn_Playit,              			       "Play Via Mic       ",       	

    0,0
};


void Test_IIS(void)
{
	int i;
		
	while(1)
	{
		i=0;
		printf("\n\n================== WM8960 Function Test =====================\n\n");
		
		while(1)
		{   //display menu
			printf("%2d:%s",i,func_iis_test[i][1]);
			i++;
			if((int)(func_iis_test[i][0])==0)
			{
				printf("\n");
				break;
			}
			if((i%1)==0)
			printf("\n");
		}
		printf("\n==========================================================\n");
		printf("Select #Item or Press Enter key to exit : ");
		i = GetIntNum(); //gh0522.cha
		if(i==-1) break;		// return.
		if(i>=0 && (i<((sizeof(func_iis_test)-1)/8)) )	// select and execute...
			( (void (*)(void)) (func_iis_test[i][0]) )();
	}
	
}


//============================== [ IIS Drivers ] ===============================

// Setting Port related to IIS  
void IIS_Port_Init(void)
{
	save_GPG_CON = rGPGCON;	// L3
	save_GPE_CON = rGPECON;	// IIS
   	
   	//----------------------------------------------------------
	//PORT G GROUP
	//Ports  :   GPG0       GPG1        GPG2  
	//Signal :  L3MODE     L3DATA      L3CLK
	//Setting:  OUTPUT     OUTPUT      OUTPUT 
	//	        [1:0]      [3:2]       [5:4]
	//Binary :  01          01           01 
	//----------------------------------------------------------    
    //rGPGCON = rGPGCON & ~(0x3f) | (1<<4)|(1<<2)|(1); // output setting
   
   	//-------------------------------------------------------------------------------
	//PORT E GROUP
	//Ports  :   GPE4  			GPE3           GPE2         GPE1           GPE0 
	//Signal :   I2S DO         I2S DI         CDCLK        I2S CLK        I2S LRCLK
	//Binary :   10,            10,            10,          10,            10  
	//-------------------------------------------------------------------------------
	rGPECON = rGPECON & ~(0x3ff) | 0x2aa; // IIS function setting
	rGPEUDP = 0xffffffff; // pull_up_down disable
	rGPHCON=rGPHCON&~(0xf<<26)|(0xa<<26);
	rMISCCR = ((rMISCCR  & ~(0x07 << 4)) | (0x04 << 4));	
	
	
#if 0
   rGPECON=rGPECON& ~(3<<28) | (2<<28);// IICSCL setting
   rGPECON=rGPECON& ~(3<<30) | (2<<30);// IICSDA setting   
	printf("\n rGPECON(0x56000040) = 0x%x\n ", rGPECON);   
   rGPEDAT=rGPEDAT & ~(1<<14) ;//disable IICSCL 
   rGPEDAT=rGPEDAT & ~(1<<15) ;//disable IICSDA 
	printf("\n rGPEDAT(0x56000044) = 0x%x\n ", rGPEDAT);      
#endif
}


void IIS_Port_Return(void)
{
	rGPGCON = save_GPG_CON;
   	rGPECON = save_GPE_CON;
}


struct WM8960_REG WM8960_Speak_InitValue[]= {

	{((WM8960_APOP1  	<<1)+0x0), 0x94},		//Enable POBCTRL, SOFT_ST and BUFDCOPEN
    {((WM8960_APOP2  	<<1)+0x0), 0x40},		//Enable DISOP
	{WM8960_DELAY, 0x19},						//Delay (400mS) to remove any residual charge on HP output.    
	{((WM8960_POWER2 	<<1)+0x0), 0x7f},		//Enable LOUT1 and ROUT1
	{((WM8960_APOP2  	<<1)+0x0), 0x00},		//Enable DISOP
	{((WM8960_POWER1 	<<1)+0x0), 0xc0},		//Enable VMID SEL = 2x50K Ohm Dividere
	{WM8960_DELAY, 0x64},				//Delay (50mSeconds) to allow HP amps to settle
	{((WM8960_APOP1  	<<1)+0x0), 0x01},		//Enable POBCTRL, SOFT_ST and BUFDCOPEN
	{((WM8960_POWER2 	<<1)+0x1), 0xf8},		//Enable LOUT1 and ROUT1
	{((WM8960_ADDCTL1	<<1)+0x1), 0xD1},		//
	{((WM8960_POWER3  	<<1)+0x0), 0x0C},		//Enable left and right output mixers
	{((WM8960_LOUTMIX1	<<1)+0x1), 0x50},		//Enable Left DAC to left mixer (LINPUT3 to Output Mixer)
	{((WM8960_ROUTMIX2	<<1)+0x1), 0x50},		//Enable Right DAC to right mixer    
	{((WM8960_ADDCTL4	<<1)+0x0), 0x0A},		// JD2 used for Jack Detect Input
	{((WM8960_ADDCTL2	<<1)+0x0), 0x40},		// HPDETECT HIGH = Speaker 
	{((WM8960_ADDCTL3	<<1)+0x0), 0x08},		// HPDETECT HIGH = Speaker 

//PLL 12.288MHz	 SysCLK 12.288MHz	F2 98.304 MHz
	//CLKSEL = 0
	//MS = 0
	//WL = 10
	//SYSCLKDIV =0
	//ADCDIV=000
	//DACDIV=000
	
	{((WM8960_CLOCK1  	<<1)+0x0), 0x00}, // DACDIV, CLKSEL
	{((WM8960_IFACE1  	<<1)+0x0), 0x00},

	{((WM8960_LOUT1   	<<1)+0x1), 0x79},		//LOUT1VOL (HP) = -20dB
	{((WM8960_ROUT1   	<<1)+0x1), 0x79},		//ROUT1VOL (HP) = -20dB, Enable OUT1VU, load volume settings to both left and right channels
	{((WM8960_LOUT2   	<<1)+0x1), 0x79},		//LOUT2VOL (HP) = -20dB
	{((WM8960_ROUT2   	<<1)+0x1), 0x79},		//ROUT2VOL (HP) = -20dB, Enable OUT2VU, load volume settings to both left and right channels
	{((WM8960_CLASSD3 	<<1)+0x0), 0xcc},		//Left and Right Speakers Enabled	
	{((WM8960_IFACE1  	<<1)+0x0), 0x02},		//
	{((WM8960_CLASSD1 	<<1)+0x0), 0xf7},		//Left and Right Speakers Enabled	
	{((WM8960_DACCTL1 	<<1)+0x0), 0x04}		//DAC Digital Soft Mute = Unmute (Delay from R25 = 080 to unmute >250mS)
	
};
struct WM8960_REG WM8960_Power_Up[]= {

	/* WM8960_HP_DAC_Power_Up_Sequence_GBR_400R */
   /* DAC to HP Full Power Up Sequence, from COLD start */
//   {((29<<1)+0x0), 0x40},		//Enable DISOP
//   {0xff,0x19},					//Delay (400mS) to remove any residual charge on HP output.    
   {((28<<1)+0x0), 0x94},		//Enable POBCTRL, SOFT_ST and BUFDCOPEN
   {((26<<1)+0x0), 0x60},		//Enable LOUT1 and ROUT1
   {0xff,0x32},					//Delay (50mSeconds) to allow HP amps to settle

   {((28<<1)+0x0), 0x00},		//Disable DLOUT1 DROUT1 and HPSTBY. BUFDCOPEN, POBCTRL, SOFT_ST remain enabled  
   {((25<<1)+0x0), 0x80},		//Enable VMID SEL = 2x50K Ohm Dividere
   {WM8960_DELAY,0x64},					//Delay (100mSeconds) to allow VMID to initially charge
   {((25<<1)+0x0), 0xC0},		//Enable VREF. VMID SEL = 2x50K Ohm Divider remain.
   {((28<<1)+0x0), 0x10},		//Disable POBCTRL and SOFT_ST. BUFDCOPEN remain enabled  (Delay from R25 = 080 to Disable POBCTRL and SOFT_ST >100mS)
   {((26<<1)+0x1), 0xE0},		//Enable DACL, DACR. LOUT1 and ROUT1 remain enabled 
   {((47<<1)+0x0), 0x0C},		//Enable left and right output mixers
   {((34<<1)+0x1), 0x50},		//Enable Left DAC to left mixer  18dB
   {((37<<1)+0x1), 0x50},		//Enable Right DAC to right mixer
   {((02<<1)+0x0), 0x7f},		//LOUT1VOL (HP) = -20dB
   {((03<<1)+0x1), 0x7f},		//ROUT1VOL (HP) = -20dB, Enable OUT1VU, load volume settings to both left and right channels

   {((05<<1)+0x0), 0x04}		//DAC Digital Soft Mute = Unmute (Delay from R25 = 080 to unmute >250mS)

};
#define WM8960_POWERUP_SIZE (sizeof(WM8960_Power_Up)/sizeof(WM8960_Power_Up[0]))

struct WM8960_REG WM8960_JACK_init[]= {


	//???????? 
	{((WM8960_APOP1  	<<1)+0x0), 0x94},		//Enable POBCTRL, SOFT_ST and BUFDCOPEN
    {((WM8960_APOP2  	<<1)+0x0), 0x40},		//Enable DISOP 
	{WM8960_DELAY, 0x19},						//Delay (400mS) to remove any residual charge on HP output.    
	{((WM8960_POWER2 	<<1)+0x0), 0x61},		//Enable LOUT1 and ROUT1 SPKL , SPKR disable
	{((WM8960_APOP2  	<<1)+0x0), 0x00},		//Enable DISOP
	{((WM8960_POWER1 	<<1)+0x0), 0xc0},		//Enable VMID SEL = 2x50K Ohm Dividere
	{WM8960_DELAY, 0x64},				//Delay (50mSeconds) to allow HP amps to settle
	{((WM8960_APOP1  	<<1)+0x0), 0x01},		//Enable POBCTRL, SOFT_ST and BUFDCOPEN
	{((WM8960_POWER2 	<<1)+0x1), 0xd1},		//Enable LOUT1 and ROUT1
	{((WM8960_ADDCTL1	<<1)+0x1), 0xD1},		//Slow Clock Enable 
	{((WM8960_POWER3  	<<1)+0x0), 0x0C},		//Enable left and right output mixers
	{((WM8960_LOUTMIX1	<<1)+0x1), 0x50},		//Enable Left DAC to left mixer (LINPUT3 to Output Mixer)
	{((WM8960_ROUTMIX2	<<1)+0x1), 0x50},		//Enable Right DAC to right mixer   
	//General purpose input /output
	{((WM8960_IFACE2	<<1)+0x0), (0x01<<6)}, // GPIO pin setting 
	{((WM8960_ADDCTL4	<<1)+0x0), 0x02},		// ADCLRC /GPIO set
	
     //HP jack dectect cap-less mode
	{((WM8960_ADDCTL2	<<1)+0x0), 0x60},		// HPDETECT High =Headphone, switch enable
	{((WM8960_ADDCTL3	<<1)+0x0), 0x0d},		// HPDETECT HIGH = Speaker /ALC Sample Rate 8K    
 
	{((WM8960_LDAC  	<<1)+0x0), 0xfe}, // LDACVOL
	{((WM8960_RDAC  	<<1)+0x0), 0xfe}, // LDACVOL	

	{((WM8960_CLOCK1  	<<1)+0x0), 0x00}, // DACDIV, CLKSEL
	{((WM8960_IFACE1  	<<1)+0x0), 0x00},

	{((WM8960_LOUT1   	<<1)+0x1), 0x7f},		//LOUT1VOL (HP) = -20dB
	{((WM8960_ROUT1   	<<1)+0x1), 0x7f},		//ROUT1VOL (HP) = -20dB, Enable OUT1VU, load volume settings to both left and right channels
	{((WM8960_LOUT2   	<<1)+0x1), 0x7f},		//LOUT2VOL (HP) = -20dB
	{((WM8960_ROUT2   	<<1)+0x1), 0x7f},		//ROUT2VOL (HP) = -20dB, Enable OUT2VU, load volume settings to both left and right channels
	{((WM8960_CLASSD3 	<<1)+0x0), 0xcc},		//Left and Right Speakers Enabled	
	{((WM8960_IFACE1  	<<1)+0x0), 0x02},		//
	{((WM8960_CLASSD1 	<<1)+0x0), 0xf7},		//Left and Right Speakers Enabled	
	{((WM8960_DACCTL1 	<<1)+0x0), 0x04}		//DAC Digital Soft Mute = Unmute (Delay from R25 = 080 to unmute >250mS)

	
};

struct WM8960_REG WM8960_MIC_init[]= {


    {((WM8960_RESET<<1)+0x0), 0x00},	//Reset WM8960                                                                                                  
    {((WM8960_POWER1<<1)+0x0), 0xEA},	//VMID=50K, Enable AINL and MICBIAS                                                                          
    //{((WM8960_POWER2<<1)+0x0), 0x40},	//Enable LOUT1                                                                                               
    {((WM8960_POWER3<<1)+0x0), 0x28},	//Enable left output mixer, Enable Left Input PGA                                                            
    {((32<<1)+0x1), 0x28},	//Not Connect LINPUT2 to non-inverting PGA (LMP2), LMICBOOST = +20dB, Connect left input PGA to Left input mixer 
    {((33<<1)+0x0), 0x00},	//Disconnect RINPUT1 to input PGA (RMN1)                                                                     
    {((00<<1)+0x1), 0x3e},	//Unmute left input PGA (LINMUTE), Left Input PGA Vol = 0dB, Volume Update                                   
    //{((45<<1)+0x0), 0x80},	//Left Input Boost mixer to Left output mixer (LB2L0), Vol = 0dB                                             
    //{((02<<1)+0x1), 0x79},	//LOUT1 Vol = 0dB, volume update enabled                                                                     
    {((48<<1)+0x0), 0x02},	//Microphone Bias Voltage Control = 0.65 * AVDD    
    //ADC control converter 
    {((5<<1)+0x0), 0x00},	//ADC polarity control Not inverted
    {((23<<1)+0x0), 0x00},	//ADC Data Output Select 
    {((21<<1)+0x1), 0xfE},	//LADCVOL volumn control 

	//with ADCLRC must be set  ALRCGPIO set 1 
    {((9<<1)+0x0), 0x60},	//ALRCGPIO GPIO Pin 
  };

#define WM8960_MIC_SIZE (sizeof(WM8960_MIC_init)/sizeof(WM8960_MIC_init[0]))

#define WM8960_JACK_SIZE (sizeof(WM8960_JACK_init)/sizeof(WM8960_JACK_init[0]))

#define WM8960_INIT_SIZE (sizeof(WM8960_Speak_InitValue)/sizeof(WM8960_Speak_InitValue[0]))

void WriteCodecModule( char CodecIICID, char Addr, char Data)
{
	char D[2];
	
	D[0]=Addr;
	D[1]=Data;
	IIC_Write( CodecIICID, D, 2);
	IIC_Wait();
}

void ReadCodecModule( char CodecIICID, char Addr, char *Data)
{
	IIC_Write( CodecIICID, &Addr, 1);
	IIC_Read( CodecIICID, Data, 1);
	IIC_Wait();
}

void InitWM8960(int mode)
{
   int i;
   IIC_open(200000);
   if(mode==PLAY_IIS_SPEAK)
   {
	for(i=0; i<WM8960_INIT_SIZE; i++) 
		WriteCodecModule( 0x34, WM8960_Speak_InitValue[i].addr, WM8960_Speak_InitValue[i].data);   

   }
   else if(mode==PLAY_IIS_HP)
   {
	for(i=0; i<WM8960_JACK_SIZE; i++) 
		WriteCodecModule( 0x34, WM8960_JACK_init[i].addr, WM8960_JACK_init[i].data);   

   }
   else if(mode==RECORD_MIC_IN)
   {
	for(i=0; i<WM8960_MIC_SIZE; i++) 
		WriteCodecModule( 0x34, WM8960_MIC_init[i].addr, WM8960_MIC_init[i].data);   

   }   
   IIC_close();  
}


unsigned int IIS_Select_SamplingRate(void)
{
	int i;
	
	printf("\nSelect ADC/DAC Rate\n");
	printf("0:11.025kHz, 1:22.05kHz, 2:44.1kHz, 3:88.2kHz\n");
	printf("4:8kHz, 5:16kHz, 6:32kHz, 7:48kHz, 8:64kHz, 9:96kHz\n");
	i = GetIntNum();	


	switch(i)
    {
		case 0:
    		return 11025;
		case 1:
    		return 22050;
       	case 2:
			return 44100;
       	case 3:
			return 88200;
       	case 4:
			return 8000;
       	case 5:
			return 16000;
       	case 6:
			return 32000;
       	case 7:
			return 48000;
       	case 8:
			return 64000;
       	case 9:
			return 96000;
       	default:
			return 44100;
	}      
}


// IIS source clock selection
void Select_PCLK(void)
{
	unsigned int pdf = 5; // prescaler division factor
	
	// SYSCON register setting
	rPCLKCON |= (1<<9); // PCLK bus block of i2s enable
	rSCLKCON |= (1<<9); // I2S source clock enable
	rCLKSRC = (rCLKSRC & ~(3<<14)) | (0<<14); // I2S first source clock selection
	//rCLKDIV1 &= ~(0xf<<12)|(0x6<<12);	// I2S clock divider for EPLL.
	// IIS clock register setting
	rIISMOD = (rIISMOD & ~(7<<10)) | (0<<12)|(1<<10); // internal codec clock source, PCLK Master mode
	rIISPSR = (1<<15)|(pdf);  // prescaler enable, prescaler division factor[9:0]
	//In case PCLK = 50 MHz, IIS Codec CLK = 50/(5+1) = 8.34MHz-->22KHz
	printf("\nIIS Master CLK(PCLK) = %4.2f MHz", (float)PCLK/1000000);
	IIS_Codec_CLK = (float)PCLK/(pdf+1);
	printf("\nIIS Codec CLK = %4.2f MHz", IIS_Codec_CLK/1000000);

	IIS_MasterClk_Sel = 0;
}


void Select_EXTCLK(void)
{
	unsigned int pdf = 0; // prescaler division factor
		
	printf("\n Board To Board Test Source Board is OK? \n");
	
	// SYSCON register setting
	rPCLKCON |= (1<<9); // PCLK bus block of i2s enable
	rSCLKCON |= (1<<9); // I2S source clock enable
	rCLKSRC = (rCLKSRC & ~(3<<14)) | (1<<14); // I2S first source clock selection
	rCLKDIV1 &= ~(0xf<<12);	// I2S clock divider for EPLL.
	
	// IIS clock register setting
	rIISMOD = (rIISMOD & ~(7<<10)) | (1<<12)|(1<<10); // Get Codec clock froem extenal codec chip, using codeclki master mode
	rIISPSR = (1<<15)|(pdf);  // prescaler enable, prescaler division factor[9:0]
			
	IIS_MasterClk_Sel == 1;	
}


void Select_EPLL(void)
{
	unsigned int pdf = 10; // prescaler division factor
	
	// SYSCON register setting
	rPCLKCON |= (1<<9); // PCLK bus block of i2s enable
	rSCLKCON |= (1<<9); // I2S source clock enable
	rEPLLCON= (40<<16) | (1<<8) | 1;	// EPLL OUT IS 96Mhz   : (Mdiv<<16) | (Pdiv<<8) | Sdiv;	
	rCLKSRC = (rCLKSRC & ~(3<<14)) | (0<<14); // I2S source clock selection EPLL divided clock
	rCLKSRC = (rCLKSRC & ~(1<<6)) | (1<<6); //   ESYSCLK Soure is divided EPlL clock
	rCLKDIV1 &= ~(0xf<<12);	// I2S clock divider for EPLL.
			
	Delay(100);
	
	rGPHCON = (rGPHCON & ~(3<<28)) | (2<<28);
	rMISCCR = (rMISCCR & ~(7<<8))  | (1<<8);
	printf("Check EPLL OUT!! and  Press Ant Key if You Want Go!!\n");
	getchar();
	
	// IIS clock register setting
	rIISMOD = (rIISMOD & ~(7<<10)) | (0<<12)|(1<<10); // internal codec clock source, EPLL Div Master mode
	rIISPSR = (1<<15)|(pdf);  // prescaler enable, prescaler division factor[9:0]

	IIS_MasterClk_Sel = 2;
}


void Select_EPLL_Ref_CLK(void)
{
	unsigned int pdf = 1; // prescaler division factor	

	rPCLKCON |= (1<<9); // PCLK bus block of i2s enable	
	rSCLKCON |= (1<<9); // I2S source clock enable
	rEPLLCON= (1<<24) | (40<<16) | (1<<8) | 1;	// EPLL Off , EPLL OUT IS 96Mhz   : (Mdiv<<16) | (Pdiv<<8) | Sdiv;	
	rCLKSRC = (rCLKSRC & ~(3<<14)) | (3<<14); // I2S source clock selection EPLL reference clock
	rCLKSRC = (rCLKSRC & ~(1<<6)) | (0<<6); //   ESYSCLK Soure is divided EPlL clock
	rCLKDIV1 &= ~(0xf<<12);	// I2S clock divider for EPLL.
			
	Delay(100);
	
	rGPHCON = (rGPHCON & ~(3<<28)) | (2<<28);
	rMISCCR = (rMISCCR & ~(7<<8))  | (1<<8);
	printf("Check EPLL OUT!! and  Press Ant Key if You Want Go!!\n");
	getchar();
	
	// IIS clock register setting
	rIISMOD = (rIISMOD & ~(7<<10)) | (0<<12)|(1<<10); // internal codec clock source, EPLL Div Master mode
	rIISPSR = (1<<15)|(pdf);  // prescaler enable, prescaler division factor[9:0]

	IIS_MasterClk_Sel == 3;	
}


void Select_IIS_Master_CLK(void)
{
	int sel;
	unsigned short Sampling_Rate;
	
	//printf("\nIIS Master Clock Source => MPLLin\n");
    printf("\nSelect IIS Master Clock Source PCLK\n");
    Select_PCLK();

}


// download wave files
void down_wav(void)
{
	//Non-cacheable area = 0x31000000 ~ 0x33cf0000
    Buf   = (unsigned char *)IIS_BUFFER;
    _temp = Buf;
    
    printf("Download the PCM(no ADPCM) file by DNW serial port(With header)!!\n");
    printf("Download address is %xh\n", Buf);
    while(((unsigned int)_temp - (unsigned int)Buf) < 4)
    {
       //Uart_Printf("\ndata=%d",size);
       Led_Display(0);
       Delay(1500);
       Led_Display(15);
       Delay(1500);
    }
    printf("\ndata=%x",size);
    //Uart_Getch();
      
    size = *(Buf) | *(Buf + 1)<<8 | *(Buf + 2)<<16 | *(Buf + 3)<<24;
    printf("\ndata=%x",size);
    //Uart_Getch();
    
    printf("\nNow, Downloading... [ File Size : %8d(      0)]",size);
    while(((unsigned int)_temp - (unsigned int)Buf) < (size)){
	  printf("\b\b\b\b\b\b\b\b\b%8d)",(unsigned int)_temp - (unsigned int)Buf);
		Delay(5000);
    }
    printf("\b\b\b\b\b\b\b\b\b%8d)]\n",(unsigned int)_temp - (unsigned int)Buf);

    rINTSUBMSK |= BIT_SUB_RXD1;
    rINTMSK  |= BIT_UART1;
    size = *(Buf + 0x2c) | *(Buf + 0x2d)<<8 | *(Buf + 0x2e)<<16 | *(Buf + 0x2f)<<24;
    size = (size>>1)<<1;
    fs   = *(Buf + 0x1c) | *(Buf + 0x1d)<<8 | *(Buf + 0x1e)<<16 | *(Buf + 0x1f)<<24;

    printf("Sample Size = %d\n",size/2);
    printf("Sampling Frequency = %d Hz\n",fs);
    printf("\n[ Now play the wave file .....]\n");
    printf("If you want to mute or no mute push the 'EIN0' key repeatedly\n");
}


//=================================== [ IIS Test codes ] ==================================

void __irq DMA2_Done(void)
{
	rINTSUBMSK |= BIT_SUB_DMA2; 	
	rINTMSK |= (BIT_DMA);
	rSUBSRCPND = BIT_SUB_DMA2;
	ClearPending(BIT_DMA);	

	rIISCON	&= ~(1<<2)|(0<<0);
   rIISCON |= (1<<2)|(1<<0);
	
  	printf("\nTX DMA Done ~~~");

	rINTMSK &= ~(BIT_DMA);
	rINTSUBMSK &= ~BIT_SUB_DMA2; 	
}


void __irq DMA1_Rec_Done(void)
{
	rINTSUBMSK |= BIT_SUB_DMA1; 	
	rINTMSK |= (BIT_DMA);
	rSUBSRCPND = BIT_SUB_DMA1;
	ClearPending(BIT_DMA);	
	
	rIISCON	&= ~(1<<1)|(0<<0);
	rIISCON |= (1<<1)|(1<<0);
	printf("Record DMA Done!\n");

	rINTMSK &= ~(BIT_DMA);
	rINTSUBMSK &= ~BIT_SUB_DMA1; 	

    Rec_Done = 1;
} 


void __irq Play_Rec_Simul(void)
{
	rINTSUBMSK |= BIT_SUB_DMA1; 	
	rINTMSK |= (BIT_DMA);
	rSUBSRCPND = BIT_SUB_DMA1;
	ClearPending(BIT_DMA);	

    rIISCON	&= ~(1<<0);
	rIISCON |= 0x1;
      
	printf("\nR~~~");	

	rINTMSK &= ~(BIT_DMA);
	rINTSUBMSK &= ~BIT_SUB_DMA1; 	
}


void __irq Rx_IIS_Int(void)
{
    rSUBSRCPND = BIT_SUB_RXD1;          //Clear pending bit (Requested)
    rSUBSRCPND;

    ClearPending(BIT_UART1);

    *_temp ++= RdURXH1(); 
}
 

void Play_Iis_speak(void)
{	
	printf("\n Speak Play \n");
        
	IIS_Port_Init(); 
	Select_PCLK();


	InitWM8960(PLAY_IIS_SPEAK);
	IIS_PlayWave_DMA2(rec_buf, 12000);

	// need to add mute function
	
	IIS_Port_Return();
}
void Play_Iis_HP(void)
{	
	printf("\n Headphone Cap-less mode  Play \n");
        
	IIS_Port_Init(); 
	Select_PCLK();

	InitWM8960(PLAY_IIS_HP);
	IIS_PlayWave_DMA2(rec_buf, 12000);

	// need to add mute function
	
	IIS_Port_Return();
}


//Record Sound via MIC-In
void RecordSound_ViaMICIn_Playit(void)
{
	printf("\nRecord Sound via MIC-In and Play it.\n");

   	IIS_Port_Init(); 
	Select_IIS_Master_CLK();
  
	InitWM8960(RECORD_MIC_IN);
	IIS_RecSound_DMA1(RECORD_MIC_IN, 464000);
	
	printf("\nPress Any Key If You Want Play WAV File\n" );
	getchar();
	
	InitWM8960(PLAY_IIS_SPEAK);
	IIS_PlayWave_DMA2(Mic_rec_buf0, 464000);

	// need to add mute function

	IIS_Port_Return();
}


void IIS_RecSound_DMA1(int mode, unsigned int rec_size)
{
	int Exit_Key, iIndex;
	
		

	pISR_DMA = (unsigned)DMA1_Rec_Done;
	ClearPending(BIT_DMA);	
	rSUBSRCPND = BIT_SUB_DMA1;
	
	// No Pause Rx DMA Operatin, Tx disable, Rx enable, Tx DMA disable, Rx DMA enable, I2S disalbe.
 	rIISCON = (0<<11)+(0<<6)+(0<<5) + (1<<4) + (0<<3) +(0<<2) + (1<<1) + 0;	   
	// Using PCLK, Receive Mode, Left Channel is High, I2S Format, LRCLK 384fs, BLK 32fs, BLC 16bit.
	rIISMOD = (0<<10)+(1<<8) + (0<<7) + (0<<5) +  (2<<3) + (0<<1) + (0<<0); 
	
	
	// FIFO flush
	rIISFIC = (1<<7) + 0; // jungil	  
	rIISFIC = (0<<7) + 0; // jungil	  

	printf("\nIISLRCK = %d Hz", (int) IIS_Codec_CLK/384);

	// Reg_Buf Memory init	
	for(iIndex=0;iIndex<rec_size;iIndex+=1)
	{
		*((unsigned int*)(Mic_rec_buf0)+iIndex)=0x0;
	}
	printf("\n Check 0x31000028 Memory Init  \n");


	printf("\n Press any key to start record!\n"); // jungil
	getchar(); 

	// DMA 1 Mask Release. 
	rINTMSK = ~(BIT_DMA);
	rINTSUBMSK = ~(BIT_SUB_DMA1);

	//IIS Rx start
	rIISCON |= 0x1;	 // I2S start

	//--- DMA1 Initialize
	rDISRCC1 = (1<<1) + (1<<0);   
	rDISRC1  = ((unsigned int)0x55000014);    
	rDIDSTC1 = (0<<1) + (0<<0);     
	rDIDST1  = 0x31000028;                            
//	rDCON1   = (1<<31)+(0<<30)+(1<<29)+(0<<28)+(0<<27)+(0<<22)+(2<<20)+(474200/4);
	rDCON1   = (1<<31)+(0<<30)+(1<<29)+(0<<28)+(0<<27)+(0<<22)+(2<<20)+(rec_size/4);	
	rDMAREQSEL1 = (5<<1) + (1<<0); 
	rDMASKTRIG1 = (0<<2) + (1<<1) + (0<<0);          //No-stop[2], DMA1 channel On[1], No-sw trigger[0] 
			 
	if(mode ==RECORD_MIC_IN)
		printf("\n\nAre you ready to record sound via MIC-In on SMDK2443?");

	
	printf("\n Recording............If you want stop, Press 'x' or 'X' key \n");

	while(!getchar())
	{
		Exit_Key=getchar();

		if( (Exit_Key == 'x') | (Exit_Key == 'X')) 
		{
			Rec_Done=1;
			break;
		}
	}

	Rec_Done = 0;

	//IIS Rx stop
	Delay(10);				//For end of H/W Rx
	rIISCON     = 0x0;			//IIS stop
	
	rDMASKTRIG1 = (1<<2);	//DMA2 stop
	rIISFIC    = 0x0;			//For FIFO flush

//	ClearPending(BIT_DMA);	
	rINTSUBMSK |= (BIT_SUB_DMA1);	
	rINTMSK |= (BIT_DMA);

	printf("\nEnd of Record!\n");	
    	
}

void IIS_RecSound_DMA1_Repeat(unsigned char *start_addr, unsigned int rec_size)
{
	unsigned char Exit_Key;

	printf("\nRecording At IISLRCK = %d Hz\n", (int) IIS_Codec_CLK/384);
	printf("\nRecord Buffer 1 == %x", rec_buf1);
	
	//--- DMA1 Initialize
	rDISRCC1 = (1<<1) + (1<<0);   
	rDISRC1  = ((unsigned int)0x55000014);    
	rDIDSTC1 = (0<<1) + (0<<0);     
	rDIDST1  = (int)rec_buf1;                            
//	rDCON1   = (1<<31)+(0<<30)+(1<<29)+(0<<28)+(0<<27)+(0<<22)+(2<<20)+(474200/4);
	rDCON1   = (1<<31)+(0<<30)+(1<<29)+(0<<28)+(0<<27)+(0<<22)+(2<<20)+(464000/4);	
	rDMAREQSEL1 = (5<<1) + (1<<0); 
	rDMASKTRIG1 = (0<<2) + (1<<1) + (0<<0);          //No-stop[2], DMA1 channel On[1], No-sw trigger[0] 

	//IIS Rx start
	//rIISCON |= 0x1;

	while(!Uart_GetKey())
	{
		Exit_Key=getchar();

		if( (Exit_Key == 'x') | (Exit_Key == 'X')) 
		{
			Rec_Done=1;
			break;
		}
	}

	//IIS Rx stop
	Delay(10);				//For end of H/W Rx
	rIISCON     = 0x0;			//IIS stop
	rDMASKTRIG1 = (1<<2);	//DMA1 stop
	rIISFIC    = 0x0;			//For FIFO flush

	printf("\n\nEnd of Record!");		
}


void IIS_PlayWave_DMA2(unsigned char *start_addr, unsigned int play_size)
{
	unsigned char Exit_Key;
	unsigned int iLRCLK=0, iBCLK=0, iBLC=0;
	
	
	pISR_DMA = (unsigned)DMA2_Done;
	ClearPending(BIT_DMA);	
	rSUBSRCPND = BIT_SUB_DMA2;

		
	// this setting for only TX   	
  	rIISCON = (0<<5) + (0<<4) +(1<<2) + (0<<1)+0;	   
  	// using pclk, tx only , left for channel,iis format ,384fs , 32fs ,16bit  	
 	rIISMOD = rIISMOD & ~(0xfff) | (0<<10)|(0<<8)|(0<<7)|(0<<5)|(2<<3)|(0<<1)|(0<<0);

  	
	rIISFIC = 0+(1<<15);	  
    rIISFIC = 0+(0<<15);	  
    	
	printf("\nIISLRCK = %d Hz", (int) IIS_Codec_CLK/384);

	rINTMSK = ~(BIT_DMA);
	rINTSUBMSK = ~(BIT_SUB_DMA2);


	
	printf("If you want to exit, Press the 'x' key.\n");
	printf("Now Play...\n");
	
	printf("If you want to disable IIS tx(PAUSE), Press '0' key\n");
	printf("If you want to enable IIS tx, Press '1' key\n");
	printf("If you want to stop DMA channel(STOP), Press '2' key\n");
	printf("If you want to start DMA channel, Press '3' key\n");
	
	//DMA2 Register Setting 
	rDISRC2  = (int)(start_addr); 
	rDISRCC2 = (0<<1) + (0<<0); 		  
	rDIDST2  = ((unsigned int)0x55000010);			
	rDIDSTC2 = (1<<1) + (1<<0); 
	// 31: ACK 	30: sync pclk  29:curr_tc int setting 28:unit transfer 27:single service 22: reload 20:half word 
	rDCON2   = (1<<31)+(0<<30)+(1<<29)+(0<<28)+(0<<27)+(0<<22)+(2<<20)+(play_size/4);
	// 5:1= 4th bit is the tx iis     0:set = hw zero = sw
	rDMAREQSEL2 = (4<<1)|(1);
	
	rDMASKTRIG2 = (0<<2) + (1<<1) + (0<<0);	    //No-stop, DMA2 channel On, and No-sw trigger 
       
	//IIS Tx Start
	rIISCON |= 0x1;		 //IIS Interface start

	while(1)
	{
		Exit_Key=getchar();
		
    	//->
		//khs.050510
		//	added for I2S bug test
    	if(Exit_Key == '0'){
			rIISCON |= (1<<4);
    		printf("\nPAUSE...\n");
    	}
		if(Exit_Key == '1'){
			rIISCON &= ~(1<<3);
		}
		if(Exit_Key == '2'){
			rDMASKTRIG2 |= (1<<2);
			rIISFIC &= ~(1<<13);	//* tx fifo disable
			while((rIISFIC & 0x0FC0)!=0);	//* wait until tx fifo gets empty
    		printf("\nSTOP...\n");
		}
		if(Exit_Key == '3'){
			rIISFIC |= (1<<13);	//* tx fifo enable
			rDISRC2  = (int)(start_addr);
			rDMASKTRIG2 &= ~(1<<2);
			rDMASKTRIG2 |= (1<<1);
		}
		//<-		

		if( (Exit_Key == 'x') | (Exit_Key == 'X')) 
			break;
		printf("STAT2: 0x%x CURR_TC: 0x%x		DCDST2: 0x%x\n", rDSTAT2&0x300000, rDSTAT2&0xfffff, rDCDST2);
	}
    	
	//IIS Tx Stop
	Delay(10);			 //For end of H/W Tx
	rIISCON	&= ~(1<<0);	    //IIS Interface stop

	rDMASKTRIG2  = (1<<2);	 //DMA2 stop
	rIISFIC = 0x0;	    //For FIFO flush
		
	rINTSUBMSK |= (BIT_SUB_DMA2);
	rINTMSK |= (BIT_DMA);
 	
	printf("\nEnd of Play!\n");
}











void Data_init(void)
{
	int i;
    unsigned int *rec_buf = (unsigned int *)0x31000028; 
	
	for(i=5;i<500000;i++)
	{
//	printf("merong!!!\n");	
		*(rec_buf+i)=i;
	}
		*(rec_buf+0)=0xa5a5a5a5;
		*(rec_buf+1)=0x5a5a5a5a;
		*(rec_buf+2)=0xffff0000;
		*(rec_buf+3)=0xffffffff;
		*(rec_buf+4)=0x0000ffff;

}

void IIS_PlayWave_DMA2_MASTER(unsigned char *start_addr, unsigned int play_size)
{
	unsigned char Exit_Key;
	
	pISR_DMA = (unsigned)DMA2_Done;
	ClearPending(BIT_DMA);	
	rSUBSRCPND = BIT_SUB_DMA2;
	
	// this setting for only TX   	
  	rIISCON = (1<<5) + (0<<4) + (1<<3) + (1<<2) + (0<<1) + (0<<0);	   
 	// 384fs , 32fs, 16BLC
 	rIISMOD = rIISMOD & ~(0x1fff) | (0<<12)|(0<<10)|(0<<8)|(0<<7)|(0<<5)|(2<<3)|(0<<1)|(0<<0);
 
	// FIFO Flush
	rIISFIC = rIISFIC | (1<<15);
	rIISFIC = rIISFIC & (0<<15);
        	
	printf("\nIISLRCK = %d Hz", (int) IIS_Codec_CLK/384);

	// DMA2 MAsk Release
	rINTMSK = ~(BIT_DMA);
	rINTSUBMSK = ~(BIT_SUB_DMA2);

	printf("\nConnect head-phone plug into speaker-out socket on SMDK2411 and Press any key.\n");
	getchar();
	
	printf("If you want to exit, Press the 'x' key.\n");
	printf("Now Play...\n");
	printf("If you want to disable IIS tx(PAUSE), Press '0' key\n");
	printf("If you want to enable IIS tx, Press '1' key\n");
	printf("If you want to stop DMA channel(STOP), Press '2' key\n");
	printf("If you want to start DMA channel, Press '3' key\n");
	
	//DMA2 Register Setting 
	rDISRC2  = (int)(start_addr); 
	rDISRCC2 = (0<<1) + (0<<0); 		  
	rDIDST2  = ((unsigned int)0x55000010);			
	rDIDSTC2 = (1<<1) + (1<<0); 
	// 24bit APB ADDR Fix, 22bit auto Off, 20bit Word Transmit, 0:19 TC
	rDCON2   = (1<<31)+(0<<30)+(1<<29)+(0<<28)+(0<<27)+(1<<24)+(1<<22)+(2<<20)+(5); 
	// 5:1= 4th bit is the tx iis     0:set = hw zero = sw
	rDMAREQSEL2 = (4<<1)+(1<<0);
	rDMASKTRIG2 = (0<<2) + (1<<1) + (0<<0);	    //No-stop, DMA2 channel On, and No-sw trigger 
       
	//IIS Tx Start
	rIISCON |= 0x1;		

	while(!Uart_GetKey())
	{
		Exit_Key=getchar();

    	if(Exit_Key == '0')
    	{
			rIISCON |= (1<<4);
    		printf("\nPAUSE...\n");
    	}
		if(Exit_Key == '1')
		{
			rIISCON &= ~(1<<3);
		}
		if(Exit_Key == '2')
		{
			rDMASKTRIG2 |= (1<<2);
			rIISFIC &= ~(1<<13);	//* tx fifo disable
			while((rIISFIC & 0x0FC0)!=0);	//* wait until tx fifo gets empty
    		printf("\nSTOP...\n");
		}
		if(Exit_Key == '3')
		{
			rIISFIC |= (1<<13);	//* tx fifo enable
			rDISRC2  = (int)(start_addr);
			rDMASKTRIG2 &= ~(1<<2);
			rDMASKTRIG2 |= (1<<1);
		}
		if( (Exit_Key == 'x') | (Exit_Key == 'X')) 
			break;
		printf("STAT2: 0x%x CURR_TC: 0x%x		DCDST2: 0x%x\n", rDSTAT2&0x300000, rDSTAT2&0xfffff, rDCDST2);
	}
    	
	//IIS Tx Stop
	Delay(10);			 //For end of H/W Tx
	rIISCON	&= ~(1<<0);	    //IIS Interface stop

	rDMASKTRIG2  = (1<<2);	 //DMA2 stop
	rIISFIC = 0x0;	    //For FIFO flush

	rINTSUBMSK |= (BIT_SUB_DMA2);
	rINTMSK |= (BIT_DMA);
 
	printf("\nEnd of Play!\n");
}

void IIS_PlayWave_DMA2_SLAVE(unsigned char *start_addr, unsigned int play_size)
{
	unsigned char Exit_Key;
	
	pISR_DMA = (unsigned)DMA2_Done;
	ClearPending(BIT_DMA);	
	rSUBSRCPND = BIT_SUB_DMA2;
	
	// this setting for only TX   	
  	rIISCON = (1<<5) + (0<<4) + (1<<3) + (1<<2) + (0<<1) + (0<<0);	   
 	// 384fs , 32fs, 16BLC
 	rIISMOD = rIISMOD & ~(0x1fff) | (3<<10)|(0<<8)|(0<<7)|(0<<5)|(2<<3)|(0<<1)|(0<<0);
 
	// FIFO Flush
	rIISFIC = rIISFIC | (1<<15);
	rIISFIC = rIISFIC & (0<<15);
        	
	printf("\nIISLRCK = %d Hz", (int) IIS_Codec_CLK/384);

	// DMA2 MAsk Release
	rINTMSK = ~(BIT_DMA);
	rINTSUBMSK = ~(BIT_SUB_DMA2);

	printf("\nConnect head-phone plug into speaker-out socket on SMDK2411 and Press any key.\n");
	getchar();
	
	printf("If you want to exit, Press the 'x' key.\n");
	printf("Now Play...\n");
	printf("If you want to disable IIS tx(PAUSE), Press '0' key\n");
	printf("If you want to enable IIS tx, Press '1' key\n");
	printf("If you want to stop DMA channel(STOP), Press '2' key\n");
	printf("If you want to start DMA channel, Press '3' key\n");
	
	       
	//DMA2 Register Setting 
	rDISRC2  = (int)(start_addr); 
	rDISRCC2 = (0<<1) + (0<<0); 		  
	rDIDST2  = ((unsigned int)0x55000010);			
	rDIDSTC2 = (1<<1) + (1<<0); 
	// 24bit APB ADDR Fix, 22bit auto Off, 20bit Word Transmit, 0:19 TC
	rDCON2   = (1<<31)+(0<<30)+(1<<29)+(0<<28)+(0<<27)+(1<<24)+(1<<22)+(2<<20)+(5000); 
	// 5:1= 4th bit is the tx iis     0:set = hw zero = sw
	rDMAREQSEL2 = (4<<1)+(1<<0);
	rDMASKTRIG2 = (0<<2) + (1<<1) + (0<<0);	    //No-stop, DMA2 channel On, and No-sw trigger 
       
	//IIS Tx Start
	rIISCON |= 0x1;		

	while(!Uart_GetKey())
	{
		Exit_Key=getchar();

    	if(Exit_Key == '0')
    	{
			rIISCON |= (1<<4);
    		printf("\nPAUSE...\n");
    	}
		if(Exit_Key == '1')
		{
			rIISCON &= ~(1<<3);
		}
		if(Exit_Key == '2')
		{
			rDMASKTRIG2 |= (1<<2);
			rIISFIC &= ~(1<<13);	//* tx fifo disable
			while((rIISFIC & 0x0FC0)!=0);	//* wait until tx fifo gets empty
    		printf("\nSTOP...\n");
		}
		if(Exit_Key == '3')
		{
			rIISFIC |= (1<<13);	//* tx fifo enable
			rDISRC2  = (int)(start_addr);
			rDMASKTRIG2 &= ~(1<<2);
			rDMASKTRIG2 |= (1<<1);
		}
		if( (Exit_Key == 'x') | (Exit_Key == 'X')) 
			break;
		printf("STAT2: 0x%x CURR_TC: 0x%x		DCDST2: 0x%x\n", rDSTAT2&0x300000, rDSTAT2&0xfffff, rDCDST2);
	}
    	
	//IIS Tx Stop
	Delay(10);			 //For end of H/W Tx
	rIISCON	&= ~(1<<0);	    //IIS Interface stop

	rDMASKTRIG2  = (1<<2);	 //DMA2 stop
	rIISFIC = 0x0;	    //For FIFO flush

	rINTSUBMSK |= (BIT_SUB_DMA2);
	rINTMSK |= (BIT_DMA);
 
	printf("\nEnd of Play!\n");
}



void IIS_RecSound_DMA1_MASTER(int mode, unsigned int rec_size)
{
	int Exit_Key;
	int iIndex, iFault=0, iGood=0, iLoop=0; 

	pISR_DMA = (unsigned)DMA1_Rec_Done;
	
	ClearPending(BIT_DMA);	
	rSUBSRCPND = BIT_SUB_DMA1;
	
	// No Pause Rx DMA Operatin, Tx disable, Rx enable, Tx DMA disable, Rx DMA enable, I2S disalbe.
 	rIISCON = (0<<5) + (1<<4) + (0<<3) +(0<<2) + (1<<1) + 0;	   
	rIISMOD = rIISMOD & ~(0x1fff) | (0<<12)|(0<<10)|(1<<8) | (0<<7) | (0<<5) | (2<<3) | (0<<1) | (0<<0); //32fs
	
	// FIFO flush
	rIISFIC = rIISFIC | (1<<7);
	rIISFIC = rIISFIC & (0<<7);

	printf("\nIISLRCK = %d Hz", (int) IIS_Codec_CLK/384);

	// Reg_Buf Memory init	
	for(iIndex=0;iIndex<500000;iIndex+=1)
	{
		*((unsigned int*)(0x31000028)+iIndex)=0x0;
	}
	printf("\n Check 0x31000028 Memory Init and Press Ant Key! \n");
	getchar();

	printf("\n Press any key to start record!\n"); 
	getchar(); 

	// DMA 1 Mask Release. 
	rINTMSK = ~(BIT_DMA);
	rINTSUBMSK = ~(BIT_SUB_DMA1);

	//--- DMA1 Initialize
	rDISRCC1 = (1<<1) + (1<<0);   
	rDISRC1  = ((unsigned int)0x55000014);    
	rDIDSTC1 = (0<<1) + (0<<0);     
	rDIDST1  = (int)rec_buf;                            
	// 24bit APB ADDR fixed, 22bit Auto Off, 20bit Word transmit, 0:19 TC
	rDCON1   = (1<<31)+(0<<30)+(1<<29)+(0<<28)+(0<<27)+(1<<24)+(1<<22)+(2<<20)+(5000);	// auto off test
	rDMAREQSEL1 = (5<<1) + (1<<0); 
	rDMASKTRIG1 = (0<<2) + (1<<1) + (0<<0);          //No-stop[2], DMA1 channel On[1], No-sw trigger[0] 

	//IIS Rx start
	rIISCON |= 0x1;	 // I2S start
		 
	if(mode ==1)
		printf("\n\nAre you ready to record sound via MIC-In on SMDK2443?");
	if(mode ==2)
		printf("\n\nAre you ready to record sound via Line-In on SMDK2443?");
	
	printf("\n Recording............If you want stop, Press 'x' or 'X' key \n");

	while(!getchar())
	{
		Exit_Key=getchar();

		if( (Exit_Key == 'x') | (Exit_Key == 'X')) 
		{
			Rec_Done=1;
			break;
		}
	}

	Rec_Done = 0;

	//IIS Rx stop
	Delay(10);				//For end of H/W Rx
	rIISCON     = 0x0;			//IIS stop
	
	rDMASKTRIG1 = (1<<2);	//DMA1 stop
	rIISFIC    = 0x0;			//For FIFO flush

	rINTSUBMSK |= (BIT_SUB_DMA1);	
	rINTMSK |= (BIT_DMA);

	printf("\nEnd of Record!\n");
	
	getchar();
	
 	for(iIndex = 0 ; iIndex < 2000000 ; iIndex = iIndex + 4)
  	{
  		if ( *(unsigned int *)(0x31000028+iIndex)==(iLoop+5 ))
  		{
  			iGood++;
  			iLoop++;
  		}
  		else
  		{
  			iFault++;
  			iLoop++;
  		}
  	}
  	printf("\n\n Result Bad: %d  Good: %d", iFault, iGood );
}

void IIS_PlayWave_DMA2_EXTCLK(unsigned char *start_addr, unsigned int play_size)
{
	unsigned char Exit_Key;
	unsigned int iLRCLK=0, iBCLK=0, iBLC=0;
	
	
	
	pISR_DMA = (unsigned)DMA2_Done;
	ClearPending(BIT_DMA);	
	rSUBSRCPND = BIT_SUB_DMA2;

//	printf("\n Root Frequency Sample ( 0:256fs    1:512fs    2:384fs    3:768fs ) = ");
// 	iLRCLK = GetIntNum(); 
//	printf("\n Bit Frequency Sample (0:32fs    1:48fs    2:16fs    3:24fs) = ");
//	iBCLK = GetIntNum(); 
//	printf("\n BLC (0: 16bit    1:8bit ) = ");
//	iBLC = GetIntNum(); 
		
	// this setting for only TX   	
  	rIISCON = (0<<5) + (0<<4) +(1<<2) + (0<<1)+0;	   
  	// using pclk, tx only , left for channel,iis format ,384fs , 32fs ,16bit  	
//   	rIISMOD = rIISMOD & ~(0xfff) | (0<<10)|(0<<8)|(0<<7)|(0<<5)|(2<<3)|(0<<1)|(0<<0);
   	rIISMOD = rIISMOD & ~(0x1fff) | (1<<12)|(1<<10)|(0<<8)|(0<<7)|(0<<5)|(2<<3)|(0<<1)|(0<<0);  	
//  	rIISMOD = rIISMOD & ~(0xfff) | (1<<10)|(0<<8)|(0<<7)|(0<<5)|(iLRCLK<<3)|(iBCLK<<1)|(iBLC<<0);  	// EPLL using.
  	
	rIISFIC = 0+(1<<15);	  
    rIISFIC = 0+(0<<15);	  
    	
	printf("\nIISLRCK = %d Hz", (int) IIS_Codec_CLK/384);

	rINTMSK = ~(BIT_DMA);
	rINTSUBMSK = ~(BIT_SUB_DMA2);

	printf("\nConnect head-phone plug into speaker-out socket on SMDK2443 and Press any key.\n");
	getchar();
	
	printf("If you want to exit, Press the 'x' key.\n");
	printf("Now Play...\n");
	
	printf("If you want to disable IIS tx(PAUSE), Press '0' key\n");
	printf("If you want to enable IIS tx, Press '1' key\n");
	printf("If you want to stop DMA channel(STOP), Press '2' key\n");
	printf("If you want to start DMA channel, Press '3' key\n");
	
	//DMA2 Register Setting 
	rDISRC2  = (int)(start_addr); 
	rDISRCC2 = (0<<1) + (0<<0); 		  
	rDIDST2  = ((unsigned int)0x55000010);			
	rDIDSTC2 = (1<<1) + (1<<0); 
	// 31: ACK 	30: sync pclk  29:curr_tc int setting 28:unit transfer 27:single service 22: reload 20:half word 
	rDCON2   = (1<<31)+(0<<30)+(1<<29)+(0<<28)+(0<<27)+(0<<22)+(2<<20)+(play_size/4);
	// 5:1= 4th bit is the tx iis     0:set = hw zero = sw
	rDMAREQSEL2 = (4<<1)|(1);
	
	rDMASKTRIG2 = (0<<2) + (1<<1) + (0<<0);	    //No-stop, DMA2 channel On, and No-sw trigger 
       
	//IIS Tx Start
	rIISCON |= 0x1;		 //IIS Interface start

	while(1)
	{
		Exit_Key=getchar();
    	if(Exit_Key == '0'){
			rIISCON |= (1<<4);
    		printf("\nPAUSE...\n");
    	}
		if(Exit_Key == '1'){
			rIISCON &= ~(1<<3);
		}
		if(Exit_Key == '2'){
			rDMASKTRIG2 |= (1<<2);
			rIISFIC &= ~(1<<13);	//* tx fifo disable
			while((rIISFIC & 0x0FC0)!=0);	//* wait until tx fifo gets empty
    		printf("\nSTOP...\n");
		}
		if(Exit_Key == '3'){
			rIISFIC |= (1<<13);	//* tx fifo enable
			rDISRC2  = (int)(start_addr);
			rDMASKTRIG2 &= ~(1<<2);
			rDMASKTRIG2 |= (1<<1);
		}
		//<-		

		if( (Exit_Key == 'x') | (Exit_Key == 'X')) 
			break;
		printf("STAT2: 0x%x CURR_TC: 0x%x		DCDST2: 0x%x\n", rDSTAT2&0x300000, rDSTAT2&0xfffff, rDCDST2);
	}
    	
	//IIS Tx Stop
	Delay(10);			 //For end of H/W Tx
	rIISCON	&= ~(1<<0);	    //IIS Interface stop

	rDMASKTRIG2  = (1<<2);	 //DMA2 stop
	rIISFIC = 0x0;	    //For FIFO flush
		
	rINTSUBMSK |= (BIT_SUB_DMA2);
	rINTMSK |= (BIT_DMA);
 	
	printf("\nEnd of Play!\n");
}
