/*********************************************************************
;* Project Name : s3c2443x
;*
;* Copyright 2006 by Samsung Electronics, Inc.
;* All rights reserved.
;*
;* Project Description :
;* This software is only for verifying functions of the s3c2443x
;* Anybody can use this code without our permission.
;**********************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "Option.h"
#include "System.h"
#include "Console.h"
#include "Exception.h"
#include "MMUCache.h"
#include "2443addr.h"
#include "PLL.h"

#include "Display.h"
#include "DisplaySample.h"
#include "DisplaySample_240_320.h"
#include "DisplaySample_320_240.h"
#include "FONT.h"

uint32 debug_print=0;
uint8 debug_continue=TRUE;
void (*PutPixel)(uint32 x,uint32 y,uint32 c);


void SetLcdPort(void)
{
	// set gpio for lcd
	rGPCCON = 0xaaaaaaaa;	// CTRL,  VD[7:0]
	rGPDCON = 0xaaaaaaaa;	// VD[23:8]

#if 0	// by edchoi
	#if 1 // tempory
	rGPBCON = (rGPBCON & ~(3<<8)) | (1<<8); // Backlight Pwm control
	rGPBDAT  |= (1<<8);
	#else // control the backlight using pwm timer output
	rGPBCON = (rGPBCON & ~(3<<8)) | (2<<8); // set Timer out
	#endif

	// LCD _nRESET control
	rGPBCON = (rGPBCON & ~(3<<2)) |(1<<2);
	rGPBDAT &= ~(1<<1);
	Delay(100);
	rGPBDAT |= (1<<1);
	Delay(100);
#else
	rGPGCON &= ~(0x3 << 8);
	rGPGCON |= (0x1 << 8);
	rGPGUDP &= ~(0x3 << 8);
	rGPGUDP |= (0x3 << 8);
	rGPGDAT |= (0x1 << 4);
#endif
}

void delayLoop(uint32 count) 
{ 
    uint32 j; 
    for(j = 0; j < count; j++)  ; 
}


//===================================================
// functions related to LCDC general settings
//
void LCDC_Common_Init(void)
{
	uint8 clkval=0;
	uint16 hcync_cnt,vclk_cnt;

	// by edchoi rVIDCON0 = VIDCON0_S_RGB_IF|VIDCON0_S_RGB_PAR|VIDCON0_S_VCLK_GATING_OFF|VIDCON0_S_CLKDIR_DIVIDED|VIDCON0_S_CLKSEL_HCLK;
	rVIDCON0 = VIDCON0_S_RGB_IF|VIDCON0_S_BGR_PAR|VIDCON0_S_VCLK_GATING_OFF|VIDCON0_S_CLKDIR_DIVIDED|VIDCON0_S_CLKSEL_HCLK;
	//rVIDCON0 = VIDCON0_S_RGB_IF|VIDCON0_S_BGR_PAR|VIDCON0_S_VCLK_GATING_OFF|VIDCON0_S_CLKDIR_DIVIDED|VIDCON0_S_CLKSEL_HCLK;
	//rVIDCON0 = VIDCON0_S_RGB_IF|VIDCON0_S_RGB_PAR|VIDCON0_S_VCLK_GATING_OFF|VIDCON0_S_CLKDIR_DIVIDED|VIDCON0_S_CLKSEL_UPLL;
	
	hcync_cnt = (lcd_vbpd+lcd_vfpd+lcd_vspw+lcd_line_value);
	vclk_cnt = (lcd_hbpd+lcd_hfpd+lcd_hspw+lcd_horizon_value);
	
	if(rVIDCON0 & (1<<2))	//ePLL
	{
		clkval = (uint8)(((float)96000000/(float)(hcync_cnt*vclk_cnt*lcd_frame_rate*2))+0.5)-1;
		rEPLLCON = (40<<16)|(1<<8)|(1<<0); // M=40;P=1, S=1);
		rEPLLCON &= ~(1<<24); // epll on
	}
	else	//HCLK
	{
		clkval = (uint8)(((float)HCLK/(float)(hcync_cnt*vclk_cnt*lcd_frame_rate*2))+0.5)-1;
	}
	
	rVIDCON0 |= (clkval <<VIDCON0_CLKVAL_F_SHIFT);

	//check point
	rVIDCON1 = VIDCON1_S_HSYNC_INVERTED|VIDCON1_S_VSYNC_INVERTED;
	
	rVIDTCON0=((lcd_vbpd-1)<<VIDTCON0_BPD_S)|((lcd_vfpd-1)<<VIDTCON0_FPD_S)|(lcd_vspw-1);
	rVIDTCON1=((lcd_hbpd-1)<<VIDTCON0_BPD_S)|((lcd_hfpd-1)<<VIDTCON0_FPD_S)|(lcd_hspw-1);
	rVIDTCON2 = ((lcd_line_value-1)<<VIDTCON2_LINEVAL_S)|(lcd_horizon_value-1);
}

void Basic_Display_Setting( int32 win_num,int32 buf_num, int32 bpp, uint32 width, uint32 height)
{
	uint32 pagewidth_in_byte=0,offsize_in_byte=0;
	uint32 pagewidth,offsize;

	lcd_bpp = bpp;
	
	screen_width=width;
	screen_height=height;

	pagewidth=lcd_horizon_value;
	offsize = screen_width - lcd_horizon_value;

	switch(bpp)
	{
		case WINCONx_1BPP_PALLET:
			PutPixel=_PutPixel1Bit;
			break;
		case WINCONx_2BPP_PALLET:
			PutPixel=_PutPixel2Bit;
			break;
		case WINCONx_4BPP_PALLET:
			PutPixel=_PutPixel4Bit;
			break;
		case WINCONx_8BPP_PALLET:
		case WINCONx_8BPP_NO_PALLET:
			PutPixel=_PutPixel8Bit;
			break;
		case WINCONx_16BPP_565:
		case WINCONx_16BPP_A555:
		case WINCONx_16BPP_1555:
			PutPixel=_PutPixel16Bit;
			break;
		case WINCONx_18BPP_666:
		case WINCONx_18BPP_A665:
		case WINCONx_19BPP_A666:
		case WINCONx_24BPP_888:
		case WINCONx_24BPP_A887:
		case WINCONx_25BPP_A888:
			PutPixel=_PutPixel32Bit;
			break;
			
	}

	// 2006.06.02
	// Frame buffer address setting : Windows0 have the two buffer like as buffer0, buffer1. 
	// Each buffer have the register for setting the start and end address. But Window1 have one buffer
	// base lcd_framebuffer = Noncache address(0x31000000)

	pagewidth_in_byte = pagewidth/8*lcd_bit_order[bpp];
	if((pagewidth_in_byte%lcd_burst_in_byte[lcd_burst_mode])!=0)
	{
		pagewidth_in_byte += lcd_burst_in_byte[lcd_burst_mode] - (pagewidth_in_byte%lcd_burst_in_byte[lcd_burst_mode]);
	}		
	offsize_in_byte = offsize/8*lcd_bit_order[bpp];

	LCDC_Common_Init();

	*WINCONx_Reg_Addr[win_num] = (buf_num<<WINCON_BUFSEL)|(0<<WINCON_BUFAUTOEN)|(0<<WINCON_SWAP_S)|(lcd_burst_mode<<WINCON_BURSTLEN_S)|(bpp<<WINCON_BPP_S); // 4word burst, 16bpp, 
	*VIDOSDxA_Reg_Addr[win_num] = (0<<VIDOSDxAB_HORIZON_X_S)|(0);
	*VIDOSDxB_Reg_Addr[win_num] = ((lcd_horizon_value-1)<<VIDOSDxAB_HORIZON_X_S)|(lcd_line_value-1);	

	lcd_framebuffer = (uint32 *)lcd_frame_buffer[win_num][buf_num];

	// buffer start address
	*VIDWxADD0_Reg_Addr[win_num][buf_num] = (uint32)lcd_framebuffer;		
	// buffer end address
	*VIDWxADD1_Reg_Addr[win_num][buf_num] = (uint32)lcd_framebuffer + (pagewidth_in_byte+offsize_in_byte)*(lcd_line_value);
	// buffer size 
	*VIDWxADD2_Reg_Addr[win_num][buf_num] = (offsize_in_byte<<VIDWxADD2_OFFSET_SIZE_S)|pagewidth_in_byte;

	*WINxMAP_Reg_Addr[win_num] = 0;

	if (win_num>0)
	{
		*VIDOSDxC_Reg_Addr[win_num] = 0;
		
		*WxKEYCON0_Reg_Addr[win_num] = 0;
		*WxKEYCON1_Reg_Addr[win_num] = 0;
	
	}
}

void Display_Start(int8 win_num)
{	
	LcdWindowOnOff(win_num,LCD_ON);
	LcdEnvidOnOff(LCD_ON);

	return;
}

void Display_End(int8 win_num)
{	
	printf("\nIf you want to end this test, press any key\n");
	
	while(!(Uart_getc()));

	LcdWindowOnOff(win_num,LCD_OFF);
	LcdEnvidOnOff(LCD_OFF);	

	return;
}


void Prepare_Image(int8 win_num, int8 bpp)
{
	uint32 i, j;
	uint32 *framebuffer,data;
	
	framebuffer = lcd_framebuffer;

	if(bpp<=WINCONx_8BPP_PALLET)
		Prepare_Pallet(win_num,bpp);

	switch(bpp)
	{
		case WINCONx_1BPP_PALLET:
			for(i=0;i<2;i++)
			{
				LCD_FilledRectangle(0, screen_height/2*i, screen_width-1, screen_height/2*(i+1)-1, i);
				lprintf(50,screen_height/2*i,i^0x1,"Color %d",i);
			}
			break;

		case WINCONx_2BPP_PALLET:
			for(i=0;i<4;i++)
			{
				LCD_FilledRectangle(0, screen_height/4*i, screen_width-1, screen_height/4*(i+1)-1, i);
				lprintf(50,screen_height/4*i,i^0x3,"Color %d",i);
			}
			break;

		case WINCONx_4BPP_PALLET:
			for(i=0;i<16;i++)
			{
				LCD_FilledRectangle(0, screen_height/16*i, screen_width-1, screen_height/16*(i+1)-1, i);
				lprintf(50,screen_height/16*i,i^0xf,"Color %d",i);
			}
			break;

		case WINCONx_8BPP_PALLET:
			if(lcd_type == LCD_MODULE_LTS222)
			{
				for(i=0; i<(screen_width*screen_height*lcd_bit_order[bpp]/32); i++) 
				{
					data =(DEMO256[i*4]<<24);
					data |= (DEMO256[i*4+1] <<16);
					data |= (DEMO256[i*4+2] <<8);
					data |= DEMO256[i*4+3];
					*framebuffer++ = data;
				}
				lprintf(0,0,0xff,"8BPP paletteized mode.");
			}
			else
				printf("This test for the LCD type is not prepared\n");				
			break;

		case WINCONx_8BPP_NO_PALLET:
			if (win_num==0)
				printf("This mode is not supported in window 0.\n");
			else
			{
				for(i=0;i<screen_width;i++)
					for(j=0;j<screen_height;j++)
						PutPixel(i,j,(i*j)&0xff);
						
				lprintf(0,0,0xff,"8BPP(A232) Non-paletteized mode.");
			}
			break;

		case WINCONx_16BPP_565:
			for(i=0;i<screen_width;i++)
				for(j=0;j<screen_height;j++)
					PutPixel(i,j,(i*j)&0xffff);
			lprintf(0,0,0xffff,"16BPP(565) Non-paletteized mode.");
			break;
			
		case WINCONx_16BPP_A555:
			if (win_num==0)
				printf("This mode is not supported in window 0.\n");
			else
			{
				for(i=0;i<screen_width;i++)
					for(j=0;j<screen_height;j++)
						PutPixel(i,j,(i*j)&0xffff);
				lprintf(0,0,0xffff,"16BPP(A555) Non-paletteized mode.");
			}
			break;
			
		case WINCONx_16BPP_1555:
			for(i=0;i<screen_width;i++)
				for(j=0;j<screen_height;j++)
					PutPixel(i,j,(i*j)&0xffff);
			lprintf(0,0,0xffff,"16BPP(1555) Non-paletteized mode.");
			break;
			
		case WINCONx_18BPP_666:
			for(i=0;i<screen_width;i++)
				for(j=0;j<screen_height;j++)
					PutPixel(i,j,(i*j)&0x3ffff);
			lprintf(0,0,0x3ffff,"18BPP(666) Non-paletteized mode.");
			break;
			
		case WINCONx_18BPP_A665:
			if (win_num==0)
				printf("This mode is not supported in window 0.\n");
			else
			{
				for(i=0;i<screen_width;i++)
					for(j=0;j<screen_height;j++)
						PutPixel(i,j,(i*j)&0x3ffff);
				lprintf(0,0,0x3ffff,"18BPP(A665) Non-paletteized mode.");
			}
			break;
			
		case WINCONx_19BPP_A666:
			if (win_num==0)
				printf("This mode is not supported in window 0.\n");
			else
			{
				for(i=0;i<screen_width;i++)
					for(j=0;j<screen_height;j++)
						PutPixel(i,j,(i*j)&0x7ffff);
				lprintf(0,0,0x7ffff,"19BPP(A666) Non-paletteized mode.");
			}
			break;
			
		case WINCONx_24BPP_888:
			for(i=0;i<screen_width;i++)
				for(j=0;j<screen_height;j++)
					PutPixel(i,j,(i*j)&0xffffff);
			lprintf(0,0,0xffffff,"24BPP(888) Non-paletteized mode.");
			break;
			
		case WINCONx_24BPP_A887:
			if (win_num==0)
				printf("This mode is not supported in window 0.\n");
			else
			{
				for(i=0;i<screen_width;i++)
					for(j=0;j<screen_height;j++)
						PutPixel(i,j,(i*j)&0xffffff);
				lprintf(0,0,0xffffff,"24BPP(A887) Non-paletteized mode.");
			}
			break;
			
		case WINCONx_25BPP_A888:
			if (win_num==0)
				printf("This mode is not supported in window 0.\n");
			else
			{
				for(i=0;i<screen_width;i++)
					for(j=0;j<screen_height;j++)
						PutPixel(i,j,(i*j)&0x1ffffff);
				lprintf(0,0,0x1ffffff,"25BPP(A888) Non-paletteized mode.");
			}
			break;

		default:
			break;
	}
	
}

void Prepare_Pallet(int8 win_num, int8 bpp)
{
	uint32 *pallet_ptr, i;
				
	//pallet format => 16bit(565)
	rWPALCON = (6<<3)|(6<<0);

	//  palette can be updated
	rWPALCON |= (1<<9);	
	
	pallet_ptr =(uint32 *)WINxPalletBase[win_num];

	switch(bpp)
	{
		case WINCONx_1BPP_PALLET:
			*(pallet_ptr+0) = 0x0000;
			*(pallet_ptr+1) = 0xffff;
			break;

		case WINCONx_2BPP_PALLET:
			*(pallet_ptr+0) = 0x0000;
			*(pallet_ptr+1) = 0xf800;
			*(pallet_ptr+2) = 0x07e0;
			*(pallet_ptr+3) = 0x001f;
			break;

		case WINCONx_4BPP_PALLET:
			*(pallet_ptr+0)	= 0x0000;
			*(pallet_ptr+1) = 0xf800;
			*(pallet_ptr+2) = 0x07e0;
			*(pallet_ptr+3) = 0x001f;
			*(pallet_ptr+4) = 0xf81f;
			*(pallet_ptr+5) = 0xffe0;
			*(pallet_ptr+6) = 0x07ff;
			*(pallet_ptr+7) = 0xffff;
			*(pallet_ptr+8) = 0x18e3;
			*(pallet_ptr+9) = 0x7800;
			*(pallet_ptr+10) = 0x03e0;
			*(pallet_ptr+11) = 0x000f;
			*(pallet_ptr+12) = 0x780f;
			*(pallet_ptr+13) = 0x7be0;
			*(pallet_ptr+14) = 0x03ef;
			*(pallet_ptr+15) = 0x7bef;
			break;

		case WINCONx_8BPP_PALLET:
			for(i=0;i<256;i++)
				*(pallet_ptr+i) = (uint32)DEMO256pal[i];
			break;
			
		default :
			break;
	}
	
	rWPALCON &= ~(1<<9);
}

void LcdEnvidOnOff(uint8 onoff)
{
	if(onoff==LCD_ON)
		rVIDCON0 |= (3); // ENVID On using Per Frame method
	else 
		rVIDCON0 &= (~3); // ENVID Off using Per Frame method
}    

void LcdWindowOnOff(uint8 num, uint8 onoff)
{
	switch(num)
	{
		case LCD_WIN_0:
			if(onoff==LCD_ON)
				rWINCON0 |= 0x01;
			else
				rWINCON0 &= ~0x01;
			break;
					
		case LCD_WIN_1:
			if(onoff==LCD_ON)
				rWINCON1 |= 0x01;
			else
				rWINCON1 &= ~0x01;
			break;
			
		case LCD_WIN_ALL:
			if(onoff==LCD_ON)
			{
				rWINCON0 |= 0x01;
				rWINCON1 |= 0x01;
			}
			else
			{
				rWINCON0 &= ~0x01;
				rWINCON1 &= ~0x01;
			}
		
		default:
			break;
	}
	return;
}
//
//===================================================

//===================================================
//
//
void _PutPixel1Bit(uint32 x,uint32 y,uint32 c)
{
	if(x<screen_width && y<screen_height)
		*(uint32 *)(lcd_framebuffer+y*screen_width/32+x/32)= (*(uint32 *)(lcd_framebuffer+y*screen_width/32+x/32) 
		& ~(0x80000000>>(x%32)*1))| ((c&0x00000001)<<((32-1-(x%32))*1));   
}

void _PutPixel2Bit(uint32 x,uint32 y,uint32 c)
{
	if(x<screen_width && y<screen_height)
		*(uint32 *)(lcd_framebuffer+y*screen_width/16+x/16)= (*(uint32 *)(lcd_framebuffer+y*screen_width/16+x/16) 
		& ~(0xc0000000>>(x%16)*2))| ((c&0x00000003)<<((16-1-(x%16))*2));
}

void _PutPixel4Bit(uint32 x,uint32 y,uint32 c)
{
	if(x<screen_width && y<screen_height)
		*(uint32 *)(lcd_framebuffer+y*screen_width/8+x/8)= (*(uint32 *)(lcd_framebuffer+y*screen_width/8+x/8) 
		& ~(0xf0000000>>(x%8)*4))| ((c&0x0000000f)<<((8-1-(x%8))*4));
}

void _PutPixel8Bit(uint32 x,uint32 y,uint32 c)
{
	if(x<screen_width && y<screen_height)
		*(uint32 *)(lcd_framebuffer+y*screen_width/4+x/4)= (*(uint32 *)(lcd_framebuffer+y*screen_width/4+x/4) 
		& ~(0xff000000>>(x%4)*8))| ((c&0x000000ff)<<((4-1-(x%4))*8));
}

void _PutPixel16Bit(uint32 x,uint32 y,uint32 c)
{
	if(x<screen_width && y<screen_height)
		*(uint32 *)(lcd_framebuffer+y*screen_width/2+x/2)= (*(uint32 *)(lcd_framebuffer+y*screen_width/2+x/2) 
		& ~(0xffff0000>>(x%2)*16))|((c&0x0000ffff)<<((2-1-(x%2))*16));
}

void _PutPixel32Bit(uint32 x,uint32 y,uint32 c)
{
	if(x<screen_width && y<screen_height)
		*(uint32 *)(lcd_framebuffer+y*screen_width+x)=c;
}



void LCD_Rectangle(int32 x1,int32 y1,int32 x2,int32 y2,int32 color)
{
    LCD_Line(x1,y1,x2,y1,color);
    LCD_Line(x2,y1,x2,y2,color);
    LCD_Line(x1,y2,x2,y2,color);
    LCD_Line(x1,y1,x1,y2,color);
}


void LCD_FilledRectangle(int32 x1,int32 y1,int32 x2,int32 y2,int32 color)
{
    int32 i;

    for(i=y1;i<=y2;i++)
	LCD_Line(x1,i,x2,i,color);
}



// LCD display is flipped vertically
// But, think the algorithm by mathematics point.
//   3I2
//   4 I 1
//  --+--   <-8 octants  mathematical cordinate
//   5 I 8
//   6I7
void LCD_Line(int32 x1,int32 y1,int32 x2,int32 y2,int32 color)
{
	int32 dx,dy,e;
	dx=x2-x1; 
	dy=y2-y1;
    
	if(dx>=0)
	{
		if(dy >= 0) // dy>=0
		{
			if(dx>=dy) // 1/8 octant
			{
				e=dy-dx/2;
				while(x1<=x2)
				{
					PutPixel(x1,y1,color);
					if(e>0){y1+=1;e-=dx;}	
					x1+=1;
					e+=dy;
				}
			}
			else		// 2/8 octant
			{
				e=dx-dy/2;
				while(y1<=y2)
				{
					PutPixel(x1,y1,color);
					if(e>0){x1+=1;e-=dy;}	
					y1+=1;
					e+=dx;
				}
			}
		}
		else		   // dy<0
		{
			dy=-dy;   // dy=abs(dy)

			if(dx>=dy) // 8/8 octant
			{
				e=dy-dx/2;
				while(x1<=x2)
				{
					PutPixel(x1,y1,color);
					if(e>0){y1-=1;e-=dx;}	
					x1+=1;
					e+=dy;
				}
			}
			else		// 7/8 octant
			{
				e=dx-dy/2;
				while(y1>=y2)
				{
					PutPixel(x1,y1,color);
					if(e>0){x1+=1;e-=dy;}	
					y1-=1;
					e+=dx;
				}
			}
		}	
	}
	else //dx<0
	{
		dx=-dx;		//dx=abs(dx)
		if(dy >= 0) // dy>=0
		{
			if(dx>=dy) // 4/8 octant
			{
				e=dy-dx/2;
				while(x1>=x2)
				{
					PutPixel(x1,y1,color);
					if(e>0){y1+=1;e-=dx;}	
					x1-=1;
					e+=dy;
				}
			}
			else		// 3/8 octant
			{
				e=dx-dy/2;
				while(y1<=y2)
				{
					PutPixel(x1,y1,color);
					if(e>0){x1-=1;e-=dy;}	
					y1+=1;
					e+=dx;
				}
			}
		}
		else		   // dy<0
		{
			dy=-dy;   // dy=abs(dy)

			if(dx>=dy) // 5/8 octant
			{
				e=dy-dx/2;
				while(x1>=x2)
				{
					PutPixel(x1,y1,color);
					if(e>0){y1-=1;e-=dx;}	
					x1-=1;
					e+=dy;
				}
			}
			else		// 6/8 octant
			{
				e=dx-dy/2;
				while(y1>=y2)
				{
					PutPixel(x1,y1,color);
					if(e>0){x1-=1;e-=dy;}	
					y1-=1;
					e+=dx;
				}
			}
		}	
	}
}


void LCD_ClearScr(uint32 c)
{	
	int32 i,j;

	for(j=0;j<screen_height;j++)
		for(i=0;i<screen_width;i++)
				PutPixel(i,j,c);      
}


void lputs(uint32 x, uint32 y, uint32 c, char *str)
{
	int i,j,k;
	int oneLine;

	for(i=0;i<strlen(str);i++)
	{
		for(j=0; j<16; j++)
		{
			if ((str[i]>=0x20)&&(str[i]<0x80))
				oneLine=chr_font8X16[str[i]-0x20][j];
			else
				oneLine=0;

			for(k=0;k<8;k++)
			{
				if(oneLine&(1<<k))
					PutPixel(x+7-k,y+j,c);
			}
		}
		x+=8;
	}
}

void lprintf(uint32 x, uint32 y, uint32 c, const char *fmt,...)
{
	va_list ap;
	char string[256];

	va_start(ap,fmt);
	vsprintf(string,fmt,ap);
	lputs(x,y,c,string);
	va_end(ap);
}

//
//===================================================

//===================================================
// functions related to LDI initialization
//
void LB043WQ1_ON(void)
{
	rGPGCON = (rGPGCON & ~(3<<8)) | (1<<8); // LCD_PWRDN Control (GPG4)
	rGPGDAT |= (1<<4);
}

void LB043WQ1_OFF(void)
{
	rGPGCON = (rGPGCON & ~(3<<8)); 			// LCD_PWRDN Control (GPG4)
	rGPGDAT &= ~(1<<4);
}

void LB043WQ1_BACKLIGHT_ON(void)
{
//	rGPBCON = (rGPBCON & ~(3<<6)) | (1<<6);	// Backlight PWM Control (GPB3)
//	rGPBDAT |= (1<<3);

	rGPBCON = (rGPBCON & ~(3<<6)) | (2<<6);	// Backlight PWM Control (TOUT3)
	rTCFG0 = (rTCFG0 & ~(0xf << 8)) | (255 << 8);	// Prescaler 1 Setting..
	rTCFG1 = (rTCFG1 & ~(0xf << 12)) | (3 << 12);	// Divider 1/16 Setting..
	
	rTCNTB3 = 102;
	rTCMPB3 = 101;
	rTCON |= (1<<17);
	rTCON = (rTCON & ~(0x1f << 16)) | (1<<19) | (1 <<16);
}

void LB043WQ1_BACKLIGHT_OFF(void)
{
	rGPBCON = (rGPBCON & ~(3<<6));
	rGPBDAT &= ~(1<<3);
}

void LB043WQ1_BACKLIGHT_Set(S32 cal)
{

	if(cal < 1) 		cal = 1;
	else if(cal > 100)	cal = 100;
	
//	LCD.bright = cal;
	rTCMPB3 = cal;
}


void Init_LDI(void)
{
	U8 clkval;
	U16 hcync_cnt, vclk_cnt;
	U32 pagewidth_in_byte = 0;
	extern  uint32 *lcd_framebuffer;
	extern  uint32 *lcd_framebuffer1;	

	lcd_horizon_value = INNO43_HOZVAL;
	lcd_line_value = INNO43_LINEVAL;
	lcd_vbpd = INNO43_VBPD;
	lcd_vfpd = INNO43_VFPD;
	lcd_vspw = INNO43_VSPW;
	lcd_hbpd = INNO43_HBPD;
	lcd_hfpd = INNO43_HFPD;
	lcd_hspw = INNO43_HSPW;
	lcd_frame_rate = INNO43_FRAME_RATE;



	rMISCCR |= (1<<28);		// TFT LCD Control En
	rGPCCON = 0xaaaaaaaa;	// CTRL, VD[7:0]
	rGPDCON = 0xaaaaaaaa;	// VD[23:8]
	
	LB043WQ1_BACKLIGHT_OFF();
	LB043WQ1_OFF();
	
	hcync_cnt = (LB043WQ1_VBPD + LB043WQ1_VFPD + LB043WQ1_VSPW + LB043WQ1_YSIZE);
	vclk_cnt = (LB043WQ1_HBPD + LB043WQ1_HFPD + LB043WQ1_HSPW + LB043WQ1_XSIZE);
	clkval = (U8)(((float)HCLK / (float)(hcync_cnt * vclk_cnt * LB043WQ1_FRAME_RATE)) + 0.5) - 1;
	rVIDCON0 = (VIDCON0_S_RGB_IF|VIDCON0_S_BGR_PAR|(clkval <<VIDCON0_CLKVAL_F_SHIFT)|VIDCON0_S_VCLK_GATING_OFF|VIDCON0_S_CLKDIR_DIVIDED|VIDCON0_S_CLKSEL_HCLK);
	rVIDCON1 = VIDCON1_S_HSYNC_INVERTED|VIDCON1_S_VSYNC_INVERTED;

	// Timing
	rVIDTCON0=((LB043WQ1_VBPD-1)<<VIDTCON0_BPD_S)|((LB043WQ1_VFPD-1)<<VIDTCON0_FPD_S)|((LB043WQ1_VSPW-1)<<VIDTCON0_SPW_S);
	rVIDTCON1=((LB043WQ1_HBPD-1)<<VIDTCON0_BPD_S)|((LB043WQ1_HFPD-1)<<VIDTCON0_FPD_S)|((LB043WQ1_HSPW-1)<<VIDTCON0_SPW_S);
	rVIDTCON2 = ((LB043WQ1_YSIZE-1)<<VIDTCON2_LINEVAL_S)|((LB043WQ1_XSIZE-1)<<VIDTCON2_HOZVAL_S);
	
	// Color Bit Setting.
#ifdef TFT_16BIT
	pagewidth_in_byte = LB043WQ1_XSIZE / 8 * 16;
	rWINCON0 = WINCONx_HALFW_SWAP_ON|WINCONx_4WORD_BURST|WINCONx_16BPP_565; //  inverse halfword, 4word burst, 16bpp,
#endif

#if 1//def TFT_24BIT
	pagewidth_in_byte = LB043WQ1_XSIZE / 8 * 32;
	rWINCON0 = WINCONx_16WORD_BURST|WINCONx_24BPP_888; // 16word burst, 24bpp, 
#endif
	
	rVIDOSD0A = (0 << VIDOSDxAB_HORIZON_X_S) | (0 << VIDOSDxAB_VERTICAL_Y_S); // OSD LEFT TOP PIXEL
	rVIDOSD0B = ((LB043WQ1_XSIZE - 1) << VIDOSDxAB_HORIZON_X_S) | ((LB043WQ1_YSIZE - 1) << VIDOSDxAB_VERTICAL_Y_S);//OSD RIGHT BOTTOM PIXEL

	lcd_framebuffer = (U32 *)LCD_WIN0_FRAME_BUFFER0;
	lcd_framebuffer1 = (U32 *)LCD_WIN0_FRAME_BUFFER1;
	
	rVIDW00ADD0B0 = (U32)lcd_framebuffer;
	rVIDW00ADD1B0 = (U32)lcd_framebuffer + (pagewidth_in_byte) * (LB043WQ1_YSIZE);
	// buffer size 
	rVIDW00ADD2B0 = (0<<13) | pagewidth_in_byte;

//	rVIDW00ADD0B1 = (U32)lcd_framebuffer1;
//	rVIDW00ADD1B1 = (U32)lcd_framebuffer1 + (pagewidth_in_byte) * (LB043WQ1_YSIZE);
	// buffer size 
//	rVIDW00ADD2B1 = (0<<13) | pagewidth_in_byte;

	rWIN0MAP = 0;
	rVIDCON0 |= 0x03; // ENVID On using Per Frame method
	rWINCON0 |= 0x01; //LCD ON

	//TFT_CLR(WHITE);

	LB043WQ1_BACKLIGHT_ON();
	LB043WQ1_ON();


	
	printf("LDI Initialization is finished...\n\n");	
}

void Temp_forDebug(uint32 i,uint32 j)
{
	uint8 ch=0;
	printf(" * Write_LDI(0x%02x, 0x%04x) *\n",i,j);
	
	while(!ch)
		//ch = Uart_GetKey();
		
	if(ch=='x')
		debug_continue=FALSE;
}


//========================================================================================
// library function for FIMV test
//
void Setup_RgbIf( void)
{
	uint32 LcdBppMode;

	rMISCCR |= (1<<28);		// select LCD controller for TFT lcd controller

	SetLcdPort();
	Init_LDI();

	LcdWindowOnOff(LCD_WIN_ALL,LCD_OFF);
	LcdEnvidOnOff(LCD_OFF);

	printf("\nSelect Display BPP mode\n\n     0: RGB 16bpp (565)     1: RGB 24bpp (888)\n");

	LcdBppMode = GetIntNum();
	if((LcdBppMode<0)||(LcdBppMode>1))
		LcdBppMode = 0;

	switch(LcdBppMode)
	{
	case 0 :	LcdBppMode = WINCONx_16BPP_565;
				break;
	case 1 :	LcdBppMode = WINCONx_24BPP_888;
				break;
	}

	Basic_Display_Setting(LCD_WIN_0,LCD_BUF_0,LcdBppMode,lcd_horizon_value,lcd_line_value);

	if (LcdBppMode == WINCONx_16BPP_565)
		*WINCONx_Reg_Addr[LCD_WIN_0] |= (1<<WINCON_SWAP_S);

	Display_Start(LCD_WIN_0);

}

//
//========================================================================================

//========================================================================================
// library function for Power Test
//
void Test_Lcd_QVGA(void)
{
	uint32 i,j,k;
	
	SetLcdPort();	
	Init_LDI();
	
	LcdWindowOnOff(LCD_WIN_ALL,LCD_OFF);
	LcdEnvidOnOff(LCD_OFF);
	
	Basic_Display_Setting(LCD_WIN_1,LCD_BUF_0,WINCONx_16BPP_565,lcd_horizon_value,lcd_line_value);	

	if(lcd_type==LCD_MODULE_LTS222)
	{
		for(j=0;j<screen_height;j++)
			for(i=0;i<screen_width;i++)
				PutPixel(i,j,snail16bpp[k++]);
	}
	else // by edchoi if(lcd_type==LCD_MODULE_LTV350)
	{
		for(j=0;j<screen_height;j++)
			for(i=0;i<screen_width;i++);
			//	PutPixel(i,j,prayer16bpp[k++]);
	}
	
	
	Display_Start(LCD_WIN_1);
}
//
//========================================================================================
