/**
* Project Name : Jelly(S3C2443) verification project
*
* Copyright 2008 by Embedded Crazy boys.Inc
* All rights reserved.
*
* Project Description :
* This software is only for verifying functions of the Jelly(S3C2443) . 
* Anybody can use this code.
*/

/**
* File Name    : nand.c
* Description  : Jelly(S3C2443)  USB monitor main code
* Author       : http://cafe.naver.com/embeddedcrazyboys
* Dept         : Mobile solution, AP
* Created Date : 2008.07.04
* Version      : 0.0
* History
*   R0.0 (2006.08.): http://cafe.naver.com/embeddedcrazyboys
*/



/**************** K9s1208 NAND flash ********************/
// 1block=(512+16)bytes x 32pages
// 4096block
// Block: A[23:14], Page: [13:9]
/**************** K9K2G16 NAND flash *******************/
// 1block=(2048+64)bytes x 64pages
// 2048block
// Block: A[23:14], page: [13:9]
/*****************************************************/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "def.h"
#include "2443addr.h"
#include "option.h"
#include "Nand.h"



#define BAD_CHECK	(0)
#define ECC_CHECK	(0)

#define ASM		1
#define C_LANG	2
#define DMA		3
#define TRANS_MODE 3

U8 NF8_Spare_Data[16];
U8 Adv_NF8_Spare_Data[64];



U32 srcAddress;
U32 targetBlock;	    // Block number (0 ~ 4095)
U32 targetSize;	    // Total byte size 

U32 downloadsize;
extern volatile U32 downloadAddress;

volatile int NFConDone, NFECCEncDone, NFECCDecDone;
U32 Block_Number =K9G8G_MANGO_BLOCK_NUM ;
U32 Page_Number_Per_1Block=K9G8G_MANGO_1B_PAGE_NUM ;
U8   Page_Shift_Number=7;
//*************************************************
//*************************************************
//**           H/W dependent functions           **
//************************************************* 
//*************************************************

//The code is made for bi-endian mode

// block0: reserved for boot strap
// block1~4095: used for OS image
// badblock SE: xx xx xx xx xx 00 ....
// good block SE: ECC0 ECC1 ECC2 FF FF FF ....

//=============    spare area configuration    ====================================
//     0		1		2		3		4		5		6		7		8		9		0xa ...	
//  Mecc0   Mecc1     Mecc2         FF           FF      Bad Block     FF           FF        Secc0     Secc1          FF
//====================================================================


static U8 se8Buf[16]={
	0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff
};

static U8 Adv_se8Buf[64]={
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
};


void __irq NFCon_Int(void);


//*************************************************




void * Adv_n8_func[][2]=
{  	
	// K9F2g08 (Advanced 8bit)	
	(void *)NF8_Print_Id,			"Read ID         ",
	(void *)Nand_Reset,				"Nand reset      ",
	(void *)Test_Adv_NF8_Block_Erase,	"Block erase     ",
	(void *)NAND_Erase_All,             "NAND Erase ALL     ",
	(void *)Test_Adv_NF8_Page_Read,		"Page read       ",
	(void *)Test_Adv_NF8_Page_Write,	"Page write      ",
	(void *)Test_Adv_NF8_Rw,			"Nand R/W test   ",
	(void *)Adv_NF8_PrintBadBlockNum,	"Check Badblock  ",
	(void *)Test_Adv_NF8_Lock,			"Nand Block lock ",
	(void *)Test_Adv_NF8_SoftUnLock,	"Soft Unlock     ",
	(void *)Adv_NF8_Program,			"K9F2G08 Program ",
	(void *)Test_MLC_Adv_ECC,			"MLC ECC         ",
	(void *)Adv_NF8_FromSdram_to_Nand, "From Sdram[0x31000000] to Nand Write 0",
	(void *)Adv_NF8_FromSdram_to_BlockNand, "From Sdram[0x31000000] to Nand Per block",	
	
	0,0
};
void NAND_main(void)
{

   K9F2G08_main();
}


void K9F2G08_main(void)
{
	int i;
	U8 ch;

	printf("\n Nand flash start.\n");

	NF8_Init();
   	NF8_Print_Id();
	while(1) {
		Adv_PrintSubMessage();
		printf("\nSelect(-1 to exit): ");
		i = GetIntNum();
		
		if(i==-1) break;
		
		if(i>=0 && (i<(sizeof(Adv_n8_func)/8)) ) 
	    	( (void (*)(void)) (Adv_n8_func[i][0]) )();	// execute selected function.
	}

}

U8 Read_Status(void)
{
	// Read status
	U8 ch;
	int i;
	
	NF_nFCE_L();

	NF_CMD(0x70);
	for(i=0; i<10; i++);
	ch = NF_RDDATA();

	NF_nFCE_H();

	return ch;
}

void NF8_Print_Id(void)
{
	U16 id;
	U8 maker, device;

//	NF8_Init();
	id = NF8_CheckId();
	
	device = (U8)id;
	maker = (U8)(id>>8);
	if(id==ID_K9K2G08U0M)
		{
		printf("ID_K9K2G08U0M Maker:%x, Device:%x\n", maker, device);
		Block_Number=K9F2G_JELLY_BLOCK_NUM ;
		Page_Number_Per_1Block=K9F2G_JELLY_1B_PAGE_NUM ;
		Page_Shift_Number=6; //because 1block is 64page
		}
	else if(id==ID_K9G8G08U0M)
		{
		printf("ID_K9G8G08U0M Maker:%x, Device:%x\n", maker, device);		
		Block_Number=K9G8G_MANGO_BLOCK_NUM  ;
		Page_Number_Per_1Block=K9G8G_MANGO_1B_PAGE_NUM  ;	
		Page_Shift_Number=7;//because 1block is 128page
		}

}


void InputTargetBlock_Adv(void)
{
	U32 no_block, no_page, no_byte;

	printf("\nAvailable target block number: 0~2047\n");
	printf("Input target block number:");
    targetBlock=GetIntNum();	// Block number(0~4095)
    if(targetSize==0)
    {
    	#if 0
    	printf("Input target size(0x4000*n):");
    	targetSize=GetIntNum();	// Total byte size
	#else
   	printf("Input program file size(bytes): ");
    	targetSize=GetIntNum();	// Total byte size
    	#endif
    }
	
	no_block = (U32)((targetSize/2048)/64);
	no_page = (U32)((targetSize/2048)%64);
	no_byte = (U32)(targetSize%2048);
	printf("File:%d[%d-block,%d-page,%d-bytes].\n", targetSize, no_block, no_page, no_byte);
}

void Test_Adv_NF8_Block_Erase(void)
{
	U32 block=0;

	printf("NAND Block erase\n");
/*
	if((Read_Status()&0x80)==0) {
		printf("Write protected.\n");
		return;
	}
*/
	printf("Block # to erase: ");
	block = GetIntNum();

//	NF8_Init();
	
	if(Adv_NF8_EraseBlock(block)==FAIL) return;

	printf("%d-block erased.\n", block);

}

///////////////  Advanced nand flash /////////////////////
   
void Test_Adv_NF8_Page_Read(void)
{
	U32 block=0, page=0;
	U32 i;
	unsigned char * downPt;
	
	downPt=(unsigned char *)_NONCACHE_STARTADDRESS;
		
	printf("(K9F2G08) NAND Page Read.\n");

	printf("Block # to read: ");
	block = GetIntNum();
	printf("Page # to read: ");
	page = GetIntNum();

	if(Adv_NF8_ReadPage(block, page, (U8 *)downPt )==FAIL) {
		printf("Read error.\n");
	} else {
		printf("Read OK.\n");
	}
	// Print data.
	printf("Read data(%d-block,%d-page)\n", block, page);
	
	for(i=0; i<2048; i++) {
		if((i%16)==0) printf("\n%4x: ", i);
		printf("%02x ", *(U8 *)downPt++);
		}
	printf("\n");
	printf("Spare:");
	for(i=0; i<64; i++) {
		if((i%16)==0) printf("\n%4x: ", i);
	 	printf("%02x ", Adv_NF8_Spare_Data[i]);
	}
	printf("\n");

}


///////////////////Advanced nand flash//////////////////////////////

void Test_Adv_NF8_Page_Write(void)
{
	U32 block=0, page=0;
	int i, offset;
	unsigned char * srcPt;
	srcPt=(unsigned char *)0x31100000;
		
	printf(" NAND Page Write.\n");
	printf("You must erase block before you write data!!! \n");

	printf("Block # to write: ");
	block = GetIntNum();
	printf("Page # to write: ");
	page = GetIntNum();
	printf("offset data(-1:random): ");
	offset = GetIntNum();

#if ADS10==TRUE
	srand(0);
#endif


	// Init wdata.
	for(i=0; i<2048; i++) {
	#if ADS10==TRUE
		if(offset==-1) *srcPt++ = rand()%0xff;
	#else
		if(offset==-1) *srcPt++ = i%0xff;
	#endif
		else *srcPt++ =  i+offset;
	}
    srcPt=(unsigned char *)0x31100000;
	printf("Write data[%d block, %d page].\n", block, page);
	if(Adv_NF8_WritePage(block, page, srcPt)==FAIL) {
		printf("Write Error.\n");
	} else {
		printf("Write OK.\n");
	}

 	printf("Write data is");
	for(i=0; i<2048; i++) {
		if((i%16)==0) printf("\n%4x: ", i);
		printf("%02x ", *srcPt++);
	}
	printf("\n");

	printf("Spare:");
	for(i=0; i<64; i++) {
		if((i%16)==0) printf("\n%4x: ", i);
	 	printf("%02x ", Adv_NF8_Spare_Data[i]);
	}
	printf("\n\n");

}

void Test_Adv_NF8_Rw(void)
{
	U32 block=0, page=0;
	U32 i, status=FAIL, error, offset;
	unsigned char *srcPt, *dstPt;

	
	srcPt=(unsigned char *)0x31100000;
	dstPt=(unsigned char *)0x31200000;

	printf("K9F2G08 R/W test.\n");

	printf("Block number: ");
	block = GetIntNum();
	printf("Page nember: ");
	page = GetIntNum();
	printf("offset data(-1:random): ");
	offset = GetIntNum();

#if ADS10==TRUE
	srand(0);
#endif
       
	
	// Init R/W data.
	for(i=0; i<2048; i++) *dstPt++=0x0;

	for(i=0; i<2048; i++) {
	#if ADS10==TRUE
		if(offset==-1) *srcPt++ = rand()%0xff;
	#else
		if(offset==-1) *srcPt++ = i%0xff;
	#endif
		else *srcPt++ = i+offset;
	}

    

	srcPt=(unsigned char *)0x31100000;
	dstPt=(unsigned char *)0x31200000; 
	
		// Block erase
	printf("%d block erase.\n", block);
	if(Adv_NF8_EraseBlock(block)==FAIL)	return;
	
	printf("Write data[%d block, %d page].\n", block, page);
	if(Adv_NF8_WritePage(block, page, srcPt)==FAIL) return;

	printf("Read data.\n");
   	if(Adv_NF8_ReadPage(block, page, dstPt)==FAIL) 	;
	
 
	printf("Checking data.\n");

	for(error=0, i=0; i<2048; i++) {
		if(*srcPt++!=*dstPt++) {
			printf("Error:%d[W:%x,R:%x]\n", i, *srcPt, *dstPt);
			error++;
		}
	}
	
	if(error!=0) 
		printf("Fail to R/W test(%d).\n", error);
	else
		printf(" %d pageR/W test OK.\n",page);

 }




void Adv_NF8_FromSdram_to_BlockNand(void)
{
//    unsigned long interrupt_reservoir;
    int i,error;
    int programError=0;
	U8 *srcPt,*saveSrcPt,*dstpt;
	U32 blockIndex;
	U32 no_block, no_page, no_byte;	
   U32 dstaddress;
  	printf("\n[K9F2G08 NAND Flash Per Block writing program]\n");
    printf("\n");
	printf("+-----------------------------------------------------+\n");
	printf("|   S3C2443X Nand Write Programer ver R1.0 2008. 6.   |\n");
	printf("+-----------------------------------------------------+\n");
	printf("|  Ram Start Address : 0x%8X                     |\n", downloadAddress);
	printf("|  Ram End Address   : 0x%8X                     |\n", downloadAddress + downloadsize);
	printf("+-----------------------------------------------------+\n");
	printf("|  Start Block       : 0x00000000h                    |\n");
	printf("|  Start Sector      : 0x00000000h                    |\n");
	printf("+-----------------------------------------------------+\n\n");

   NF8_Init();
   NF8_Print_Id();



    rINTMSK = BIT_ALLMSK; 	
    srcAddress=downloadAddress; 
    dstaddress=0x32000000;
	no_block = (U32)((downloadsize/2048)/64);
	no_page = (U32)((downloadsize/2048)%64);
	no_byte = (U32)(downloadsize%2048);
	printf("File:%d[%d-block,%d-page,%d-bytes].\n", downloadsize, no_block, no_page, no_byte);	
    srcPt=(U8 *)srcAddress;
    dstpt=(U8 *)dstaddress;
	printf("Block # to write: ");
	blockIndex = GetIntNum();	


     while(1) {
        saveSrcPt=srcPt;	

		#if BAD_CHECK
		if(Adv_NF8_IsBadBlock(blockIndex)==FAIL) {
		    blockIndex++;   // for next block
		    continue;
		}
		#endif

		if(Adv_NF8_EraseBlock(blockIndex)==FAIL) {
		    blockIndex++;   // for next block
		    continue;
		}

		// After 1-Block erase, Write 1-Block(32 pages).
		for(i=0;i<Page_Number_Per_1Block;i++) {
		    if(Adv_NF8_WritePage(blockIndex,i,srcPt)==FAIL) {// block num, page num, buffer
		        programError=1;
		        break;
		    }
      Delay(10);

		    if(Adv_NF8_ReadPage(blockIndex,i,dstpt)==FAIL) {
			//	printf("ECC Error(block=%d,page=%d!!!\n",blockIndex,i);
		    }

			
		srcPt+=2048;	dstpt += 2048; // Increase buffer addr one pase size
		if(i==0)  printf(".");
			//printf("\b\b\b\b\b\b\b\b%04d,%02d]", blockIndex, i);
		if((U32)srcPt>=(srcAddress+downloadsize)) // Check end of buffer
		break;	// Exit for loop
		}
		
		if(programError==1) {
		    blockIndex++;
		    srcPt=saveSrcPt;
		    programError=0;
		    continue;
		}

		if((U32)srcPt>=(srcAddress+downloadsize)) break;	// Exit while loop

		blockIndex++;
    }
    printf("\nCompare Test...\n");
	
	// Compare
	srcPt = (U8 *)srcAddress;
	dstpt = (U8 *)dstaddress;
	
	for(error=0,i=0;i<downloadsize;i++) {
		if(*srcPt++ != *dstpt++) {
			
			//printf("Error:%d[W:%x,R:%x]\n", i, *srcPt, *dstpt);
			error++;
		}
	}
	
    if(!error) printf("Nand Write Success\n");
    else printf("Error Count : %d", error);    

}


void Adv_NF8_FromSdram_to_Nand(void)
{
//    unsigned long interrupt_reservoir;
    int i,error;
    int programError=0;
	U8 *srcPt,*saveSrcPt,*dstpt;
	U32 blockIndex;
	U32 no_block, no_page, no_byte;	
   U32 dstaddress;
  	printf("\n[K9F2G08 NAND Flash writing program]\n");
    printf("\n");
	printf("+-----------------------------------------------------+\n");
	printf("|   S3C2443X Nand Write Programer ver R1.0 2008. 6.   |\n");
	printf("+-----------------------------------------------------+\n");
	printf("|  Ram Start Address : 0x%8X                     |\n", downloadAddress);
	printf("|  Ram End Address   : 0x%8X                     |\n", downloadAddress + downloadsize);
	printf("+-----------------------------------------------------+\n");
	printf("|  Start Block       : 0x00000000h                    |\n");
	printf("|  Start Sector      : 0x00000000h                    |\n");
	printf("+-----------------------------------------------------+\n\n");

   NF8_Init();
   NF8_Print_Id();



    rINTMSK = BIT_ALLMSK; 	
    srcAddress=downloadAddress; 
    dstaddress=0x32000000;
	no_block = (U32)((downloadsize/2048)/64);
	no_page = (U32)((downloadsize/2048)%64);
	no_byte = (U32)(downloadsize%2048);
	printf("File:%d[%d-block,%d-page,%d-bytes].\n", downloadsize, no_block, no_page, no_byte);	
    srcPt=(U8 *)srcAddress;
    dstpt=(U8 *)dstaddress;
    blockIndex=0;

     while(1) {
        saveSrcPt=srcPt;	

		#if BAD_CHECK
		if(Adv_NF8_IsBadBlock(blockIndex)==FAIL) {
		    blockIndex++;   // for next block
		    continue;
		}
		#endif

		if(Adv_NF8_EraseBlock(blockIndex)==FAIL) {
		    blockIndex++;   // for next block
		    continue;
		}

		// After 1-Block erase, Write 1-Block(32 pages).
		for(i=0;i<Page_Number_Per_1Block;i++) {
		    if(Adv_NF8_WritePage(blockIndex,i,srcPt)==FAIL) {// block num, page num, buffer
		        programError=1;
		        break;
		    }
      Delay(10);

		    if(Adv_NF8_ReadPage(blockIndex,i,dstpt)==FAIL) {
			//	printf("ECC Error(block=%d,page=%d!!!\n",blockIndex,i);
		    }

			
		srcPt+=2048;	dstpt += 2048; // Increase buffer addr one pase size
		if(i==0)  printf(".");
			//printf("\b\b\b\b\b\b\b\b%04d,%02d]", blockIndex, i);
		if((U32)srcPt>=(srcAddress+downloadsize)) // Check end of buffer
		break;	// Exit for loop
		}
		
		if(programError==1) {
		    blockIndex++;
		    srcPt=saveSrcPt;
		    programError=0;
		    continue;
		}

		if((U32)srcPt>=(srcAddress+downloadsize)) break;	// Exit while loop

		blockIndex++;
    }
    printf("\nCompare Test...\n");
	
	// Compare
	srcPt = (U8 *)srcAddress;
	dstpt = (U8 *)dstaddress;
	
	for(error=0,i=0;i<downloadsize;i++) {
		if(*srcPt++ != *dstpt++) {
			
			//printf("Error:%d[W:%x,R:%x]\n", i, *srcPt, *dstpt);
			error++;
		}
	}
	
    if(!error) printf("Nand Write Success\n");
    else printf("Error Count : %d", error);    

}

void Adv_NF8_Program(void)
{
//    unsigned long interrupt_reservoir;
    int i;
    int programError=0;
	U8 *srcPt,*saveSrcPt;
	U32 blockIndex;

  	printf("\n NAND Flash writing program]\n");
    printf("The program buffer: 0x30100000~0x31ffffff\n");

//	NF8_Init();

    rINTMSK = BIT_ALLMSK; 	
    srcAddress=0x30100000; 

    InputTargetBlock_Adv();
	
    srcPt=(U8 *)srcAddress;
    blockIndex=targetBlock;

     while(1) {
        saveSrcPt=srcPt;	

		#if BAD_CHECK
		if(Adv_NF8_IsBadBlock(blockIndex)==FAIL) {
		    blockIndex++;   // for next block
		    continue;
		}
		#endif

		if(Adv_NF8_EraseBlock(blockIndex)==FAIL) {
		    blockIndex++;   // for next block
		    continue;
		}

		// After 1-Block erase, Write 1-Block(32 pages).
		for(i=0;i<64;i++) {
		    if(Adv_NF8_WritePage(blockIndex,i,srcPt)==FAIL) {// block num, page num, buffer
		        programError=1;
		        break;
		    }

		#if ECC_CHECK
		    if(Adv_NF8_ReadPage(blockIndex,i,srcPt)==FAIL) {
				printf("ECC Error(block=%d,page=%d!!!\n",blockIndex,i);
		    }
		#endif
			
		srcPt+=2048;	// Increase buffer addr one pase size
		if(i==0)  printf(".");
			//printf("\b\b\b\b\b\b\b\b%04d,%02d]", blockIndex, i);
		if((U32)srcPt>=(srcAddress+targetSize)) // Check end of buffer
		break;	// Exit for loop
		}
		
		if(programError==1) {
		    blockIndex++;
		    srcPt=saveSrcPt;
		    programError=0;
		    continue;
		}

		if((U32)srcPt>=(srcAddress+targetSize)) break;	// Exit while loop

		blockIndex++;
    }

}

void Adv_NF8_PrintBadBlockNum(void)
{
    int i;
    U16 id;

    printf("\n[ K9F2G08 NAND Flash bad block check]\n");
    
    id=NF8_CheckId();
	printf("ID_K9K2G08U0M id:%x\n", id);

    for(i=0;i<Block_Number;i++)  	Adv_NF8_IsBadBlock(i);   // Print bad block
}


void Test_Adv_NF8_Lock(void)
{
    U32 num;
    U32 S_block, E_block;
	printf("(K9F2G08) NAND Lock Test !!!\n");
	printf("Select Lock type, Softlock(1)/Lock-tight(2) : ");

	num=GetIntNum();;
			
	printf("\nEnter programmable start block address ");
	S_block = GetIntNum();
	printf("Enter programmable end block address ");
    E_block = GetIntNum();

	rNFSBLK=(S_block<<6);
	rNFEBLK=(E_block<<6);
	
	if(num==1){
		rNFCONT|=(1<<16);
              printf("Software Locked\n ");
	}
	if(num==2){
		rNFCONT|=(1<<17);
              printf("Lock-tight: To clear Lock-tight, reset S3C2440!!!\n ");
	}
      printf("%d block ~ %d block are Programmable\n ", S_block, (E_block-1));
}



void Test_Adv_NF8_SoftUnLock(void)
{
    U32 S_block, E_block;
	printf("K9F2G08 NAND SoftUnLock Test !!!\n");
	
	rNFSBLK=0x0;
	rNFEBLK=0x0;

	rNFCONT&=~(1<<16);
	if(rNFCONT&(1<<17)){
		rNFCONT&=~(1<<17);
		printf("Lock-tight\n ");
		printf("You can't unlock Protected blocks !!!\n ");
		printf("%d block ~ %d block are Programmable\n ", (rNFSBLK>>6), ((rNFEBLK>>6)-1));
	}
       else printf("All blocks are Programmable\n ");
}





void Adv_PrintSubMessage(void)
{
	int i;
	
	i=0;	
	printf("\n\n");
	while(1)
	{   //display menu
	    printf("%2d:%s",i,Adv_n8_func[i][1]);
	    i++;
	    if((int)(Adv_n8_func[i][0])==0)
	    {
		 printf("\n");
		 break;
	    }
	    if((i%4)==0) printf("\n");
	}
}

static int Adv_NF8_EraseBlock(U32 block)
{
	U32 blockPage=(block<<Page_Shift_Number);
    int i;

    NFConDone=0;
    rNFCONT|=(1<<9);
    rNFCONT|=(1<<10);
    pISR_NFCON= (unsigned)NFCon_Int;
    rSRCPND=BIT_NFCON;
    rINTMSK=~(BIT_NFCON);
	
#if BAD_CHECK
    if(Adv_NF8_IsBadBlock(block)==FAIL)
	return FAIL;
#endif

	NF_nFCE_L();
    
	NF_CMD(0x60);   // Erase one block 1st command, Block Addr:A9-A25
	// Address 3-cycle
	NF_ADDR(blockPage&0xff);	    // Page number=0
	NF_ADDR((blockPage>>8)&0xff);   
	NF_ADDR((blockPage>>16)&0xff);


	NF_CLEAR_RB();
	NF_CMD(0xd0);	// Erase one blcok 2nd command
//	NF_DETECT_RB();
    while(NFConDone==0);
	rNFCONT&=~(1<<9);
	rNFCONT&=~(1<<10); // Disable Illegal Access Interrupt
	if(rNFSTAT&0x20) return FAIL;

	NF_CMD(0x70);   // Read status command

      if (NF_RDDATA()&0x1) // Erase error
      {	
    	NF_nFCE_H();
		rGPGDAT|=(1<<1);
		printf("[ERASE_ERROR:block#=%d]\n",block);

		return FAIL;
      }
       else 
       {
    	NF_nFCE_H();
    	return OK;
      }
}


void __irq NFCon_Int(void)
{
	
    NFConDone=1;
    
//  printf("Interrupt is occurred!!!\n");
//rINTMSK|=BIT_NFCON;
	ClearPending(BIT_NFCON);
	if(rNFSTAT&0x20) printf("Illegal Access is detected!!!\n");
	else if(rNFSTAT&0x10)
	{
		rNFSTAT|=0x10;
//		printf("RnB is Detected!!!\n");
	}
	else if(rNFSTAT&0x80) 
	{
		NFECCEncDone=1;
		rNFSTAT|=0x80;
		printf("ECC encoding is completed!!!\n");
	}
	else if(rNFSTAT&0x40) 
	{
		NFECCDecDone=1;
		rNFSTAT|=0x40;	
		printf("ECC decoding is completed!!!\n"); 
	}
	
}


static int Adv_NF8_IsBadBlock(U32 block)
{
    int i;
    unsigned int blockPage;
	U8 data;
    
    
    blockPage=(block<<Page_Shift_Number);	// For 2'nd cycle I/O[7:5] 
    
	NF_nFCE_L();
	NF_CLEAR_RB();

	NF_CMD(0x00);		 // read command
	NF_ADDR((Block_Number+0)&0xff);			// 2060 = 0x080c
	NF_ADDR(((Block_Number+0)>>8)&0xff);		// A[10:8]
	NF_ADDR((blockPage)&0xff);		// A[11;18]
	NF_ADDR((blockPage>>8)&0xff);	// A[26:19]
	NF_ADDR((blockPage>>16)&0xff);	// A27
	NF_CMD(0x30);	// 2'nd command
	
	NF_DETECT_RB();	 // Wait tR(max 12us)

    data=NF_RDDATA();

	NF_nFCE_H();    

     if(data!=0xff)
     {
    	printf("[block %d has been marked as a bad block(%x)]\n",block,data);
    	return FAIL;
     }
     else
     {
     printf("[Not bad block(0x%x:0x%x)]\n",block,data);
    	return OK;
     }
}


static int NF8_MarkBadBlock(U32 block)
{
       int i;
	U32 blockPage=(block<<5);
 
     se8Buf[0]=0xff;
     se8Buf[1]=0xff;    
     se8Buf[2]=0xff;    
     se8Buf[5]=0x44;   // Bad blcok mark=44
    
	NF_nFCE_L(); 
	NF_CMD(0x50);   //????
	NF_CMD(0x80);   // Write 1st command
    
	NF_ADDR(0x0);		    // The mark of bad block is 
	NF_ADDR(blockPage&0xff);	    // marked 5th spare array 
	NF_ADDR((blockPage>>8)&0xff);   // in the 1st page.
	NF_ADDR((blockPage>>16)&0xff);  //
    
	for(i=0;i<16;i++)
      {
	   NF_WRDATA(se8Buf[i]);	// Write spare array
      }

	NF_CLEAR_RB();
	NF_CMD(0x10);   // Write 2nd command
	NF_DETECT_RB();

	NF_CMD(0x70);
    
	for(i=0;i<3;i++);  //twhr=60ns////??????
    
      if (NF_RDDATA()&0x1) // Spare arrray write error
      {	
    	NF_nFCE_H();
    	printf("[Program error is occurred but ignored]\n");
       }
      else 
      {
    	NF_nFCE_H();
      }

	printf("[block #%d is marked as a bad block]\n",block);
       return OK;
}

///////////////Advanced nand flash //////////////////////

static int Adv_NF8_ReadPage(U32 block,U32 page,U8 *buffer)
{
       int i;
       unsigned int blockPage;
	   U32 Mecc, Secc;
 	   U8 *bufPt=buffer;
	   U8 se[16], ecc0, ecc1, ecc2, ecc3,a,b,c,d,e;
    
       blockPage=(block<<Page_Shift_Number)+page;// if 1block is 128Page , shift 7  and if 1block is 64page , shift 6
	   NF_RSTECC();    // Initialize ECC
	   NF_MECC_UnLock();
    
	   NF_nFCE_L();    

	   NF_CLEAR_RB();
	   
	   NF_CMD(0x00);	// Read command
       NF_ADDR(0); 	// Column (A[7:0]) = 0
       NF_ADDR(0);		// A[11:8]
       NF_ADDR((blockPage)&0xff);	// A[19:12]
       NF_ADDR((blockPage>>8)&0xff);	// A[27:20]
       NF_ADDR((blockPage>>16)&0xff);	// A[27:20]
      
       NF_CMD(0x30);	// 2'nd command
       NF_DETECT_RB();
	
      #if TRANS_MODE==C_LANG
	    for(i=0;i<2048;i++) {
	    	*bufPt++=NF_RDDATA8();	// Read one page
	    }
      #elif TRANS_MODE==DMA
		// Nand to memory dma setting
	    rSRCPND=BIT_DMA;	// Init DMA src pending.
	    rSUBSRCPND=BIT_SUB_DMA0;
	    rDISRC0=NFDATA; 	// Nand flash data register
	    rDISRCC0=(0<<1) | (1<<0); //arc=AHB,src_addr=fix
	    rDIDST0=(unsigned)bufPt;
	    rDIDSTC0=(0<<1) | (0<<0); //dst=AHB,dst_addr=inc;
	    rDCON0=(1<<31)|(1<<30)|(1<<29)|(1<<28)|(1<<27)|(0<<23)|(1<<22)|(2<<20)|(2048/4/4);
		//Handshake,AHB,interrupt,(4-burst),whole,S/W,no_autoreload,word,count=128;

		// DMA on and start.
	    rDMASKTRIG0=(1<<1)|(1<<0);

	   	while(!(rSUBSRCPND & BIT_SUB_DMA0));	// Wait until Dma transfer is done.
		
	   	rSUBSRCPND=BIT_SUB_DMA0;
	   	rSRCPND=BIT_DMA;

       #endif

   

	 NF_MECC_Lock();

	 NF_SECC_UnLock();
     Adv_NF8_Spare_Data[0]=NF_RDDATA8();
	 Mecc=NF_RDDATA();
	 rNFMECCD0=((Mecc&0xff00)<<8)|(Mecc&0xff);
	 rNFMECCD1=((Mecc&0xff000000)>>8)|((Mecc&0xff0000)>>16);
	
	 
	 Adv_NF8_Spare_Data[1]=Mecc&0xff;
	 Adv_NF8_Spare_Data[2]=(Mecc&0xff00)>>8;
	 Adv_NF8_Spare_Data[3]=(Mecc&0xff0000)>>16;
	 Adv_NF8_Spare_Data[4]=(Mecc&0xff000000)>>24;
	 
	 Adv_NF8_Spare_Data[5]=NF_RDDATA8();  // read 5
	 Adv_NF8_Spare_Data[6]=NF_RDDATA8();  // read 6
	 Adv_NF8_Spare_Data[7]=NF_RDDATA8();  // read 7
	 
	 NF_SECC_Lock();
	 Secc=NF_RDDATA();
	 rNFSECCD=((Secc&0xff00)<<8)|(Secc&0xff);
	 Adv_NF8_Spare_Data[8]=Secc&0xff;
	 Adv_NF8_Spare_Data[9]=(Secc&0xff00)>>8;
	 Adv_NF8_Spare_Data[10]=(Secc&0xff0000)>>16;
	 Adv_NF8_Spare_Data[11]=(Secc&0xff000000)>>24;
	 
	 for(i=12;i<64;i++) {
    	Adv_NF8_Spare_Data[i]=NF_RDDATA8();	// Read spare array with 4byte width
       }
	
	 NF_nFCE_H();    

	 if ((rNFECCERR0&0xf) == 0x0){
	       //printf("ECC OK!\n");
		return OK;
	 }
	 else {
		//printf("ECC FAIL!\n");
	       return FAIL;
	 }


}

//////////////// Advanced nand flash ////////////////////////////////

static int Adv_NF8_WritePage(U32 block,U32 page,U8 *buffer)
{
    int i;
	U32 blockPage, Mecc, Secc;
	U8 *bufPt=buffer;

	NFConDone=0;
    rNFCONT|=(1<<9);
    rNFCONT|=(1<<10);
    pISR_NFCON= (unsigned)NFCon_Int;
    rSRCPND=BIT_NFCON;
    rINTMSK=~(BIT_NFCON);
	  
	NF_RSTECC();    // Initialize ECC
    NF_MECC_UnLock();
	blockPage=(block<<Page_Shift_Number)+page;

	NF_nFCE_L(); 
	NF_CMD(0x80);   // Write 1st command
	NF_ADDR(0); 	// Column (A[7:0]) = 0
	NF_ADDR(0); 	// A[11:8]
	NF_ADDR((blockPage)&0xff);	// A[19:12]
	NF_ADDR((blockPage>>8)&0xff);	// A[27:20]
	NF_ADDR((blockPage>>16)&0xff);	// A[27:20]
	
	
#if TRANS_MODE==C_LANG
     
	for(i=0;i<2048;i++) {
		NF_WRDATA8(*bufPt++);	// Write one page to NFM from buffer
    }
#elif TRANS_MODE==DMA
      
	// Memory to Nand dma setting
	rSUBSRCPND=BIT_SUB_DMA0;
	rSRCPND=BIT_DMA;	// Init DMA src pending.
	rDISRC0=(unsigned)bufPt; 	// Nand flash data register
	rDISRCC0=(0<<1) | (0<<0); //arc=AHB,src_addr=inc
	rDIDST0=NFDATA;
	rDIDSTC0=(0<<1) | (1<<0); //dst=AHB,dst_addr=fix;
	rDCON0=(1<<31)|(1<<30)|(1<<29)|(0<<28)|(1<<27)|(0<<23)|(1<<22)|(2<<20)|(2048/4);
	//  only unit transfer in writing!!!!
	//Handshake,AHB,interrupt,(unit),whole,S/W,no_autoreload,word,count=128;
	
	// DMA on and start.
	rDMASKTRIG0=(1<<1)|(1<<0);
	
	while(!(rSUBSRCPND & BIT_SUB_DMA0));	// Wait until Dma transfer is done.
	
	rSUBSRCPND=BIT_SUB_DMA0;
	rSRCPND=BIT_DMA;	
#endif


       NF_MECC_Lock();
	// Get ECC data.
	// Spare data for 8bit
	// byte  0                  1    2     3     4          5               6      7            8         9
	// ecc  [Bad marking] [0]  [1]  [2]  [3]    x                       SECC0  SECC1
	Mecc = rNFMECC0;

	Adv_se8Buf[0]=0xff;
	Adv_se8Buf[1]=(U8)(Mecc&0xff);
	Adv_se8Buf[2]=(U8)((Mecc>>8) & 0xff);
	Adv_se8Buf[3]=(U8)((Mecc>>16) & 0xff);
	Adv_se8Buf[4]=(U8)((Mecc>>24) & 0xff);
			// Marking good block

	NF_SECC_UnLock();
	//Write extra data(ECC, bad marking)
	for(i=0;i<8;i++) {
		NF_WRDATA8(Adv_se8Buf[i]);	// Write spare array(Main ECC)
		Adv_NF8_Spare_Data[i]=Adv_se8Buf[i];
    	}  
     NF_SECC_Lock(); 
	Secc=rNFSECC; 
	Adv_se8Buf[8]=(U8)(Secc&0xff);
	Adv_se8Buf[9]=(U8)((Secc>>8) & 0xff);
	
	for(i=5;i<64;i++) {
		NF_WRDATA8(Adv_se8Buf[i]);  // Write spare array(Spare ECC and Mark)
		Adv_NF8_Spare_Data[i]=Adv_se8Buf[i];
	}  
 	NF_CLEAR_RB();
	NF_CMD(0x10);	 // Write 2nd command
//	NF_DETECT_RB();
	while(NFConDone==0);
	 rNFCONT&=~(1<<9);
	 rNFCONT&=~(1<<10); // Disable Illegal Access Interrupt
	 if(rNFSTAT&0x20) return FAIL;

	NF_CMD(0x70);   // Read status command   
    
	for(i=0;i<3;i++);  //twhr=60ns
    
    if (NF_RDDATA()&0x1) {// Page write error
        NF_nFCE_H();
		printf("[PROGRAM_ERROR:block#=%d]\n",block);
		NF8_MarkBadBlock(block);
		return FAIL;
    } else {
        NF_nFCE_H();
	    return OK;
	}

}



static U16 NF8_CheckId(void)
{
    int i;
    U16 id, id4th;
//    rEBICON|=(1<<8);
    
    NF_nFCE_L();
 //   NF_nCS1_L();
	
    NF_CMD(0x90);
    NF_ADDR(0x0);

    for (i=0; i<10; i++);
//    printf("NFSTAT: 0x%x\n", rNFSTAT);

    id=NF_RDDATA8()<<8;	// Maker code 0xec
    id|=NF_RDDATA8();	// Devide code(K9S1208V:0x76), (K9K2G16U0M:0xca)

    NF_nFCE_H();
 //   NF_nCS1_H();
    return id;
}


void Nand_Reset(void)
{
    	int i;
   
	NF_nFCE_L();

	NF_CLEAR_RB();
	NF_CMD(0xFF);	//reset command
	
	for(i=0;i<10;i++);  //tWB = 100ns. //??????
	NF_DETECT_RB();
	
	NF_nFCE_H();

}

void NAND_Erase_All(void)
{

	U32 i, block;
	
	printf(" NAND Block erase ALL\n");

	printf("Erase All Yes(1)/No(0) : ");
	block = GetIntNum();

	if(block != 1) return;
	
	for(i=0;i<Block_Number;i++) {
		
		printf("%d Block Erasing -> ", i);
		if(Adv_NF8_EraseBlock(i) == FAIL) {
		
			printf("Failed\n");
		} else {
		
			printf("Success\n");
		}
	}
}

static void NF8_Init(void)
{
	// for S3C2413

	rNFCONF = (TACLS<<12)|(TWRPH0<<8)|(TWRPH1<<4)|(0<<0);	
	// TACLS		[14:12]	CLE&ALE duration = HCLK*TACLS.
	// TWRPH0		[10:8]	TWRPH0 duration = HCLK*(TWRPH0+1)
	// TWRPH1		[6:4]	TWRPH1 duration = HCLK*(TWRPH1+1)
	// AdvFlash(R)	[3]		Advanced NAND, 0:256/512, 1:1024/2048
	// PageSize(R)	[2]		NAND memory page size
	//						when [3]==0, 0:256, 1:512 bytes/page.
	//						when [3]==1, 0:1024, 1:2048 bytes/page.
	// AddrCycle(R)	[1]		NAND flash addr size
	//						when [3]==0, 0:3-addr, 1:4-addr.
	//						when [3]==1, 0:4-addr, 1:5-addr.
	// BusWidth(R/W) [0]	NAND bus width. 0:8-bit, 1:16-bit.
	
	//rNFCONT = (0<<17)|(0<<16)|(0<<10)|(0<<9)|(0<<8)|(1<<7)|(1<<6)|(1<<5)|(1<<4)|(1<<1)|(1<<0);
	rNFCONT = (0<<17)|(0<<16)|(0<<10)|(0<<9)|(0<<8)|(1<<7)|(1<<6)|(1<<5)|(1<<4)|(0x3<<1)|(1<<0);
	// Lock-tight	[17]	0:Disable lock, 1:Enable lock.
	// Soft Lock	[16]	0:Disable lock, 1:Enable lock.
	// EnablillegalAcINT[10]	Illegal access interupt control. 0:Disable, 1:Enable
	// EnbRnBINT	[9]		RnB interrupt. 0:Disable, 1:Enable
	// RnB_TrandMode[8]		RnB transition detection config. 0:Low to High, 1:High to Low
	// SpareECCLock	[7]		0:Unlock, 1:Lock
	// MainECCLock	[6]		0:Unlock, 1:Lock
	// InitMECC(W)	[5]		1:Init main area ECC decoder/encoder.
	// InitSECC(W)	[4]		1:Init spare area ECC decoder/encoder.
	// Reg_nCE1		[2]		0:nFCE=0, 1:nFCE=1.
	// Reg_nCE0		[1]		0:nFCE=0, 1:nFCE=1.
	// NANDC Enable	[0]		operating mode. 0:Disable, 1:Enable.

//	rNFSTAT = 0;
    
//    Nand_Reset();
}




#define	SRAM_SADDR	0x40000000
#define	SRAM_EADDR	(SRAM_SADDR+ (4*1024))	// Total : 4KB, 0x3100_0000-0x3101_FFFF



void SRAM_Test(int Print_msg)
{
	
	U32 addr=0, indata, src_data, i;
	U32 error;
	static int offset=0;
	U16 jump_offset=1;

	
	//printf("Check: SRAM Area must be in non-cacheable area!\n");
	//printf("SRAM W/R test[%xh-%xh], R/W offset[%d]\n", SRAM_SADDR, SRAM_EADDR-1, offset);
	for(i=0; i<3; i++) {		// 8/16/32-bit.
//	for(i=2; i<3; i++) {		// 8/16/32-bit.

		if(i==0) jump_offset=1;
		else if(i==1) jump_offset=2;
		else if(i==2) jump_offset=4;
		
		///////////////////// Clear Source/Target ////////////////////
		//printf("Clear data.\n");
		for(addr=0; (SRAM_SADDR+addr)<SRAM_EADDR; addr+=4) {
			*(U32 *)(SRAM_SADDR+addr) = 0x0;
			if(!(addr%(4*1024))) {
				//printf("\b\b\b\b\b\b\b\b\b\b%10x", SRAM_SADDR+addr);
				}
		}
		//printf("\b\b\b\b\b\b\b\b\b\b%10x\n", SRAM_SADDR+addr);

		///////////////////// Write ////////////////////
		//printf("Write data[%x-%x].\n", SRAM_SADDR, SRAM_EADDR);
		if(Print_msg>0) printf("Wr(%d-bit)...", jump_offset*8);
		if(Print_msg>1) printf("ADDR:%8x", SRAM_SADDR);
		for(addr=0; (SRAM_SADDR+addr)<SRAM_EADDR; addr+=jump_offset) {
			
			src_data = addr+offset;
		        
			switch(jump_offset) {
				case 1:
					*(U8 *)(SRAM_SADDR+addr) = src_data;
				break;
				case 2:
					*(U16 *)(SRAM_SADDR+addr) = src_data;
				break;
				case 4:
					*(U32 *)(SRAM_SADDR+addr) = src_data;
				break;
			}
			if(!(addr%(4*1024))) {
				if(Print_msg>1) printf("\b\b\b\b\b\b\b\b%8x", SRAM_SADDR+addr);
				}
		}
		if(Print_msg>1) printf("\b\b\b\b\b\b\b\b%8x  ", SRAM_SADDR+addr);

		///////////////////// Verify //////////////////////
		//printf("Verify[%x-%x].\n", SRAM_SADDR, SRAM_EADDRs
		if(Print_msg>1) printf("ADDR:%8x", SRAM_SADDR);

		for(error=0, addr=0; (SRAM_SADDR+addr)<SRAM_EADDR; addr+=jump_offset) {
			switch(jump_offset) {
				case 1:
					src_data = (U8)(addr+offset);
					indata = *(U8 *)(SRAM_SADDR+addr);
				break;
				case 2:
					src_data = (U16)(addr+offset);
					indata = *(U16 *)(SRAM_SADDR+addr);
				break;
				case 4:
					src_data = (U32)(addr+offset);
					indata = *(U32 *)(SRAM_SADDR+addr);
				break;
			}

			if(!(addr%(4*1024))) {
				if(Print_msg>1) printf("\b\b\b\b\b\b\b\b%8x", SRAM_SADDR+addr);
			}
			indata = indata|0x800;
			if(indata != (src_data|0x800)) {
				error++;
				printf("%xH[W:%x, R:%x]\n", addr, src_data, indata);
			}
		}
		if(Print_msg>1) printf("\b\b\b\b\b\b\b\b%8x  ", SRAM_SADDR+addr);
		
		if(error!=0) {
			if(Print_msg>0) printf("ERROR(%d)...\n\n", error);
		} else {
			if(Print_msg>0) printf("OK!\n");
			//printf(".");
		}

	}

offset++;
}




void Test_MLC_Adv_ECC(void)
{

    int i;
    U32 block, page;
	U32 blockPage, Mecc0, Mecc1, Mecc2, Mecc3, Mecc4, Mecc5, Mecc6, Mecc7, Secc;
	//U8 *bufPt=buffer;
	U8 *bufPt;
	
	
	Adv_NF8_EraseBlock(1);
	bufPt=(unsigned char *)0x31100000;

	NFConDone=0, NFECCEncDone=0, NFECCDecDone=0;
	
	rNFCONF = (rNFCONF & ~(1<<30))|(1<<24); // System Clock is more than 66Mhz, ECC type is MLC.
	rNFCONT |= (1<<18); //ECC for programming.
	
    rNFCONT|=(1<<9); // Enable RnB Interrupt 
    rNFCONT|=(1<<10); // Enable Illegal Access Interrupt
    
    rNFCONT|=(1<<13); // Enable ECC encoding & decoding conpletion interrupt enable
    rNFCONT|=(1<<12); // Enable ECC encoding & decoding conpletion interrupt enable
    
    
    pISR_NFCON= (unsigned)NFCon_Int;
    rSRCPND=BIT_NFCON;
    rINTMSK&=~(BIT_NFCON);
 
    /////////////////////////////////////////////////
    // block1, page0 writing with valid data, ecc   /
    /////////////////////////////////////////////////
    
    block=1;
    page=0;
    
	blockPage=(block<<Page_Shift_Number)+page;

	NF_nFCE_L(); 
	NF_CMD(0x0);//??????
	NF_CMD(0x80);   // Write 1st command
	NF_ADDR(0); 	// Column (A[7:0]) = 0
	NF_ADDR(0); 	// A[11:8]
	NF_ADDR((blockPage)&0xff);	// A[19:12]
	NF_ADDR((blockPage>>8)&0xff);	// A[27:20]
	NF_ADDR((blockPage>>16)&0xff);	// A[27:20]


// 0~511	
    NF_RSTECC();    // Initialize ECC
    NF_MECC_UnLock();
    rNFCONT=0x43075;
    for(i=0;i<512;i++) {
		NF_WRDATA8(i);	// Write one page to NFM from buffer
    }
    
    while(!(rNFSTAT&(1<<7))) ;
    rNFSTAT|=(1<<7);
 
   	Mecc0 = rNFMECC0;
	Mecc1 = rNFMECC1;

    
// 512~1023
    NF_RSTECC();    // Initialize ECC
    NF_MECC_UnLock();
    for(i=512;i<1024;i++) {
		NF_WRDATA8(i);	// Write one page to NFM from buffer
    }
   while(!(rNFSTAT&(1<<7))) ;
   rNFSTAT|=(1<<7);
    
	Mecc2 = rNFMECC0;
	Mecc3 = rNFMECC1;


// 1024~1535
    NF_RSTECC();    // Initialize ECC
    NF_MECC_UnLock();
    for(i=1024;i<1536;i++) {
		NF_WRDATA8(i);	// Write one page to NFM from buffer
    }
   while(!(rNFSTAT&(1<<7))) ;
   rNFSTAT|=(1<<7);

   Mecc4 = rNFMECC0;
   Mecc5 = rNFMECC1;

// 1536~2047
    NF_RSTECC();    // Initialize ECC
    NF_MECC_UnLock();
    for(i=1536;i<2048;i++) {
		NF_WRDATA8(i);	// Write one page to NFM from buffer
    }
   while(!(rNFSTAT&(1<<7))) ;
   rNFSTAT|=(1<<7);
   
   	Mecc6 = rNFMECC0;
	Mecc7 = rNFMECC1;
    

	// Get ECC data.
	// Spare data for 8bit
	// byte  0     1    2     3    4     5     6     7      8      9
	// ecc  [0]  [1]  [2]  [3]    [4]   [5]   [6]                    
    	
	Adv_se8Buf[1]=(U8)(Mecc0&0xff);
	Adv_se8Buf[2]=(U8)((Mecc0>>8) & 0xff);
	Adv_se8Buf[3]=(U8)((Mecc0>>16) & 0xff);
	Adv_se8Buf[4]=(U8)((Mecc0>>24) & 0xff);
	Adv_se8Buf[5]=(U8)(Mecc1&0xff);
	Adv_se8Buf[6]=(U8)((Mecc1>>8) & 0xff);
	Adv_se8Buf[7]=(U8)((Mecc1>>16) & 0xff);
	
   	Adv_se8Buf[8]=(U8)(Mecc2&0xff);
	Adv_se8Buf[9]=(U8)((Mecc2>>8) & 0xff);
	Adv_se8Buf[10]=(U8)((Mecc2>>16) & 0xff);
	Adv_se8Buf[11]=(U8)((Mecc2>>24) & 0xff);
	Adv_se8Buf[12]=(U8)(Mecc3&0xff);
	Adv_se8Buf[13]=(U8)((Mecc3>>8) & 0xff);
	Adv_se8Buf[14]=(U8)((Mecc3>>16) & 0xff);
  	
    Adv_se8Buf[15]=(U8)(Mecc4&0xff);
	Adv_se8Buf[16]=(U8)((Mecc4>>8) & 0xff);
	Adv_se8Buf[17]=(U8)((Mecc4>>16) & 0xff);
	Adv_se8Buf[18]=(U8)((Mecc4>>24) & 0xff);
	Adv_se8Buf[19]=(U8)(Mecc5&0xff);
	Adv_se8Buf[20]=(U8)((Mecc5>>8) & 0xff);
	Adv_se8Buf[21]=(U8)((Mecc5>>16) & 0xff);

	Adv_se8Buf[22]=(U8)(Mecc6&0xff);
	Adv_se8Buf[23]=(U8)((Mecc6>>8) & 0xff);
	Adv_se8Buf[24]=(U8)((Mecc6>>16) & 0xff);
	Adv_se8Buf[25]=(U8)((Mecc6>>24) & 0xff);
	Adv_se8Buf[26]=(U8)(Mecc7&0xff);
	Adv_se8Buf[27]=(U8)((Mecc7>>8) & 0xff);
	Adv_se8Buf[28]=(U8)((Mecc7>>16) & 0xff);

	for(i=0;i<64;i++) {
		NF_WRDATA8(Adv_se8Buf[i]);	// Write spare array(Main ECC)
		Adv_NF8_Spare_Data[i]=Adv_se8Buf[i];
    }
      
  	NF_CLEAR_RB();
	NF_CMD(0x10);	 // Write 2nd command

	NF_DETECT_RB();
//	while(NFConDone==0);
  	rNFCONT&=~(1<<9);
	rNFCONT&=~(1<<10); // Disable Illegal Access Interrupt
	if(rNFSTAT&0x20) 
	{
		printf("Illegal access Error\n"); 
		return;	
	}
    
	NF_CMD(0x70);   // Read status command   
    
	for(i=0;i<3;i++);  //twhr=60ns
    
    if (NF_RDDATA8()&0x1) {// Page write error
    	NF_nFCE_H();
		printf("first[PROGRAM_ERROR:block#=%d]\n",block);
	//	Adv_NF8_MarkBadBlock(block);
		return;
    } else {
    	NF_nFCE_H();
	}
	
	/////////////////////////////////////////////////
    // block1, page1 writing with invalid data, ecc /
    /////////////////////////////////////////////////
    
    NFConDone=0;
    rSRCPND=BIT_NFCON;
    rINTMSK=~(BIT_NFCON);
        
    NF_RSTECC();    // Initialize ECC
    NF_MECC_UnLock();
    
	block=1;
    page=1;
    
	blockPage=(block<<Page_Shift_Number)+page;

	NF_nFCE_L(); 
	NF_CMD(0x0);//??????
	NF_CMD(0x80);   // Write 1st command
	NF_ADDR(0);			    // Column 0
	NF_ADDR(0);			    // Column 0
	NF_ADDR(blockPage&0xff);	    //
	NF_ADDR((blockPage>>8)&0xff);   // Block & page num.
	NF_ADDR((blockPage>>16)&0xff);  //

    //NF_RSTECC();    // Initialize ECC

#if 0 //1-bit Error
	for(i=0;i<512;i++) {
			NF_WRDATA8(i);
	}
#elif 0 //1-bit Error
	for(i=0;i<512;i++) {
		if(i==200) {
			NF_WRDATA8(0xc0);
		}
		else {
			NF_WRDATA8(i);
		}		
	}
#elif 0 //2-bit Error
	for(i=0;i<512;i++) {
		if(i==128) {
			NF_WRDATA8(129);
		}
		else if(i==255) {
			NF_WRDATA8(0x7f);
		}								
		else {
			NF_WRDATA8(i);
		}		
	}
#elif 0	//3-bit Error
	for(i=0;i<512;i++) {
		if(i==10) {
			NF_WRDATA8(11);
		}
		else if(i==511) {
			NF_WRDATA8(0xdf);
		}			
		else if(i==128)	{
			NF_WRDATA8(0x82);
		}					
		else {
			NF_WRDATA8(i);
		}
	}	
#elif 1  // 4-bit Error  
	for(i=0;i<512;i++) {
		if(i==10) {
			NF_WRDATA8(11);
		}
		else if(i==500) {
			NF_WRDATA8(0xf0);
		}			
		else if(i==200) {
			NF_WRDATA8(0xc0);
		}			
		else if(i==30) {
			NF_WRDATA8(0x16);
		}			
		else {
			NF_WRDATA8(i);
		}		
	}
#endif			
    
     
    while(!(rNFSTAT&(1<<7))) ;
    //while(NFECCEncDone==0);
    rNFSTAT|=(1<<7);

    
	Mecc0 = rNFMECC0;
	Mecc1 = rNFMECC1;
	
	// 512~1023
    NF_RSTECC();    // Initialize ECC
    NF_MECC_UnLock();
    for(i=512;i<1024;i++) {
		NF_WRDATA8(i);	// Write one page to NFM from buffer
    }
    
    while(!(rNFSTAT&(1<<7))) ;
    rNFSTAT|=(1<<7);

	Mecc2 = rNFMECC0;
	Mecc3 = rNFMECC1;
    
     
// 1024~1535
    NF_RSTECC();    // Initialize ECC
    NF_MECC_UnLock();
    for(i=1024;i<1536;i++) {
		NF_WRDATA8(i);	// Write one page to NFM from buffer
    }
    
    while(!(rNFSTAT&(1<<7))) ;
    rNFSTAT|=(1<<7);
    
    Mecc4 = rNFMECC0;
	Mecc5 = rNFMECC1;
    
// 1536~2047
    NF_RSTECC();    // Initialize ECC
    NF_MECC_UnLock();
    for(i=1536;i<2048;i++) {
		NF_WRDATA8(i);	// Write one page to NFM from buffer
    }
    
    while(!(rNFSTAT&(1<<7))) ;
    rNFSTAT|=(1<<7);
    
    
    Mecc6 = rNFMECC0;
	Mecc7 = rNFMECC1;

	
	for(i=0;i<64;i++) {
		NF_WRDATA8(Adv_se8Buf[i]);	// Write spare array(Main ECC)
		Adv_NF8_Spare_Data[i]=Adv_se8Buf[i];
    }
   	
   		
 	NF_CLEAR_RB();
	NF_CMD(0x10);	 // Write 2nd command
	NF_DETECT_RB();
	//while(NFConDone==0);
	rNFCONT&=~(1<<9);
	rNFCONT&=~(1<<10); // Disable Illegal Access Interrupt
	if(rNFSTAT&0x20)	{
		printf("Error\n"); 
		return;	
	}

	NF_CMD(0x70);   // Read status command   
    
	for(i=0;i<30;i++);  //twhr=60ns
    
    if (NF_RDDATA8()&0x1) {// Page write error
    	NF_nFCE_H();
		printf("Second[PROGRAM_ERROR:block#=%d]\n",block);
	//	Adv_NF8_MarkBadBlock(block);
		return;
    } else {
    	NF_nFCE_H();		
	}
	
    
	/////////////////////////////////////////////////
    // block1, page1 reading with invalid data, ecc /
    /////////////////////////////////////////////////
     pISR_NFCON= (unsigned)NFCon_Int;
    rSRCPND|=BIT_NFCON;
    rINTMSK&=~(BIT_NFCON);
 

	block=1;
	page=1;
    blockPage=(block<<Page_Shift_Number)+page;

    printf("[1] NFCONF: %x  NFCONT: %x NFSTAT: %x SRCPND: %x INTMSK: %x\n", rNFCONF, rNFCONT, rNFSTAT, rSRCPND, rINTMSK);
	rNFCONF = (rNFCONF & ~(1<<30))|(1<<24); // System Clock is more than 66Mhz, ECC type is MLC.
	rNFCONT &= ~(1<<18); //ECC for reading.
	    
	NF_RSTECC();    // Initialize ECC
	NF_MECC_UnLock();
    
   	NF_nFCE_L();
	NF_CLEAR_RB();
	
	rNFCONT|=(1<<12);		 // Eccdone clear
	     
	NF_CMD(0x00);	// Read command
    NF_ADDR(0); 	// Column (A[7:0]) = 0
    NF_ADDR(0);		// A[11:8]
    NF_ADDR((blockPage)&0xff);	// A[19:12]
    NF_ADDR((blockPage>>8)&0xff);	// A[27:20]
    NF_ADDR((blockPage>>16)&0xff);	// A[27:20]
      
    NF_CMD(0x30);	// 2'nd command
    
    NF_DETECT_RB();
       
   

    for(i=0;i<512;i++) {
    	*bufPt++=NF_RDDATA8();	// Read one page
    }
    
 

    NF_CMD(0x05);
    NF_ADDR((2048+1)&0xff);			// 2060 = 0x080c
	NF_ADDR(((2048+1)>>8)&0xff);		// A[10:8]
	NF_CMD(0xe0);
//	for(i=0;i<10;i++);
	for(i=1;i<8;i++) {
    	Adv_NF8_Spare_Data[i]=NF_RDDATA8();	// Read one page
    }
//     NF_MECC_Lock();
    printf("[2] NFCONF: %x  NFCONT: %x NFSTAT: %x SRCPND: %x INTMSK: %x\n", rNFCONF, rNFCONT, rNFSTAT, rSRCPND, rINTMSK);
    while(!(rNFSTAT&(1<<6))) ;
	printf("[3] NFCONF: %x  NFCONT: %x NFSTAT: %x SRCPND: %x INTMSK: %x\n", rNFCONF, rNFCONT, rNFSTAT, rSRCPND, rINTMSK);
//	while(NFECCDecDone==0);
    rNFSTAT|=(1<<6);
      	printf("1st ECC check!\n");
 	if ((rNFECCERR0&(0x7<<26)) == 0x0){
	       printf("ECC OK!\n");
//		return;
	 }
	else {
		printf("ECC FAIL!\n");
		printf("status0:0x%x|status1:0x%x|bit:0x%x\n", rNFECCERR0, rNFECCERR1, rNFMLCBITPT);
//	    return;
	 }


// 512 ~ 1023
    NF_RSTECC();    // Initialize ECC
//    NF_MECC_UnLock();
    NF_CMD(0x05);
    NF_ADDR((512)&0xff);			// 2060 = 0x080c
	NF_ADDR(((512)>>8)&0xff);		// A[10:8]
	NF_CMD(0xe0);
	for(i=0;i<10;i++);
	for(i=0;i<512;i++) {
    	*bufPt++=NF_RDDATA8();	// Read one page
    }
    
//    NF_MECC_Lock();
	
	NF_CMD(0x05);
    NF_ADDR((2048+8)&0xff);			// 2060 = 0x080c
	NF_ADDR(((2048+8)>>8)&0xff);		// A[10:8]
	NF_CMD(0xe0);
	for(i=0;i<10;i++);
	for(i=8;i<15;i++) {
    	Adv_NF8_Spare_Data[i]=NF_RDDATA8();	// Read one page
    }
      while(!(rNFSTAT&(1<<6))) ;
//    while(NFECCDecDone==0);
	  rNFSTAT|=(1<<6);
    
		printf("2nd ECC check!\n");
	 if ((rNFECCERR0&(0x7<<26)) == 0x0){
	       printf("ECC OK!\n");
//		return;
	 }
	 else {
		printf("ECC FAIL!\n");
		printf("status0:0x%x|status1:0x%x|bit:0x%x\n", rNFECCERR0, rNFECCERR1, rNFMLCBITPT);
//	    return;
	 }


// 1024 ~ 1535
    NF_RSTECC();    // Initialize ECC
//    NF_MECC_Lock();
    NF_CMD(0x05);
    NF_ADDR((1024)&0xff);			// 2060 = 0x080c
	NF_ADDR(((1024)>>8)&0xff);		// A[10:8]
	NF_CMD(0xe0);
	for(i=0;i<10;i++);
	for(i=0;i<512;i++) {
    	*bufPt++=NF_RDDATA8();	// Read one page
    }
    
//    NF_MECC_Lock();
    
	NF_CMD(0x05);
    NF_ADDR((2048+15)&0xff);			// 2060 = 0x080c
	NF_ADDR(((2048+15)>>8)&0xff);		// A[10:8]
	NF_CMD(0xe0);
	for(i=0;i<10;i++);
	for(i=15;i<22;i++) {
    	Adv_NF8_Spare_Data[i]=NF_RDDATA8();	// Read one page
    }
     
	    while(!(rNFSTAT&(1<<6))) ;
//    while(NFECCDecDone==0);
    rNFSTAT|=(1<<6);
    
		printf("3rd ECC check!\n");
	 if ((rNFECCERR0&(0x7<<26)) == 0x0){
	       printf("ECC OK!\n");
//		return;
	 }
	 else {
		printf("ECC FAIL!\n");
		printf("status0:0x%x|status1:0x%x|bit:0x%x\n", rNFECCERR0, rNFECCERR1, rNFMLCBITPT);
//	    return;
	 }

	
// 1536 ~ 2047
    NF_RSTECC();    // Initialize ECC
//    NF_MECC_UnLock();
    NF_CMD(0x05);
    NF_ADDR((1536)&0xff);			// 2060 = 0x080c
	NF_ADDR(((1536)>>8)&0xff);		// A[10:8]
	NF_CMD(0xe0);
	for(i=0;i<10;i++);
	for(i=0;i<512;i++) {
    	*bufPt++=NF_RDDATA8();	// Read one page
    }
    
    
//    NF_MECC_Lock();
	
	NF_CMD(0x05);
    NF_ADDR((2048+22)&0xff);			// 2060 = 0x080c
	NF_ADDR(((2048+22)>>8)&0xff);		// A[10:8]
	NF_CMD(0xe0);
	for(i=0;i<10;i++);
	for(i=22;i<29;i++) {
    	Adv_NF8_Spare_Data[i]=NF_RDDATA8();	// Read one page
    }

	    while(!(rNFSTAT&(1<<6))) ;
//    while(NFECCDecDone==0);
    rNFSTAT|=(1<<6);
    
    
	printf("4th ECC check!\n");
	 if ((rNFECCERR0&(0x7<<26)) == 0x0){
	       printf("ECC OK!\n");
//		return;
	 }
	 else {
		printf("ECC FAIL!\n");
		printf("status0:0x%x|status1:0x%x|bit:0x%x\n", rNFECCERR0, rNFECCERR1, rNFMLCBITPT);
//	    return;
	 }

 

	
     NF_nFCE_H();    


	
}



