/**
* Project Name : Jelly(S3C2443) verification project
*
* Copyright 2008 by Embedded Crazy boys.Inc
* All rights reserved.
*
* Project Description :
* This software is only for verifying functions of the Jelly(S3C2443) . 
* Anybody can use this code.
*/

/**
* File Name    : Uartdown.cpp
* Description  : Jelly(S3C2443)  USB monitor main code
* Author       : http://cafe.naver.com/embeddedcrazyboys
* Dept         : Mobile solution, AP
* Created Date : 2008.07.04
* Version      : 0.0
* History
*   R0.0 (2006.08.): http://cafe.naver.com/embeddedcrazyboys
*/

#include <stdio.h>
#include <stdlib.h>

#include "Option.h"
#include "System.h"
#include "Console.h"
#include "MMUCache.h"
#include "Exception.h"
#include "2443addr.h"
#include "memtest.h"
#include "pll.h"
#include "uartdown.h"


#define	UART_OS_START_ADDR_OFFSET 	(0x01000000)
#define UART_DOWNLOAD_ADDRESS 			(_RAM_STARTADDRESS+UART_OS_START_ADDR_OFFSET)
extern U32 downloadsize;
extern volatile U32 downloadAddress;
extern int download_run;
extern volatile unsigned int DebugUartCh;

extern volatile U32 tempDownloadAddress;
U32 Uart_temp_download_addr;


void (*uart_restart)(void)=(void (*)(void))0x0;
void (*uart_run)(void)=(void (*)(void))UART_DOWNLOAD_ADDRESS;

volatile unsigned char *uart_downPt;
volatile unsigned int uart_err=0;

void __irq Uart0_RxInt(void)
{
	rSUBSRCPND = BIT_SUB_RXD0;
    	rSUBSRCPND;
    	ClearPending(BIT_UART0);
	*uart_downPt++=RdURXH0();//RdURXH0
}

void __irq Uart1_RxInt(void)
{

 	rSUBSRCPND = BIT_SUB_RXD1;
    	rSUBSRCPND;
    
    	ClearPending(BIT_UART1);//
	*uart_downPt++=RdURXH1();
}

void __irq Uart2_RxInt(void)
{

 	rSUBSRCPND = BIT_SUB_RXD2;
    	rSUBSRCPND;
    
    	ClearPending(BIT_UART2);//
	*uart_downPt++=RdURXH2();
}
void __irq Uart3_RxInt(void)
{

 	rSUBSRCPND = BIT_SUB_RXD3;
    	rSUBSRCPND;
    
    	ClearPending(BIT_UART3);//
	*uart_downPt++=RdURXH3();
}
void __irq Uerror(void)
{
	rSUBSRCPND = BIT_SUB_ERR0;
    	rSUBSRCPND;

    	ClearPending(BIT_UART0);
    	uart_err++;
}


void Uart_Isr_Init(int Select)
{
	
    	rINTMOD=0x0;    //All=IRQ mode
    	switch(Select)
    		{
    		case 0:
  //Uart0    	
    	rINTSUBMSK=~(BIT_SUB_RXD0);
    	rINTMSK=~(BIT_UART0);	
    	pISR_UART0=(unsigned)Uart0_RxInt;				
			break;
    		case 1:
		default:
	// Uart 1
    	rINTSUBMSK=~(BIT_SUB_RXD1);
    	rINTMSK=~(BIT_UART1);	
    	pISR_UART1=(unsigned)Uart1_RxInt;			
			break;
    		case 2:
	// Uart 2
    	rINTSUBMSK=~(BIT_SUB_RXD2);
    	rINTMSK=~(BIT_UART2);	
    	pISR_UART2=(unsigned)Uart2_RxInt;					
			break;
    		case 3:
	// Uart 3
	
	
    	rINTSUBMSK=~(BIT_SUB_RXD3);
    	rINTMSK=~(BIT_UART3);	
    	pISR_UART3=(unsigned)Uart3_RxInt;					
			break;
					
			
    		};




	

		
}												  

void Uart_Isr_Dis(int select)
{
	switch(select) {
	case 0:
		// Uart 0
		rINTSUBMSK |= (BIT_SUB_RXD0);
		rINTMSK |= (BIT_UART0);
		break;
	case 2:
		// Uart 2
		rINTSUBMSK |= (BIT_SUB_RXD2);
		rINTMSK |= (BIT_UART2);
		break;

	case 3:
		// Uart 3
		rINTSUBMSK |= (BIT_SUB_RXD3);
		rINTMSK |= (BIT_UART3);
		break;		
	case 1:
	default:
		// Uart 1
		rINTSUBMSK |= (BIT_SUB_RXD1);
		rINTMSK |= (BIT_UART1);	
		break;
	}
}	



void UartMON( void)
{
	int i,j;
	int memError=0;
	unsigned int fileSize;
	unsigned short int checkSum=0,dnCS;
	
    rINTMSK=BIT_ALLMSK;
    MMU_EnableICache();
    //Uart_Isr_Init();

	printf("\n\n+-----------------------------------------------------+\n");
	printf("| Jelly Serial Downloader ver R0.0 2008 June. |\n");
	printf("+-----------------------------------------------------+\n");
	printf("DOWNADDR:%xh ISR_ADDR:%xh \n",UART_DOWNLOAD_ADDRESS,_ISR_STARTADDRESS);
	/*******************************/
	/*      	    Memory test                  */
	/*******************************/
	//MemoryTest();

	printf("\nMemory Test(%xh-%xh):WR",	UART_DOWNLOAD_ADDRESS,(_ISR_STARTADDRESS&0xfff0000));

	for(i=UART_DOWNLOAD_ADDRESS;i<(_ISR_STARTADDRESS&0xfff0000);i+=4)
	{
		*((volatile unsigned *)i)=i;
	}

	printf("\b\bRD");

	for(i=UART_DOWNLOAD_ADDRESS;i<(_ISR_STARTADDRESS&0xfff0000);i+=4)
	{
		j=*((volatile unsigned *)i);

		if(j!=i)
		{
			memError=1;

		}
	}

	if(memError==0)
		printf("\b\bO.K.\n");
	else
		printf("\b\bFAIL\n");

	//SystemCLK(1);
	//Console();
    Delay(1000);



    Uart_Isr_Init(DebugUartCh);
    Delay(1000);	
	
	
	
	

	/*******************************/
	/*   	    Test program download   	     */
	/*******************************/

	printf("Waiting for Download [Addr : 0x%x]\n",UART_DOWNLOAD_ADDRESS);
	uart_downPt=(unsigned char *)UART_DOWNLOAD_ADDRESS;

	i=0 ;
	
	while(( (unsigned int)uart_downPt - UART_DOWNLOAD_ADDRESS ) < 4) {

		Delay(1000);
		if(i++ > 4) {
		
			i = 0;
			printf("\b\b\b\b\b");
		} else {
		
			printf("#");
		}
	}
	do {
	
		printf("\b");
	} while(i-- == 0);

	

	printf("Downloadig size check!\n");

	fileSize=*((unsigned int *)(UART_DOWNLOAD_ADDRESS));


	printf("\nNow, Downloading[FILESIZE:%7d bytes]\n",fileSize);

	j=1000; i=0;

	while(((int)uart_downPt-UART_DOWNLOAD_ADDRESS)<fileSize)
	{
		 if( ((int)uart_downPt-UART_DOWNLOAD_ADDRESS)>=j)
		{
	    	j+=1000;
			printf(".");
		}
	}
    Uart_Isr_Dis(DebugUartCh);
	/*******************************/
	/*    	 Verify check sum      */
	/*******************************/

	for(i=4;i<(fileSize-2);i++)
	{
		checkSum+=*((unsigned char *)(i+UART_DOWNLOAD_ADDRESS));
	}

	dnCS=*((unsigned char *)(UART_DOWNLOAD_ADDRESS+fileSize-2))+
	(*( (unsigned char *)(UART_DOWNLOAD_ADDRESS+fileSize-1) )<<8);

	if(checkSum!=dnCS)
	{
	printf("Checksum Error!!! MEM:%x DN:%x\n",checkSum,dnCS);
	uart_restart();
	}

	printf("\nDownload O.K.\n");
    downloadAddress=UART_DOWNLOAD_ADDRESS;
    downloadsize=fileSize-6;
	for(i=4;i<(fileSize-2);i++) 
	{
	*((unsigned char *)(i+UART_DOWNLOAD_ADDRESS-4))=*((unsigned char *)(i+UART_DOWNLOAD_ADDRESS));
	}
	if(download_run==1)
	{	
	rINTMSK=BIT_ALLMSK;
    uart_run=(void (*)(void))UART_DOWNLOAD_ADDRESS;
	uart_run();
	}

}



void Uartdownload_run( void)
{
	int i,j;
	int memError=0;
	unsigned int fileSize;
	unsigned short int checkSum=0,dnCS;
	Uart_temp_download_addr=tempDownloadAddress;
    rINTMSK=BIT_ALLMSK;
    MMU_EnableICache();
    //Uart_Isr_Init();

	printf("\n\n+-----------------------------------------------------+\n");
	printf("| Jelly Serial Downloader ver R0.0 2008 June. |\n");
	printf("+-----------------------------------------------------+\n");
	printf("DOWNADDR:%xh ISR_ADDR:%xh \n",Uart_temp_download_addr,_ISR_STARTADDRESS);
	/*******************************/
	/*      	    Memory test                  */
	/*******************************/
	//MemoryTest();

	printf("\nMemory Test(%xh-%xh):WR",	Uart_temp_download_addr,(_ISR_STARTADDRESS&0xfff0000));

	for(i=Uart_temp_download_addr;i<(_ISR_STARTADDRESS&0xfff0000);i+=4)
	{
		*((volatile unsigned *)i)=i;
	}

	printf("\b\bRD");

	for(i=Uart_temp_download_addr;i<(_ISR_STARTADDRESS&0xfff0000);i+=4)
	{
		j=*((volatile unsigned *)i);

		if(j!=i)
		{
			memError=1;

		}
	}

	if(memError==0)
		printf("\b\bO.K.\n");
	else
		printf("\b\bFAIL\n");

	//SystemCLK(1);
	//Console();
    Delay(1000);



	Uart_Isr_Init(DebugUartCh);
    Delay(1000);	
	
	
	
	

	/*******************************/
	/*   	    Test program download   	     */
	/*******************************/

	printf("Waiting for Download [Addr : 0x%x]\n",Uart_temp_download_addr);
	uart_downPt=(unsigned char *)Uart_temp_download_addr;

	i=0 ;
	
	while(( (unsigned int)uart_downPt - Uart_temp_download_addr ) < 4) {

		//Delay(1000);
		if(i++ > 4) {
		
			i = 0;
			printf("\b\b\b\b\b");
		} else {
		
			printf("#");
		}
	}
	do {
	
		printf("\b");
	} while(i-- == 0);

	

	printf("Downloadig size check!\n");

	fileSize=*((unsigned int *)(Uart_temp_download_addr));


	printf("\nNow, Downloading[FILESIZE:%7d bytes]\n",fileSize);

	j=1000; i=0;

	while(((int)uart_downPt-Uart_temp_download_addr)<fileSize)
	{
		 if( ((int)uart_downPt-UART_DOWNLOAD_ADDRESS)>=j)
		{
	    	j+=1000;
			printf(".");
		}
	}
    Uart_Isr_Dis(DebugUartCh);
	/*******************************/
	/*    	 Verify check sum      */
	/*******************************/

	for(i=4;i<(fileSize-2);i++)
	{
		checkSum+=*((unsigned char *)(i+Uart_temp_download_addr));
	}

	dnCS=*((unsigned char *)(Uart_temp_download_addr+fileSize-2))+
	(*( (unsigned char *)(Uart_temp_download_addr+fileSize-1) )<<8);

	if(checkSum!=dnCS)
	{
	printf("Checksum Error!!! MEM:%x DN:%x\n",checkSum,dnCS);
	uart_restart();
	}

	printf("\nDownload O.K.\n");
    downloadAddress=Uart_temp_download_addr;
    downloadsize=fileSize-6;
	for(i=4;i<(fileSize-2);i++) 
	{
	*((unsigned char *)(i+Uart_temp_download_addr-4))=*((unsigned char *)(i+Uart_temp_download_addr));
	}
	if(download_run==1)
	{	
	rINTMSK=BIT_ALLMSK;
    uart_run=(void (*)(void))Uart_temp_download_addr;
	uart_run();
	}

}


