/**
* Project Name : Jelly(S3C2443) verification project
*
* Copyright 2008 by Embedded Crazy boys.Inc
* All rights reserved.
*
* Project Description :
* This software is only for verifying functions of the Jelly(S3C2443) . 
* Anybody can use this code.
*/

/**
* File Name    : Uartdown.cpp
* Description  : Jelly(S3C2443)  USB monitor main code
* Author       : http://cafe.naver.com/embeddedcrazyboys
* Dept         : Mobile solution, AP
* Created Date : 2008.07.04
* Version      : 0.0
* History
*   R0.0 (2006.08.): http://cafe.naver.com/embeddedcrazyboys
*/


#include "system.h"

#define		__TEST

#include "2443addr.h"
#include "usbdev.h"
#include <stdio.h>
#include <stdlib.h>
#include "Def.h"



//#define DBG_USBD20

#ifdef DBG_USBD20
    #define Dbg printf
	#define DbgUsb(x) Dbg x
#else
	#define DbgUsb(x) 0;
#endif
#if 1
//test 
#define TEST_PKT_SIZE 53
#define TEST_ARR_SIZE 54

U8 ahwTestPkt [TEST_ARR_SIZE] = {

	0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00,
	0xAA,0x00, 0xAA,0xAA, 0xAA,0xAA, 0xAA,0xAA,
	0xEE,0xAA, 0xEE,0xEE, 0xEE,0xEE, 0xEE,0xEE,
	0xFE,0xEE,	0xFF,0xFF, 0xFF,0xFF, 0xFF,0xFF,
	0xFF,0xFF, 0xFF,0xFF, 0x7F,0xFF, 0xDF,0xBF,
	0xF7,0xEF, 0xFD,0xFB, 0x7E,0xFC, 0xDF,0xBF,
	0xF7,0xEF, 0xFD,0xFB, 0x00,0x7E

};
#endif
extern  U32 downloadAddress;
extern int download_run;
extern volatile U8 dma_done;
extern U32 tempDownloadAddress;

#define USBDEV_BASE 0x49800000


#define Outp32(addr, data) (*(volatile U32 *)(addr) = (data))
#define Outp16(addr, data) (*(volatile U16 *)(addr) = (data))
#define Outp8(addr, data)  (*(volatile U8 *)(addr) = (data))
#define Inp32(addr, data) (data = (*(volatile U32 *)(addr)))
#define Inp16(addr, data) (data = (*(volatile U16 *)(addr)))
#define Inp8(addr, data)  (data = (*(volatile U16 *)(addr)))
#define Input32(addr) (*(volatile U32 *)(addr))
enum USBD20_REGS
{
	INDEX_REG               = (USBDEV_BASE+0x00), // Index register
	EP_INT_REG              = (USBDEV_BASE+0x04), // EP Interrupt pending and clear
	EP_INT_EN_REG           = (USBDEV_BASE+0x08), // EP Interrupt enable
	FUNC_ADDR_REG           = (USBDEV_BASE+0x0c), // Function address
	FRAME_NUM_REG           = (USBDEV_BASE+0x10), // Frame number
	EP_DIR_REG              = (USBDEV_BASE+0x14), // Endpoint direction
	TEST_REG                = (USBDEV_BASE+0x18), // Test register
	SYS_STATUS_REG          = (USBDEV_BASE+0x1c), // System status
	SYS_CON_REG             = (USBDEV_BASE+0x20), // System control
	EP0_STATUS_REG          = (USBDEV_BASE+0x24), // Endpoint 0 status
	EP0_CON_REG             = (USBDEV_BASE+0x28), // Endpoint 0 control
	EP_STATUS_REG           = (USBDEV_BASE+0x2c), // Endpoints status
	EP_CON_REG              = (USBDEV_BASE+0x30), // Endpoints control
	BYTE_READ_CNT_REG       = (USBDEV_BASE+0x34), // read count
	BYTE_WRITE_CNT_REG      = (USBDEV_BASE+0x38), // write count
	MAX_PKT_REG             = (USBDEV_BASE+0x3c), // Max packet size
	DMA_CON_REG             = (USBDEV_BASE+0x40), // DMA control
	DMA_CNT_REG             = (USBDEV_BASE+0x44), // DMA count
	DMA_FIFO_CNT_REG        = (USBDEV_BASE+0x48), // DMA FIFO count
	DMA_TOTAL_CNT1_REG      = (USBDEV_BASE+0x4c), // DMA Total count1
	DMA_TOTAL_CNT2_REG      = (USBDEV_BASE+0x50), // DMA Total count2
	DMA_IF_CON_REG          = (USBDEV_BASE+0x84), // DMA interface Control
	DMA_MEM_BASE_ADDR       = (USBDEV_BASE+0x88), // Mem Base Addr
	DMA_MEM_CURRENT_ADDR    = (USBDEV_BASE+0x8c), // Mem current Addr
	EP0_FIFO                = (USBDEV_BASE+0x60), // Endpoint 0 FIFO
	EP1_FIFO                = (USBDEV_BASE+0x64), // Endpoint 1 FIFO
	EP2_FIFO                = (USBDEV_BASE+0x68), // Endpoint 2 FIFO
	EP3_FIFO                = (USBDEV_BASE+0x6c), // Endpoint 3 FIFO
	EP4_FIFO                = (USBDEV_BASE+0x70), // Endpoint 4 FIFO
	FCON                    = (USBDEV_BASE+0x100) // Burst Fifo Control
 };


// Descriptor Types
enum DESC_TYPE
{
	DEVICE_TYPE = 0x1,
	CONFIGURATION_TYPE,
	STRING_TYPE,
	INTERFACE_TYPE,
	ENDPOINT_TYPE
};

// configuration descriptor: bmAttributes
enum DESC_CONF
{
	CONF_ATTR_DEFAULT       = 0x80, // Spec 1.0 it was BUSPOWERED bit.
	CONF_ATTR_REMOTE_WAKEUP = 0x20,
	CONF_ATTR_SELFPOWERED   = 0x40
};

// endpoint descriptor
enum DESC_ENDPT
{
	EP_ADDR_IN              = 0x80,
	EP_ADDR_OUT             = 0x00,

	EP_ATTR_CONTROL         = 0x0,
	EP_ATTR_ISOCHRONOUS     = 0x1,
	EP_ATTR_BULK            = 0x2,
	EP_ATTR_INTERRUPT       = 0x3
};

enum EP_INDEX
{
	EP0, EP1, EP2, EP3, EP4
};


struct USB_DEVICE_DESCRIPTOR
{
	U8 bLength;
	U8 bDescriptorType;
	U8 bcdUSBL;
	U8 bcdUSBH;
	U8 bDeviceClass;
	U8 bDeviceSubClass;
	U8 bDeviceProtocol;
	U8 bMaxPacketSize0;
	U8 idVendorL;
	U8 idVendorH;
	U8 idProductL;
	U8 idProductH;
	U8 bcdDeviceL;
	U8 bcdDeviceH;
	U8 iManufacturer;
	U8 iProduct;
	U8 iSerialNumber;
	U8 bNumConfigurations;
};

//USB_DEVICE_DESCRIPTOR oDescDevice;


struct USB_CONFIGURATION_DESCRIPTOR
{
	U8 bLength;
	U8 bDescriptorType;
	U8 wTotalLengthL;
	U8 wTotalLengthH;
	U8 bNumInterfaces;
	U8 bConfigurationValue;
	U8 iConfiguration;
	U8 bmAttributes;
	U8 maxPower;
};

struct USB_INTERFACE_DESCRIPTOR
{
	U8 bLength;
	U8 bDescriptorType;
	U8 bInterfaceNumber;
	U8 bAlternateSetting;
	U8 bNumEndpoints;
	U8 bInterfaceClass;
	U8 bInterfaceSubClass;
	U8 bInterfaceProtocol;
	U8 iInterface;
};

struct USB_ENDPOINT_DESCRIPTOR
{
	U8 bLength;
	U8 bDescriptorType;
	U8 bEndpointAddress;
	U8 bmAttributes;
	U8 wMaxPacketSizeL;
	U8 wMaxPacketSizeH;
	U8 bInterval;
};

struct USB_CONFIGURATION_SET
{
	U8 ConfigurationValue;
};

struct USB_GET_STATUS
{
	U8 Device;
	U8 Interface;
	U8 Endpoint0;
	U8 Endpoint1;
	U8 Endpoint3;
};

USB_GET_STATUS oStatusGet;

struct USB_INTERFACE_GET
{
	U8 AlternateSetting;
};

USB_INTERFACE_GET oInterfaceGet;

struct USB_DESCRIPTORS
{
	USB_CONFIGURATION_DESCRIPTOR oDescConfig;
	USB_INTERFACE_DESCRIPTOR oDescInterface;
	USB_ENDPOINT_DESCRIPTOR oDescEndpt1;
	USB_ENDPOINT_DESCRIPTOR oDescEndpt3;
};

//USB_DESCRIPTORS oDesc;

struct DEVICE_REQUEST
{
	U8 bmRequestType;  // Device Request offset 0
	U8 bRequest;       // Device Request offset 1
	U8 wValue_L;       // Device Request offset 2
	U8 wValue_H;       // Device Request offset 3
	U8 wIndex_L;       // Device Request offset 4
	U8 wIndex_H;       // Device Request offset 5
	U8 wLength_L;      // Device Request offset 6
	U8 wLength_H;      // Device Request offset 7
};

//DEVICE_REQUEST oDeviceRequest;


// EP0 CSR register Bits
#define EP0_SENT_STALL              (0x01<<4)
#define EP0_DATA_END                (0x01<<3)
#define EP0_SETUP_END               (0x03<<2)
#define EP0_TX_SUCCESS              (0x01<<1)
#define EP0_RX_SUCCESS              (0x01<<0)

//  Defines for Endpoint CSR Register Bits
#define DMA_TOTAL_COUNT_ZERO        (0x1<<9)
#define SHORT_PKT_RECEIVED          (0x1<<8)
#define EP_FIFO_FLUSH               (0x1<<6)
#define EP_SENT_STALL               (0x1<<5)
#define EP_TX_SUCCESS               (0x1<<1)
#define EP_RX_SUCCESS               (0x1<<0)


#define EP0_STATE_INIT              (0)
#define EP0_STATE_GD_DEV_0          (11)
#define EP0_STATE_GD_DEV_1          (12)
#define EP0_STATE_GD_DEV_2          (13)
#define EP0_STATE_GD_CFG_0          (21)
#define EP0_STATE_GD_CFG_1          (22)
#define EP0_STATE_GD_CFG_2          (23)
#define EP0_STATE_GD_CFG_3          (24)
#define EP0_STATE_GD_CFG_4          (25)
#define EP0_STATE_GD_CFG_ONLY_0     (41)
#define EP0_STATE_GD_CFG_ONLY_1     (42)
#define EP0_STATE_GD_IF_ONLY_0      (44)
#define EP0_STATE_GD_IF_ONLY_1      (45)
#define EP0_STATE_GD_EP0_ONLY_0     (46)
#define EP0_STATE_GD_EP1_ONLY_0     (47)
#define EP0_STATE_GD_EP2_ONLY_0     (48)
#define EP0_STATE_GD_EP3_ONLY_0     (49)
#define EP0_STATE_GD_STR_I0         (30)
#define EP0_STATE_GD_STR_I1         (31)
#define EP0_STATE_GD_STR_I2         (32)
#define EP0_STATE_GD_DEV_QUALIFIER  (33)
#define EP0_INTERFACE_GET           (34)


#define EP0_GET_STATUS0             (35)
#define EP0_GET_STATUS1             (36)
#define EP0_GET_STATUS2             (37)
#define EP0_GET_STATUS3             (38)
#define EP0_GET_STATUS4             (39)

// SPEC1.1
// Standard bmRequestType (Type)
#define STANDARD_TYPE               0x00
#define CLASS_TYPE                  0x20
#define VENDOR_TYPE                 0x40
#define RESERVED_TYPE               0x60

// Standard bmRequestType (Recipient)
// #define DEVICE_bmREQUEST_RECIPIENT(oDeviceRequest)  ((m_poDeviceRequest->bmRequestType) & 0x07)
#define DEVICE_RECIPIENT            0
#define INTERFACE_RECIPIENT         1
#define ENDPOINT_RECIPIENT          2
#define OTHER_RECIPIENT             3

// Standard bRequest codes
#define STANDARD_GET_STATUS         0
#define STANDARD_CLEAR_FEATURE      1
#define STANDARD_RESERVED_1         2
#define STANDARD_SET_FEATURE        3
#define STANDARD_RESERVED_2         4
#define STANDARD_SET_ADDRESS        5
#define STANDARD_GET_DESCRIPTOR     6
#define STANDARD_SET_DESCRIPTOR     7
#define STANDARD_GET_CONFIGURATION  8
#define STANDARD_SET_CONFIGURATION  9
#define STANDARD_GET_INTERFACE      10
#define STANDARD_SET_INTERFACE      11
#define STANDARD_SYNCH_FRAME        12

// Descriptor types
#define DEVICE_DESCRIPTOR           1
#define CONFIGURATION_DESCRIPTOR    2
#define STRING_DESCRIPTOR           3
#define INTERFACE_DESCRIPTOR        4
#define ENDPOINT_DESCRIPTOR         5
#define DEVICE_QUALIFIER            6
#define OTHER_SPEED_CONFIGURATION   7

// string descriptor
#define LANGID_US_L                 (0x09)
#define LANGID_US_H                 (0x04)

// USB Endpoints states
#define EP0_STATE_IDLE              (0)
#define EP0_STATE_TRANSFER          (1)
#define EP0_STATE_RECEIVER          (2)

#define BULK_OUT_STATUS_NOSTALL     (0x0000)
#define BULK_OUT_STATUS_STALL       (0x0001)

#define DEVICE_STATUS_DEFAULT       (0x0000)
#define DEVICE_STATUS_SELFPOWERED   (0x0001)
#define DEVICE_STATUS_REMOTEWAKEUP  (0x0002)


#define DEVICE_DESC_SIZE            18
#define STRING_DESC0_SIZE           4
#define STRING_DESC1_SIZE           22
#define STRING_DESC2_SIZE           44
#define CONFIG_DESC_TOTAL_SIZE      32
#define CONFIG_DESC_SIZE            9
#define INTERFACE_DESC_SIZE         9
#define ENDPOINT_DESC_SIZE          7
#define DEVICE_QUALIFIER_SIZE       10
#define OTHER_SPEED_CONFIGURATION_SIZE 9

// INT_REG status value
#define INT_ERR                     (0xff80)
#define INT_REG_ERROR               (0xff1<<6)
#define INT_REG_VBUS                (0x1<<8)
#define INT_REG_VBUS_CLEAR          (0x1<<6)
#define INT_REG_HSP                 (0x1<<4)
#define INT_REG_SDE                 (0x1<<3)
#define INT_REG_RESET               (0x1)
#define INT_REG_RESUME              (0x1<<2)
#define INT_REG_SUSPEND             (0x1<<1)
#define INT_REG_EP4                 (0x1<<4)
#define INT_REG_EP3                 (0x1<<3)
#define INT_REG_EP2                 (0x1<<2)
#define INT_REG_EP1                 (0x1<<1)
#define INT_REG_EP0                 (0x1)
#define INT_DTB_MISMATCH            (0x1FF<<7)

// USB Dma Operation
#define DMA_AUTO_RX_DISABLE         (0x1<<5)
#define DMA_FLY_ENABLE              (0x1<<4)
#define DMA_FLY_DISABLE             (0x0<<4)
#define DMA_DEMEND_ENABLE           (0x1<<3)
#define DMA_DEMEND_DISABLE          (0x0<<3)
#define DMA_TX_START                (0x1<<2)
#define DMA_TX_STOP                 (0x0<<2)
#define DMA_RX_START                (0x1<<1)
#define DMA_RX_STOP                 (0x0<<1)
#define USB_DMA_MODE                (0x1<<0)
#define USB_INT_MODE                (0x0<<0)

#define MAX_BURST_INCR16            (0x3<<0)
#define MAX_BURST_INCR8             (0x2<<0)
#define MAX_BURST_INCR4             (0x1<<0)

#define DMA_ENABLE                  (0x1<<8)
#define DMA_DISABLE                 (0x0<<8)


// Feature Selectors
	#define EP_STALL          		0
	#define DEVICE_REMOTE_WAKEUP    1
	#define TEST_MODE				2

	/* Test Mode Selector*/
	#define TEST_J					1
	#define TEST_K					2
	#define TEST_SE0_NAK			3
	#define TEST_PACKET				4
	#define TEST_FORCE_ENABLE		5

	#define		TR_RSVD					0x4F20
	#define		TR_VBUS					0x8000
	#define		TR_EUERR				0x2000
	#define		TR_PERR					0x1000
	#define		TR_SPDSEL				0x00C0
	#define		TR_TMD					0x0010
	#define		TR_TPS					0x0008
	#define		TR_TKS					0x0004
	#define		TR_TJS					0x0002
	#define		TR_TSNS					0x0001


/*********************************************************************
						DEVICE QUALIFIER
**********************************************************************/
const U8 aDeviceQualifierDescriptor[] =
{
	0x0a,                   //  0 desc size
	0x06,                   //  1 desc type (DEVICE)
	0x00,                   //  2 USB release
	0x02,                   //  3 => 2.00
	0x00,                   //  4 class
	0x00,                   //  5 subclass
	0x00,                   //  6 protocol
	64,          			//  7 max pack size
	0x01,                   //  8 number of other-speed configuration
	0x00,                   //  9 reserved
};

const U8 aDescStr0[]=
{
	4, STRING_DESCRIPTOR, LANGID_US_L, LANGID_US_H, // codes representing languages
};

const U8 aDescStr1[]= // Manufacturer
{
	(0x14+2), STRING_TYPE,
	'S', 0x0, 'y', 0x0, 's', 0x0, 't', 0x0, 'e', 0x0, 'm', 0x0, ' ', 0x0, 'M', 0x0,
	'C', 0x0, 'U', 0x0,
};

const U8 aDescStr2[]= // Product
{
	(0x2a+2), STRING_TYPE,
	'S', 0x0, 'E', 0x0, 'C', 0x0, ' ', 0x0, 'S', 0x0, '3', 0x0, 'C', 0x0, '2', 0x0,
	'4', 0x0, '4', 0x0, '3', 0x0, 'X', 0x0, ' ', 0x0, 'T', 0x0, 'e', 0x0, 's', 0x0,
	't', 0x0, ' ', 0x0, 'B', 0x0, '/', 0x0, 'D', 0x0
};


void WrPktEp0(U8 *buf, int num)
{
	int i;
	U16 Wr_Data=0;

	if (num&0x1) num++;
	for(i=0;i<num;i+=2)
	{
		Wr_Data=((*(buf+1))<<8)|*buf;
		Outp32(EP0_FIFO, Wr_Data);
		buf +=2;
	}
}

void WrPktEp1(U8 *buf, int num)
{
	int i;
	U16 Wr_Data=0;

	if (num&0x1) num++;
	for(i=0;i<num;i+=2)
	{
		Wr_Data=((*(buf+1))<<8)|*buf;
	   Outp32(EP1_FIFO, Wr_Data);
	   buf +=2;
	}
}

void PrintEp0Pkt(U8 *pt, U8 count)
{
	int i;
	DbgUsb(("[DBG:"));
	for(i=0;i<count;i++)
		DbgUsb(("%x,", pt[i]));
	DbgUsb(("]"));
}

#ifdef		__TEST
void USBDEV::RdPktEp3(U16 *buf, int num)
{
	int i, cnt=(num>>1);
	U16 Rdata;

	for(i=0;i<cnt;i++)
	{
		Inp32(EP3_FIFO, Rdata);
		buf[i] = (U16)Rdata;		
	}
	m_pDownPt += num;
}
#else
void USBDEV::RdPktEp3(U8 *buf, int num)
{
	int i;
	U16 Rdata;

	for(i=0;i<num;i+=2)
	{
		Inp32(EP3_FIFO, Rdata);
		buf[i] = (U8)Rdata;
		buf[i+1] = (U8)(Rdata>>8);
	}

	m_pDownPt += num;
}
#endif

bool USBDEV::IsEnumerationDone(void)
{
	if (m_uEnumerationDone == 0)
		return false;
	else
		return true;
}

void USBDEV::PrepareEp1Fifo(U32 BaseAddr)
{
	int i;

	U8* BulkInBuf = (U8*)BaseAddr;

	if (m_uBulkInCount > m_uEp1MaxPktSize)
	{
		Outp32(INDEX_REG, EP1);
		Outp32(BYTE_WRITE_CNT_REG, m_uEp1MaxPktSize);
		WrPktEp1(BulkInBuf, m_uEp1MaxPktSize);

		m_uBulkInAddr = BaseAddr + m_uEp1MaxPktSize;
		m_uBulkInCount -= m_uEp1MaxPktSize;
	}
	else
	{
		Outp32(INDEX_REG, EP1);
		Outp32(BYTE_WRITE_CNT_REG, m_uBulkInCount);
		WrPktEp1(BulkInBuf, m_uBulkInCount);

		m_uBulkInAddr = BaseAddr + m_uBulkInCount;
		m_uBulkInCount = 0;
	}


}

void USBDEV::FlushEp1Fifo(void)
{
	U32 OrgValue;
	Outp32(INDEX_REG, EP1);
	Inp32(EP_CON_REG, OrgValue);
	Outp32(EP_CON_REG, (1<<6));
	Outp32(EP_CON_REG, OrgValue);
}

void USBDEV::SetMaxPktSizes(USB_SPEED eSpeed)
{
	if (eSpeed == USB_HIGH)
	{
		m_eSpeed = USB_HIGH;
		m_uEp0MaxPktSize = 64;
		m_uEp1MaxPktSize = 256;
		m_uEp3MaxPktSize = 256;
//		m_uEp1MaxPktSize = 512;
		//m_uEp3MaxPktSize = 64;
		//m_uEp3MaxPktSize = 512;
	}
	else
	{
		m_eSpeed = USB_FULL;
		m_uEp0MaxPktSize = 8;
		m_uEp1MaxPktSize = 64;
		m_uEp3MaxPktSize = 64;
	}
	// EP0 Max Packet size settings
	Outp32(INDEX_REG, EP0);
	Outp32(MAX_PKT_REG, m_uEp0MaxPktSize); 	// max packet size

	// EP1 OUT Max Packet size settings
	Outp32(INDEX_REG, EP1);
	Outp32(MAX_PKT_REG, m_uEp1MaxPktSize); 	// max packet size

	// EP2 IN Max Packet size settings
	Outp32(INDEX_REG, EP3);
	Outp32(MAX_PKT_REG, m_uEp3MaxPktSize);	// max packet size
}

void USBDEV::SetOpMode(USB_OP mode)
{
	U32 i;
//	m_uTransferDone = 0;
	m_uDownloadFileSize = 0;
	m_eOpMode = mode;
}

bool USBDEV::VerifyChecksum(void)
{
	U8* CalcCSPt;
	U16 dnCS;
	U16 checkSum;
    U16 i=0;
    
	// checksum calculation
	CalcCSPt = (U8*)m_uDownloadAddress;
	checkSum = 0;
//	printf("Now, Checksum calculation:%9d",0);
	while((U32)CalcCSPt < (m_uDownloadAddress+(m_uDownloadFileSize-8)))
	{
		checkSum += *CalcCSPt++;
//		if((U16)(i*100/(m_uDownloadFileSize-8))%5==0)printf("\b\b\b\b\b\b\b\b%8d",i*100/(m_uDownloadFileSize-8));
		i++;
	}
//	printf("\b\b\b\b\b\b\b\b%8d",100);
	// checkSum was calculated including dnCS. So, dnCS should be subtracted.
	checkSum=checkSum - *((unsigned char *)(m_uDownloadAddress+m_uDownloadFileSize-8-2))
		- *( (unsigned char *)(m_uDownloadAddress+m_uDownloadFileSize-8-1) );

	dnCS=*((unsigned char *)(m_uDownloadAddress+m_uDownloadFileSize-8-2))+
		(*( (unsigned char *)(m_uDownloadAddress+m_uDownloadFileSize-8-1) )<<8);

	if (checkSum!=dnCS)
	{
		printf("Checksum Error!!! MEM:%x DN:%x\n", checkSum, dnCS);
		return false;
	}
	else
	{
		//Disp("\nDownload Check Sum Result is O.K.\n\n");
		return true;
	}

}

USBDEV::USBDEV()
{
	m_poDescDevice = (USB_DEVICE_DESCRIPTOR*)malloc(sizeof (USB_DEVICE_DESCRIPTOR));
	m_poStatusGet = (USB_GET_STATUS*)malloc(sizeof (USB_GET_STATUS));
	m_poInterfaceGet = (USB_INTERFACE_GET*)malloc(sizeof (USB_INTERFACE_GET));
	m_poDesc = (USB_DESCRIPTORS*)malloc(sizeof (USB_DESCRIPTORS));
	m_poDeviceRequest = (DEVICE_REQUEST*)malloc(sizeof (DEVICE_REQUEST));
}

USBDEV::~USBDEV()
{
	if (m_poDescDevice)
		free(m_poDescDevice);
	if (m_poStatusGet)
		free(m_poStatusGet);
	if (m_poInterfaceGet)
		free(m_poInterfaceGet);
	if (m_poDesc)
		free(m_poDesc);
	if (m_poDeviceRequest)
		free(m_poDeviceRequest);
}


void USBDEV::Init(void)
{
	m_uEnumerationDone = 0;
// 	SetDescriptorTable();
	SetEndpoint();

	m_uEp0State = EP0_STATE_INIT;
	m_uEp0SubState = 0;
}

void USBDEV::SetDescriptorTable(void)
{
	// Standard device descriptor
	m_poDescDevice->bLength=0x12;	// EP0_DEV_DESC_SIZE=0x12 bytes
	m_poDescDevice->bDescriptorType=DEVICE_TYPE;
	m_poDescDevice->bDeviceClass=0xFF; // 0x0
	m_poDescDevice->bDeviceSubClass=0x0;
	m_poDescDevice->bDeviceProtocol=0x0;
	m_poDescDevice->bMaxPacketSize0=m_uEp0MaxPktSize;

	m_poDescDevice->idVendorL=0xe8;     //changed the value 06.10.16
	m_poDescDevice->idVendorH=0x04;     //changed the value 06.10.16

//	m_poDescDevice->idVendorL=0x45;
//	m_poDescDevice->idVendorH=0x53;
	m_poDescDevice->idProductL=0x34;
	m_poDescDevice->idProductH=0x12;

	m_poDescDevice->bcdDeviceL=0x00;
	m_poDescDevice->bcdDeviceH=0x01;
	m_poDescDevice->iManufacturer=0x1; // index of string descriptor
	m_poDescDevice->iProduct=0x2;	// index of string descriptor
	m_poDescDevice->iSerialNumber=0x0;
	m_poDescDevice->bNumConfigurations=0x1;
	if (m_eSpeed == USB_FULL) {
		m_poDescDevice->bcdUSBL=0x10;
		m_poDescDevice->bcdUSBH=0x01; 	// Ver 1.10
	}
	else {
		m_poDescDevice->bcdUSBL=0x00;
		m_poDescDevice->bcdUSBH=0x02; 	// Ver 2.0
	}

	// Standard configuration descriptor
	m_poDesc->oDescConfig.bLength=0x9;
	m_poDesc->oDescConfig.bDescriptorType=CONFIGURATION_TYPE;
	m_poDesc->oDescConfig.wTotalLengthL=0x20; // <cfg desc>+<if desc>+<endp0 desc>+<endp1 desc>
	m_poDesc->oDescConfig.wTotalLengthH=0;
	m_poDesc->oDescConfig.bNumInterfaces=1;
// dbg    descConf.bConfigurationValue=2; // why 2? There's no reason.
	m_poDesc->oDescConfig.bConfigurationValue=1;
	m_poDesc->oDescConfig.iConfiguration=0;
	m_poDesc->oDescConfig.bmAttributes=CONF_ATTR_SELFPOWERED;//CONF_ATTR_DEFAULT; // bus powered only.
	m_poDesc->oDescConfig.maxPower=25; // draws 50mA current from the USB bus.

	// Standard interface descriptor
	m_poDesc->oDescInterface.bLength=0x9;
	m_poDesc->oDescInterface.bDescriptorType=INTERFACE_TYPE;
	m_poDesc->oDescInterface.bInterfaceNumber=0x0;
	m_poDesc->oDescInterface.bAlternateSetting=0x0; // ?
	m_poDesc->oDescInterface.bNumEndpoints=2;	// # of endpoints except EP0
	m_poDesc->oDescInterface.bInterfaceClass=0xff; // 0x0 ?
	m_poDesc->oDescInterface.bInterfaceSubClass=0x0;
	m_poDesc->oDescInterface.bInterfaceProtocol=0x0;
	m_poDesc->oDescInterface.iInterface=0x0;

	// Standard endpoint0 descriptor
	m_poDesc->oDescEndpt1.bLength=0x7;
	m_poDesc->oDescEndpt1.bDescriptorType=ENDPOINT_TYPE;
	m_poDesc->oDescEndpt1.bEndpointAddress=1|EP_ADDR_IN; // 2400Xendpoint 1 is IN endpoint.
	m_poDesc->oDescEndpt1.bmAttributes=EP_ATTR_BULK;
	m_poDesc->oDescEndpt1.wMaxPacketSizeL=(U8)m_uEp1MaxPktSize; // 64
	m_poDesc->oDescEndpt1.wMaxPacketSizeH=(U8)(m_uEp1MaxPktSize>>8);
	m_poDesc->oDescEndpt1.bInterval=0x0; // not used

	// Standard endpoint1 descriptor
	m_poDesc->oDescEndpt3.bLength=0x7;
	m_poDesc->oDescEndpt3.bDescriptorType=ENDPOINT_TYPE;
	m_poDesc->oDescEndpt3.bEndpointAddress=3|EP_ADDR_OUT; // 2400X endpoint 3 is OUT endpoint.
	m_poDesc->oDescEndpt3.bmAttributes=EP_ATTR_BULK;
	m_poDesc->oDescEndpt3.wMaxPacketSizeL=(U8)m_uEp3MaxPktSize; // 64
	m_poDesc->oDescEndpt3.wMaxPacketSizeH=(U8)(m_uEp3MaxPktSize>>8);
	m_poDesc->oDescEndpt3.bInterval=0x0; // not used
}

void USBDEV::SetEndpoint(void)
{
	// *** End point information ***
	// EP0: control
	U16 SysStatus;
	U16 Temp;

	Outp32(INDEX_REG, EP0);
	// For L18
	Outp32(EP_DIR_REG, 0x02); 		// EP1=> TX, EP2=>RX , 0b=report mode[1], 15b=report mode[2], 3b~8b=ep2 delay_con
	Outp32(EP_INT_EN_REG, 0x4d0f); 	// EP0, 1, 2 Interrupt enable, 15b=report mode[0], 3b~14b=ep0/1 delay_con
	Inp32(EP_DIR_REG, Temp);
	DbgUsb(("EP_DIR_REG : %x \n", Temp));
	Inp32(EP_INT_EN_REG, Temp);
	DbgUsb(("EP_INT_EN_REG : %x \n", Temp));

	Outp32(TEST_REG, 0x0000);

	//Outp32(SYS_CON_REG, 0x0283);		// error interrupt enable, 16bit bus, Little format, suspend&reset enable
	//Outp32(SYS_CON_REG, 0x0003);		// error interrupt enable, 8bit bus, Little format, suspend&reset enable
	//Outp32(SYS_CON_REG, 0x0023);		// error interrupt enable, 16bit bus, Little format, suspend&reset enable
	Outp32(SYS_CON_REG, 0x4123);		// error interrupt enable, 16bit bus, Little format, suspend&reset enable
//	Outp32(SYS_CON_REG, 0x4127);		// error interrupt enable, 16bit bus, Little format, suspend&reset enable
// 	Outp32(MAX_PKT_REG, MAXP_SIZE_64BYTE); // Initial size?
	Outp32(EP0_CON_REG, 0x0000);


	// EP1 OUT Max Packet size settings
	Outp32(INDEX_REG, EP1);
// 	Outp32(MAX_PKT_REG, 512); 	// max packet size 512 bytes
	Outp32(EP_CON_REG, 0x0080); // dual enable

	// EP2 IN Max Packet size settings
	Outp32(INDEX_REG, EP3);
// 	Outp32(MAX_PKT_REG, 512);	// max packet size 512 bytes
	Outp32(EP_CON_REG, 0x0080);    		// dual enable

	Outp32(INDEX_REG, EP0);


}


extern int usbConnected;	// by edchoi
void USBDEV::HandleEvent(void)
{
	U32 uStatus;
	U32 Temp;

	Inp32(SYS_STATUS_REG, uStatus); // System status read
	//printf("SYS_STATUS_REG : %x \n", uStatus); // by edchoi
	DbgUsb(("SYS_STATUS_REG : %x \n", uStatus));

	if (uStatus & INT_REG_VBUS)
	{
		Outp32(SYS_STATUS_REG, INT_REG_VBUS); // Interrupt Clear
		DbgUsb(("\n [USB_Diag_Log]  :  INT_REG_VBUS\n"));
//		printf("\n [USB_Diag_Log]  :  INT_REG_VBUS\n");
	}

	if (uStatus & 0xfc00) 	 // Error interrupt check
	{
		
		Inp32(SYS_STATUS_REG , Temp);
//		printf("SYS_CONTROL_REG  :%x  \n",Temp);
		Outp32(SYS_STATUS_REG, INT_ERR); // Interrupt Clear
		DbgUsb(("\n [USB_Diag_Log]  :  Error_INT\n"));
		
	}

	// Which USB interrupts happen
	if (uStatus & INT_REG_SUSPEND)
	{
		Outp32(SYS_STATUS_REG, INT_REG_SUSPEND); // Interrupt Clear
		DbgUsb(("\n [USB_Diag_Log]  : Suspend Mode"));
		printf("\n [USB_Diag_Log]  : Suspend Mode");

			//if (!usbConnected) // by edchoi
			//{
			//	rGPBCON |= 0x1;
			//	rGPBDAT &= ~0x1;
			//}		
			rMISCCR|=(1<<12);	
			rUSB_PHYPWR = (0x0<<31)|(0x1<<4)|(0x0<<3)|(0x0<<2)|(0x0<<1)|(0x0<<0);

	}

	if (uStatus & INT_REG_RESUME)
	{
		Outp32(SYS_STATUS_REG, INT_REG_RESUME); // Host software send ClearPortFeature. Interrupt Clear
		DbgUsb(("\n [USB_Diag_Log]  : Resume Mode \n"));
		
		printf("\n [USB_Diag_Log]  : Resume Mode \n");
		printf("\n GPG10 =%d (GPG10>>20)=%d\n",rGPGDAT,(rGPGDAT>>20));
		rMISCCR=rMISCCR&~(1<<12);  // USBD is 0 ,normal mode ,1 is suspend mode /
		rUSB_PHYPWR = (0x0<<31)|(0x3<<4)|(0x0<<3)|(0x0<<2)|(0x0<<1)|(0x0<<0); 
    //48Mhz clock on ,PHY2.0 analog block power on,XO block power on,XO block power in suspend mode,PHY 2.0 Pll power on ,suspend signal for save mode disable 
	}

	if (uStatus & INT_REG_RESET) // Reset interrupt
	{
		Outp32(SYS_STATUS_REG, INT_REG_RESET); // Interrupt Clear
		SetEndpoint();
		m_uEp0State = EP0_STATE_INIT;
		DbgUsb(("\n [USB_Diag_Log]  : Reset Mode \n"));
//		printf("\n [USB_Diag_Log]  : Reset Mode \n");
	}

	if (uStatus & INT_REG_SDE) // Device Speed Detection interrupt
	{
		Outp32(SYS_STATUS_REG, INT_REG_SDE); // Interrupt Clear
		DbgUsb(("\n [USB_Diag_Log]  : Speed Detection interrupt \n"));

		if (uStatus & INT_REG_HSP) // Set if Device is High speed or Full speed
		{
			Outp32(SYS_STATUS_REG, INT_REG_HSP); // High Speed Device Interrupt Clear?? may be not.
			DbgUsb(("\n [USB_Diag_Log]  : High Speed Detection\n"));
			SetMaxPktSizes(USB_HIGH);
			SetDescriptorTable();
		}
		else
		{
			SetMaxPktSizes(USB_FULL);
			SetDescriptorTable();
		}
	}

	U16 ep_int_status, ep_int;

	Inp32(EP_STATUS_REG, ep_int_status); // EP interrrupt status read
	DbgUsb(("EP_STATUS_REG : %x \n", ep_int_status));
	Inp32(EP_INT_REG, ep_int);
	DbgUsb(("EP_INT_REG : %x \n", ep_int));


	if (ep_int & INT_REG_EP0)
	{
// 		DbgUsb(("\n [USB_Diag_Log]  :  Control Transfer Interrupt \n"));
		Outp32(EP_INT_REG, INT_REG_EP0); // Interrupt Clear
		HandleEvent_EP0();
	}

	// Endpoint1 bulkIn
	else if (ep_int & INT_REG_EP1)
	{
		Outp32(EP_INT_REG, INT_REG_EP1); // Interrupt Clear
	//	printf("\n [USB_Diag_Log]  :  Ep1 Interrupt  \n");
		DbgUsb(("\n [USB_Diag_Log]  :  Ep1 Interrupt  \n"));
		HandleEvent_BulkIn();
	}

	// Endpoint3 bulkOut
	else if (ep_int & INT_REG_EP3)
	{
		DbgUsb(("\n [USB_Diag_Log]  :  Bulk Out Transfer Interrupt  \n"));
		Outp32(EP_INT_REG, INT_REG_EP3); // Interrupt Clear
	//	printf("*");
//		printf("\n [USB_Diag_Log]  :  Bulk Out Transfer Interrupt  \n");
		HandleEvent_BulkOut();
	}

}

// ======================================================================
// endpointZeroFunction()
//
// This is function for Control Transfer.
// If it occurs endpoint0 interrupt, this function is called.
// This function check Device Request for Control Transfer type and
// call each other functions.
// ======================================================================

void USBDEV::HandleEvent_EP0(void)
{
	U32 DeviceRequestLength;
	U16 ep0csr;
	U16 ReadCnt, i;
	U16 ReadBuf[64]={0x0000, };
	U8  setaddress;
	U32  uRemoteWakeUp;
	U16  usConfig;

	Outp32(INDEX_REG, EP0);
	Inp32(EP0_STATUS_REG, ep0csr);

	DbgUsb((" Endpoint0 CSR Register = %x \n", ep0csr));
	// EP0 CSR register status check

	if (ep0csr & EP0_SENT_STALL) // SENT STALL : protocol stall.
	{
		DbgUsb((" Sent Stall \n"));
		Outp32(EP0_STATUS_REG, EP0_SENT_STALL);
		if (ep0csr & EP0_RX_SUCCESS)
			Outp32(EP0_STATUS_REG, EP0_RX_SUCCESS);
		m_uEp0State = EP0_STATE_INIT;
		return;
	}

	if (ep0csr & EP0_TX_SUCCESS)
	{
		DbgUsb((" EP0_TX_SUCCESS \n"));
		Outp32(EP0_STATUS_REG, EP0_TX_SUCCESS);
	}

	DbgUsb((" m_uEp0State = %x \n", m_uEp0State));

// 	if ((ep0csr & EP0_RX_SUCCESS) & (m_uEp0State == EP0_STATE_INIT))
	if (m_uEp0State == EP0_STATE_INIT)
	{
		Inp32(BYTE_READ_CNT_REG, ReadCnt);
		for(i=0;i<4;i++) Inp32(EP0_FIFO, ReadBuf[i]);

		Outp32(EP0_STATUS_REG, EP0_RX_SUCCESS);

		m_poDeviceRequest->bmRequestType=ReadBuf[0];
		m_poDeviceRequest->bRequest=ReadBuf[0]>>8;
		m_poDeviceRequest->wValue_L=ReadBuf[1];
		m_poDeviceRequest->wValue_H=ReadBuf[1]>>8;
		m_poDeviceRequest->wIndex_L=ReadBuf[2];
		m_poDeviceRequest->wIndex_H=ReadBuf[2]>>8;
		m_poDeviceRequest->wLength_L=ReadBuf[3];
		m_poDeviceRequest->wLength_H=ReadBuf[3]>>8;

		PrintEp0Pkt((U8 *)m_poDeviceRequest, 8);

		switch (m_poDeviceRequest->bRequest)
		{
			case STANDARD_SET_ADDRESS:
				setaddress = (m_poDeviceRequest->wValue_L); // Set Address Update bit
				DbgUsb(("Func_ADDR_Setaddr : %d \n", setaddress));
				m_uEp0State = EP0_STATE_INIT;
				break;

			case STANDARD_SET_DESCRIPTOR:
				DbgUsb(("\n MCU >> Set Descriptor \n"));
				break;

			case STANDARD_SET_CONFIGURATION:
				DbgUsb(("\n MCU >> Set Configuration \n"));
				usConfig = m_poDeviceRequest->wValue_L; // Configuration value in configuration descriptor
				m_uEnumerationDone = 1;
				break;

			case STANDARD_GET_CONFIGURATION:
				// Uart_Printf("\n MCU >> Get Configruation \n");
				Outp32(BYTE_WRITE_CNT_REG, 1);
				Outp32(EP0_FIFO, usConfig);
				break;

			case STANDARD_GET_DESCRIPTOR:
				switch (m_poDeviceRequest->wValue_H)
				{
					case DEVICE_DESCRIPTOR:
						DbgUsb(("\n MCU >> Get Device Descriptor \n"));
						m_uEp0State = EP0_STATE_GD_DEV_0;
						break;

					case CONFIGURATION_DESCRIPTOR:
						DbgUsb(("\n MCU >> Get Configuration Descriptor \n"));

						DeviceRequestLength = (U32)((m_poDeviceRequest->wLength_H << 8) |
							m_poDeviceRequest->wLength_L);

						if (DeviceRequestLength > CONFIG_DESC_SIZE){
						// === GET_DESCRIPTOR:CONFIGURATION+INTERFACE+ENDPOINT0+ENDPOINT1 ===
						// Windows98 gets these 4 descriptors all together by issuing only a request.
						// Windows2000 gets each descriptor seperately.
						// m_uEpZeroTransferLength = CONFIG_DESC_TOTAL_SIZE;
// 							m_uEpZeroTransferdata = 0;
							m_uEp0State = EP0_STATE_GD_CFG_0;
						}
						else
							m_uEp0State = EP0_STATE_GD_CFG_ONLY_0; // for win2k
						break;

					case STRING_DESCRIPTOR :
						switch(m_poDeviceRequest->wValue_L)
						{
							case 0:
								m_uEp0State = EP0_STATE_GD_STR_I0;
								break;
							case 1:
								m_uEp0State = EP0_STATE_GD_STR_I1;
								break;
							case 2:
								m_uEp0State = EP0_STATE_GD_STR_I2;
								break;
							default:
									break;
						}
						break;
					case ENDPOINT_DESCRIPTOR:
						switch(m_poDeviceRequest->wValue_L&0xf)
						{
						case 0:
							m_uEp0State=EP0_STATE_GD_EP0_ONLY_0;
							break;
						case 1:
							m_uEp0State=EP0_STATE_GD_EP1_ONLY_0;
							break;
						default:
							break;
						}
						break;

					case DEVICE_QUALIFIER:
						DbgUsb(("\n MCU >> Get Device Qualifier Descriptor \n"));
						m_uEp0State = EP0_STATE_GD_DEV_QUALIFIER;
						break;
				}
				break;

			case STANDARD_CLEAR_FEATURE:
				DbgUsb(("\n MCU >> Clear Feature \n"));
				switch (m_poDeviceRequest->bmRequestType)
				{
					case DEVICE_RECIPIENT:
						if (m_poDeviceRequest->wValue_L == 1)
							uRemoteWakeUp = false;
						break;

					case ENDPOINT_RECIPIENT:
						if (m_poDeviceRequest->wValue_L == 0)
						{
							if ((m_poDeviceRequest->wIndex_L & 0x7f) == 0x00)
								m_poStatusGet->Endpoint0= 0;

							if ((m_poDeviceRequest->wIndex_L & 0x8f) == 0x01) // IN  Endpoint 1
								m_poStatusGet->Endpoint1= 0;

							if ((m_poDeviceRequest->wIndex_L & 0x8f) == 0x03) // OUT Endpoint 3
								m_poStatusGet->Endpoint3= 0;
						}
						break;

					default:
						break;
				}
				m_uEp0State = EP0_STATE_INIT;
				break;

			case STANDARD_SET_FEATURE:

				
				DbgUsb(("\n MCU >> Set Feature \n"));
				switch (m_poDeviceRequest->bmRequestType)
				{
					case DEVICE_RECIPIENT:
						if (m_poDeviceRequest->wValue_L == 1)
							uRemoteWakeUp = true;
							break;

					case ENDPOINT_RECIPIENT:
						if (m_poDeviceRequest->wValue_L == 0)
						{
							if ((m_poDeviceRequest->wIndex_L & 0x7f) == 0x00)
								m_poStatusGet->Endpoint0= 1;

							if ((m_poDeviceRequest->wIndex_L & 0x8f) == 0x01)
								m_poStatusGet->Endpoint1= 1;

							if ((m_poDeviceRequest->wIndex_L & 0x8f) == 0x03)
								m_poStatusGet->Endpoint3= 1;
						}
						break;

					default:
						break;
				}
				//=======================================================
				
				switch (m_poDeviceRequest->wValue_L) {

					case EP_STALL:

						// TBD: addnl processing if reqd
						break;

					case TEST_MODE:
						U32 tmp;
						if ((0 != m_poDeviceRequest->wIndex_L ) ||(0 != m_poDeviceRequest->bmRequestType)) 
						break;
						
						// Set TEST MODE
						//output32rTR |= TR_TMD;
							 	Inp32(TEST_REG,tmp);
								tmp|=TR_TMD;
						
						Outp32(TEST_REG,TR_TMD);
						
						switch( m_poDeviceRequest->wIndex_H)
						{
						
							case TEST_J:

								//Set Test J
								//rTR |= TR_TJS;
								Inp32(TEST_REG,tmp);
								tmp|=TR_TJS;
								Outp32(TEST_REG,tmp);
								break;

							case TEST_K:

								//Set Test K
								//rTR |= TR_TKS;
								Inp32(TEST_REG,tmp);
								tmp|=TR_TKS;
								Outp32(TEST_REG,tmp);
													
								break;

							case TEST_SE0_NAK:

								//Set Test SE0NAK
								//rTR |= TR_TSNS;
								Inp32(TEST_REG,tmp);
								tmp|=TR_TSNS;
								Outp32(TEST_REG,tmp);
								
								break;

							case TEST_PACKET:
								printf ("Test_packet\n");
								Outp32(BYTE_WRITE_CNT_REG, 54);
								WrPktEp0(ahwTestPkt, TEST_PKT_SIZE);
							//	EpxSendData (ahwTestPkt, TEST_PKT_SIZE, EP0);

							 	//rTR |= TR_TPS;
							 	Inp32(TEST_REG,tmp);
								tmp|=TR_TPS;
								Outp32(TEST_REG,tmp);
								printf ("tr=%0X\n", tmp);					 	
							 	

								break;
							}

						break;

					default:
						break;
				}
				//=======================================================
				
				m_uEp0State = EP0_STATE_INIT;
				break;

			case STANDARD_GET_STATUS:
				switch(m_poDeviceRequest->bmRequestType)
				{
					case  (0x80):
						m_poStatusGet->Device=((U8)uRemoteWakeUp<<1)|0x1;		// SelfPowered
						m_uEp0State = EP0_GET_STATUS0;
						break;

					case  (0x81):
						m_poStatusGet->Interface=0;
						m_uEp0State = EP0_GET_STATUS1;
						break;

					case  (0x82):
						if ((m_poDeviceRequest->wIndex_L & 0x7f) == 0x00)
							m_uEp0State = EP0_GET_STATUS2;

						if ((m_poDeviceRequest->wIndex_L & 0x8f) == 0x01)
							m_uEp0State = EP0_GET_STATUS3;

						if ((m_poDeviceRequest->wIndex_L & 0x8f) == 0x03)
							m_uEp0State = EP0_GET_STATUS4;
						break;

					default:
						break;
				}
				break;

			case STANDARD_GET_INTERFACE:
				m_uEp0State = EP0_INTERFACE_GET;
				break;

			case STANDARD_SET_INTERFACE:
				m_poInterfaceGet->AlternateSetting= m_poDeviceRequest->wValue_L;
				m_uEp0State = EP0_STATE_INIT;
				break;

			case STANDARD_SYNCH_FRAME:
				m_uEp0State = EP0_STATE_INIT;
				break;

			default:
				break;
		}
	}

	TransferEp0();


}


// Not Supporting About Set Descriptor for USB Standard
void USBDEV::StandardSetDescriptor(void)
{
	// CLR_EP0_CSR_OUT_PACKET_READY;
}


// ======================================================================
// TransferEp0()
//
// This is function for Control Transfer Transmit.
// ======================================================================

void USBDEV::TransferEp0(void)
{
	U32 i;
	U32 dataLength;
	U16 usSysStatus;

// 	CLR_EP0_CSR_OUT_PACKET_READY;
	switch (m_uEp0State)
	{
		case EP0_STATE_INIT:
			break;

		// === GET_DESCRIPTOR:DEVICE ===
		case EP0_STATE_GD_DEV_0:
			if (m_eSpeed == 1)
			{
				Outp32(BYTE_WRITE_CNT_REG, 18);
						WrPktEp0((U8 *)m_poDescDevice+0, 18); // EP0_PKT_SIZE
						m_uEp0State = EP0_STATE_INIT;
				DbgUsb(("EndpointZeroTransfer(EP0_STATE_GD_DEV)\n"));
			}
			else
			{
				Outp32(BYTE_WRITE_CNT_REG, 8);
						WrPktEp0((U8 *)m_poDescDevice+0, 8); // EP0_PKT_SIZE
						m_uEp0State = EP0_STATE_GD_DEV_1;
				DbgUsb(("EndpointZeroTransfer(EP0_STATE_GD_DEV_0)\n"));
			}
			break;

		case EP0_STATE_GD_DEV_1:
			Outp32(BYTE_WRITE_CNT_REG, 8);
					WrPktEp0((U8 *)m_poDescDevice+8, 8); // EP0_PKT_SIZE
					m_uEp0State = EP0_STATE_GD_DEV_2;
			DbgUsb(("EndpointZeroTransfer(EP0_STATE_GD_DEV_1)\n"));
			break;

		case EP0_STATE_GD_DEV_2:
			Outp32(BYTE_WRITE_CNT_REG, 2);
					WrPktEp0((U8 *)m_poDescDevice+16, 2); // EP0_PKT_SIZE
					m_uEp0State = EP0_STATE_INIT;
			DbgUsb(("EndpointZeroTransfer(EP0_STATE_GD_DEV_2)\n"));
			break;

		// === GET_DESCRIPTOR:CONFIGURATION+INTERFACE+ENDPOINT0+ENDPOINT1 ===
		// Windows98 gets these 4 descriptors all together by issuing only a request.
		// Windows2000 gets each descriptor seperately.
		// === GET_DESCRIPTOR:CONFIGURATION ONLY for WIN2K===

		case EP0_STATE_GD_CFG_0:
			if (m_eSpeed == 1)
			{
				Outp32(BYTE_WRITE_CNT_REG, 32);
				WrPktEp0((U8 *)&m_poDesc->oDescConfig+0, 32); // EP0_PKT_SIZE
				m_uEp0State = EP0_STATE_INIT;
				DbgUsb(("EndpointZeroTransfer(EP0_STATE_GD_CFG)\n"));
			}
			else
			{
				Outp32(BYTE_WRITE_CNT_REG, 8);
				WrPktEp0((U8 *)&m_poDesc->oDescConfig+0, 8); // EP0_PKT_SIZE
				m_uEp0State = EP0_STATE_GD_CFG_1;
				DbgUsb(("EndpointZeroTransfer(EP0_STATE_GD_CFG_0)\n"));
			}
			break;

		case EP0_STATE_GD_CFG_1:
			Outp32(BYTE_WRITE_CNT_REG, 8);
			WrPktEp0((U8 *)&m_poDesc->oDescConfig+8, 8); // EP0_PKT_SIZE	WrPktEp0((U8 *)&descConf+8, 1); WrPktEp0((U8 *)&descIf+0, 7);
			m_uEp0State = EP0_STATE_GD_CFG_2;
			break;

		case EP0_STATE_GD_CFG_2:
			Outp32(BYTE_WRITE_CNT_REG, 8);
			WrPktEp0((U8 *)&m_poDesc->oDescConfig+16, 8); // EP0_PKT_SIZE	WrPktEp0((U8 *)&descIf+7, 2); WrPktEp0((U8 *)&descEndpt0+0, 6);
			m_uEp0State = EP0_STATE_GD_CFG_3;
			break;

		case EP0_STATE_GD_CFG_3:
			Outp32(BYTE_WRITE_CNT_REG, 8);
			WrPktEp0((U8 *)&m_poDesc->oDescConfig+24, 8); // EP0_PKT_SIZE	WrPktEp0((U8 *)&descEndpt0+6, 1); WrPktEp0((U8 *)&descEndpt1+0, 7);
			m_uEp0State = EP0_STATE_GD_CFG_4;
			break;

		case EP0_STATE_GD_CFG_4:
			Outp32(BYTE_WRITE_CNT_REG, 0);
			m_uEp0State = EP0_STATE_INIT;
			break;

		// === GET_DESCRIPTOR:CONFIGURATION ONLY===
		case EP0_STATE_GD_CFG_ONLY_0:
			if (m_eSpeed == 1)
			{
				DbgUsb(("[DBG : EP0_STATE_GD_CFG_ONLY]\n"));
				Outp32(BYTE_WRITE_CNT_REG, 9);
				WrPktEp0((U8 *)&m_poDesc->oDescConfig+0, 9); // EP0_PKT_SIZE
				m_uEp0State = EP0_STATE_INIT;
			}
			else
			{
				DbgUsb(("[DBG : EP0_STATE_GD_CFG_ONLY_0]\n"));
				Outp32(BYTE_WRITE_CNT_REG, 8);
				WrPktEp0((U8 *)&m_poDesc->oDescConfig+0, 8); // EP0_PKT_SIZE
				m_uEp0State = EP0_STATE_GD_CFG_ONLY_1;
			}
			break;

		case EP0_STATE_GD_CFG_ONLY_1:
			DbgUsb(("[DBG : EP0_STATE_GD_CFG_ONLY_1]\n"));
			Outp32(BYTE_WRITE_CNT_REG, 1);
			WrPktEp0((U8 *)&m_poDesc->oDescConfig+8, 1); // EP0_PKT_SIZE
			m_uEp0State = EP0_STATE_INIT;
			break;

		// === GET_DESCRIPTOR:INTERFACE ONLY===

		case EP0_STATE_GD_IF_ONLY_0:
			if (m_eSpeed == 1)
			{
				Outp32(BYTE_WRITE_CNT_REG, 9);				// INTERFACE_DESC_SIZE
				WrPktEp0((U8 *)&m_poDesc->oDescInterface+0, 9);
				m_uEp0State = EP0_STATE_INIT;
			}
			else
			{
				Outp32(BYTE_WRITE_CNT_REG, 8);				// INTERFACE_DESC_SIZE
				WrPktEp0((U8 *)&m_poDesc->oDescInterface+0, 8);
				m_uEp0State = EP0_STATE_GD_IF_ONLY_1;
			}
			break;

		case EP0_STATE_GD_IF_ONLY_1:
			Outp32(BYTE_WRITE_CNT_REG, 1);
			WrPktEp0((U8 *)&m_poDesc->oDescInterface+8, 1);
			m_uEp0State = EP0_STATE_INIT;
			break;


		// === GET_DESCRIPTOR:ENDPOINT 1 ONLY===
		case EP0_STATE_GD_EP0_ONLY_0:
			Outp32(BYTE_WRITE_CNT_REG, ENDPOINT_DESC_SIZE);
			Inp32(SYS_STATUS_REG, usSysStatus);
			WrPktEp0((U8 *)&m_poDesc->oDescEndpt1+0, ENDPOINT_DESC_SIZE);
			m_uEp0State = EP0_STATE_INIT;
			break;

		// === GET_DESCRIPTOR:ENDPOINT 2 ONLY===
		case EP0_STATE_GD_EP1_ONLY_0:
			Outp32(BYTE_WRITE_CNT_REG, ENDPOINT_DESC_SIZE);
			Inp32(SYS_STATUS_REG, usSysStatus);
			WrPktEp0((U8 *)&m_poDesc->oDescEndpt3+0, ENDPOINT_DESC_SIZE);
			m_uEp0State = EP0_STATE_INIT;
			break;

				// === GET_DESCRIPTOR:STRING ===
		case EP0_STATE_GD_STR_I0:
			Outp32(BYTE_WRITE_CNT_REG, 4);
			DbgUsb(("[GDS0_0]"));
			WrPktEp0((U8 *)aDescStr0, 4);
			m_uEp0State = EP0_STATE_INIT;
			break;

		case EP0_STATE_GD_STR_I1:
			DbgUsb(("[GDS1_%d]", m_uEp0SubState));
			if ((m_uEp0SubState*m_uEp0MaxPktSize+m_uEp0MaxPktSize)<sizeof(aDescStr1))
			{
				Outp32(BYTE_WRITE_CNT_REG, m_uEp0MaxPktSize);
				WrPktEp0((U8 *)aDescStr1+(m_uEp0SubState*m_uEp0MaxPktSize), m_uEp0MaxPktSize);
				m_uEp0State = EP0_STATE_GD_STR_I1;
				m_uEp0SubState++;
			}
			else
			{
				Outp32(BYTE_WRITE_CNT_REG, sizeof(aDescStr1)-(m_uEp0SubState*m_uEp0MaxPktSize));
				WrPktEp0((U8 *)aDescStr1+(m_uEp0SubState*m_uEp0MaxPktSize), sizeof(aDescStr1)-(m_uEp0SubState*m_uEp0MaxPktSize));
				m_uEp0State = EP0_STATE_INIT;
				m_uEp0SubState = 0;
			}
			break;

		case EP0_STATE_GD_STR_I2:
			DbgUsb(("[GDS2_%d]", m_uEp0SubState));
			if ((m_uEp0SubState*m_uEp0MaxPktSize+m_uEp0MaxPktSize)<sizeof(aDescStr2))
			{
				Outp32(BYTE_WRITE_CNT_REG, m_uEp0MaxPktSize);
				WrPktEp0((U8 *)aDescStr2+(m_uEp0SubState*m_uEp0MaxPktSize), m_uEp0MaxPktSize);
				m_uEp0State = EP0_STATE_GD_STR_I2;
				m_uEp0SubState++;
			}
			else
			{
				DbgUsb(("[E]"));
				Outp32(BYTE_WRITE_CNT_REG, sizeof(aDescStr2)-(m_uEp0SubState*m_uEp0MaxPktSize));
				WrPktEp0((U8 *)aDescStr2+(m_uEp0SubState*m_uEp0MaxPktSize), sizeof(aDescStr2)-(m_uEp0SubState*m_uEp0MaxPktSize));
				m_uEp0State = EP0_STATE_INIT;
				m_uEp0SubState = 0;
			}
			break;

		case EP0_STATE_GD_DEV_QUALIFIER:
			Outp32(BYTE_WRITE_CNT_REG, 10);
			WrPktEp0((U8 *)aDeviceQualifierDescriptor+0, 10);
			m_uEp0State = EP0_STATE_INIT;
			break;

		case EP0_INTERFACE_GET:
			Outp32(BYTE_WRITE_CNT_REG, 1);
			WrPktEp0((U8 *)m_poInterfaceGet+0, 1);
			m_uEp0State = EP0_STATE_INIT;
			break;


		case EP0_GET_STATUS0:
			Outp32(BYTE_WRITE_CNT_REG, 1);
			WrPktEp0((U8 *)m_poStatusGet+0, 1);
			m_uEp0State = EP0_STATE_INIT;
			break;

		case EP0_GET_STATUS1:
			Outp32(BYTE_WRITE_CNT_REG, 1);
			WrPktEp0((U8 *)m_poStatusGet+1, 1);
			m_uEp0State = EP0_STATE_INIT;
			break;

		case EP0_GET_STATUS2:
			Outp32(BYTE_WRITE_CNT_REG, 1);
			WrPktEp0((U8 *)m_poStatusGet+2, 1);
			m_uEp0State = EP0_STATE_INIT;
			break;

		case EP0_GET_STATUS3:
			Outp32(BYTE_WRITE_CNT_REG, 1);
			WrPktEp0((U8 *)m_poStatusGet+3, 1);
			m_uEp0State = EP0_STATE_INIT;
			break;

		case EP0_GET_STATUS4:
			Outp32(BYTE_WRITE_CNT_REG, 1);
			WrPktEp0((U8 *)m_poStatusGet+4, 1);
			m_uEp0State = EP0_STATE_INIT;
			break;

		default:
			break;


	}
}

// ======================================================================
// HandleBulkIn()
//
// This is function for Endpoint One ( Bulk In)function routine.
// ======================================================================
void USBDEV::HandleEvent_BulkIn(void)
{
	U16 ep1csr;
	U32 temp;

	// EP1 CSR register status check
	Outp32(INDEX_REG, 0x01);
	Inp32(EP_STATUS_REG, ep1csr);
	Outp32(EP_STATUS_REG, ep1csr);
	Inp32(DMA_TOTAL_CNT1_REG, temp);
	printf("EP1: DMA_TOTAL_CNT1_REG : %x\n", temp);

	if (ep1csr & EP_SENT_STALL) { // SENT STALL : protocol stall.
		DbgUsb(("Sent Stall \n"));
		Outp32(EP_STATUS_REG, EP_SENT_STALL);
	}

	if (ep1csr & EP_TX_SUCCESS)
	{
		printf("EP1_TX_SUCCESS\n");
		Outp32(EP_STATUS_REG, EP_TX_SUCCESS); // Endpoint Status Register Clear
		if (m_eOpMode == USB_CPU)
		{
			if (m_uBulkInCount != 0)
				PrepareEp1Fifo(m_uBulkInAddr);
		}
	}

	if (DMA_TOTAL_COUNT_ZERO & ep1csr)
	{
		printf("USB_DMA_MODE, DMA TX Done(DMA_TOTAL_COUNT_ZERO) !!\n");
		Outp32(DMA_CON_REG, DMA_TX_STOP|USB_INT_MODE);
		Outp32(FCON, DMA_DISABLE);
	}
}

// ======================================================================
// HandleBulkOut()
//
// This is function for Endpoint Three ( Bulk Out)function routine.
// ======================================================================
U32 checkTest = 0;

void USBDEV::HandleEvent_BulkOut(void)
{
	U32 ReadData;
	U32 RecDataCnt;
	U16 ep3csr, ep2con, sys_status;
	U32 CBWSignature=0, CBWTag=0, i;
	U8  TempBuf[16];
	U16 fifoCnt;
	U16 fifoCntByte;
	U32 DmaCurrAddr;
	U32 temp;
	U32 uUploadSize;
	U32 uUploadAddr;
	U32 uDownloadFileSize;
	
	U32 j;
	
	Outp32(INDEX_REG, EP3);
	Inp32(EP_STATUS_REG, ep3csr);
	Outp32(EP_STATUS_REG, ep3csr);
 	DbgUsb(("Bulk Out Int, ep3csr : %x\n", ep3csr));
	Inp32(DMA_TOTAL_CNT1_REG, temp);
	DbgUsb(("EP3: DMA_TOTAL_CNT1_REG : %x\n", temp));




	if (ep3csr & EP_SENT_STALL) // SENT STALL : protocol stall.
	{
		// Uart_Printf(" Sent Stall \n");
		Outp32(EP_STATUS_REG, EP_SENT_STALL);
		return;
	}

	if (ep3csr & EP_FIFO_FLUSH)
	{
		Outp32(EP_CON_REG, EP_FIFO_FLUSH);
		return;
	}


//ahn..	

	if (DMA_TOTAL_COUNT_ZERO & ep3csr)
	{		
		checkTest = 1;
		//DbgUsb(("USB_DMA_MODE, DMA RX Done(DMA_TOTAL_COUNT_ZERO) !!\n"));
		//printf("USB_DMA_MODE, DMA RX Done(DMA_TOTAL_COUNT_ZERO) !!\n");
		//printf("USB_DMA_MODE, DMA RX Done(DMA_TOTAL_COUNT_ZERO) !!\n");
		//printf("USB_DMA_MODE, DMA RX Done(DMA_TOTAL_COUNT_ZERO) !!\n");
		
		Outp32(DMA_CON_REG, DMA_RX_STOP|USB_INT_MODE);
		
		m_eOpMode = USB_CPU;
		Inp32(DMA_MEM_CURRENT_ADDR, DmaCurrAddr);
		m_pDownPt = (U8 *)DmaCurrAddr;
		
		
		Outp32(FCON, DMA_DISABLE);
		
		//printf("m_pDownPt=%x\n",m_pDownPt);
		
		//Inp32(EP_STATUS_REG, ep3csr);
		DbgUsb(("DMA_TOTAL_COUNT_ZERO, ep3csr : %x\n", ep3csr));
//		ClearPending(BIT_USBD);
		
	//	while(1)
//		for(j=0;j<300;j++)
//		{
//			Inp32(EP_STATUS_REG, ep3csr);
//			printf("\n[rSRCPND : 0x%x]-[rINTPND : 0x%x],ep3csr=%x",rSRCPND, rINTPND,ep3csr);
//		}
		
//printf("\n[rSRCPND : 0x%x]-[rINTPND : 0x%x],ep3csr=%x",rSRCPND, rINTPND,ep3csr);                
                
		
		
		//printf("DMA_TOTAL_COUNT_ZERO, ep3csr : %x\n", ep3csr);
//		return;
	  if (!ep3csr & EP_RX_SUCCESS)// if interrupt is not genarated then check the RX_success bit.
	  	return;
	  
	}

//ahn..

	if (ep3csr & EP_RX_SUCCESS)
	{
		DbgUsb(("EP3_RX_PKT_SUCCESS\n"));
	//	Outp32(EP_INT_REG, INT_REG_EP3); //  ahn Interrupt Clear
		if (m_uDownloadFileSize==0)
		{
			Inp32(BYTE_READ_CNT_REG, fifoCnt);

			if (fifoCnt == 5)
			{
			#ifdef		__TEST
				RdPktEp3((U16 *)TempBuf, 10);
				#else
				RdPktEp3((U8 *)TempBuf, 10);
				#endif
				temp = *((U8 *)(TempBuf+8))+
					(*((U8 *)(TempBuf+9))<<8);
				DbgUsb(("temp: %x\n", temp));
				if (temp==0x1)
				{
					uUploadAddr =
						*((U8 *)(TempBuf+0))+
						(*((U8 *)(TempBuf+1))<<8)+
						(*((U8 *)(TempBuf+2))<<16)+
						(*((U8 *)(TempBuf+3))<<24);

					uUploadSize =
						*((U8 *)(TempBuf+4))+
						(*((U8 *)(TempBuf+5))<<8)+
						(*((U8 *)(TempBuf+6))<<16)+
						(*((U8 *)(TempBuf+7))<<24);
				
					printf("UploadAddress : %x, UploadSize: %x\n", uUploadAddr, uUploadSize);
					if (m_eOpMode == USB_CPU)
					{
						DbgUsb(("CPU_MODE Bulk In Function\n"));
						m_uBulkInCount = uUploadSize;
						PrepareEp1Fifo(uUploadAddr);
					}
					else
					{
						DbgUsb(("DMA_MODE Bulk In Function\n"));
						Outp32(FCON, DMA_ENABLE);				// USB Dma Enable in Core Outside
						Outp32(INDEX_REG, EP1);					// IN Direction  Device -> Host
						Outp32(DMA_IF_CON_REG, MAX_BURST_INCR16);
						Outp32(BYTE_WRITE_CNT_REG, m_uEp1MaxPktSize);
						Outp32(MAX_PKT_REG, m_uEp1MaxPktSize);
						Outp32(DMA_FIFO_CNT_REG, m_uEp1MaxPktSize);
						Outp32(DMA_CNT_REG, m_uEp1MaxPktSize);
						Outp32(DMA_MEM_BASE_ADDR, uUploadAddr);
						Outp32(DMA_TOTAL_CNT1_REG, (U16)uUploadSize);
						Outp32(DMA_TOTAL_CNT2_REG, (U16)(uUploadSize>>16));
						Outp32(DMA_CON_REG, DMA_FLY_ENABLE|DMA_TX_START|USB_DMA_MODE);
					}
				}
				m_uDownloadFileSize=0;
				return;
			}
			else
			{
			    if(download_run==0)
   	    		{
					m_uDownloadAddress=tempDownloadAddress; 
				//	printf("tempDownloadAddress = %x\n",tempDownloadAddress);
	    		}
	    		else
	    		{
	    		#ifdef		__TEST
					RdPktEp3((U16 *)TempBuf, 8);
					#else
					RdPktEp3((U8 *)TempBuf, 8);
					#endif
					if (ep3csr&(0x1<<4))
						fifoCntByte = fifoCnt * 2 -1;
					else
						fifoCntByte = fifoCnt * 2;
					DbgUsb(("downloadFileSize==0, 1'st BYTE_READ_CNT_REG : %x\n", fifoCntByte));
					m_uDownloadAddress=
						*((U8 *)(TempBuf+0))+
						(*((U8 *)(TempBuf+1))<<8)+
						(*((U8 *)(TempBuf+2))<<16)+
						(*((U8 *)(TempBuf+3))<<24);
	    		}
				if(download_run==0)
   	    		{
   	    		#ifdef		__TEST
					RdPktEp3((U16 *)TempBuf, 8);
					#else
					RdPktEp3((U8 *)TempBuf, 8);
					#endif
					if (ep3csr&(0x1<<4))
						fifoCntByte = fifoCnt * 2 -1;
					else
						fifoCntByte = fifoCnt * 2;
   	    		}
				//	DbgUsb(("downloadFileSize==0, 1'st BYTE_READ_CNT_REG : %x\n", fifoCntByte));
					m_uDownloadFileSize=
						*((U8 *)(TempBuf+4))+
						(*((U8 *)(TempBuf+5))<<8)+
						(*((U8 *)(TempBuf+6))<<16)+
						(*((U8 *)(TempBuf+7))<<24);

				m_pDownPt=(U8 *)m_uDownloadAddress;
				//DbgUsb(("downloadAddress : %x, downloadFileSize: %x\n", m_uDownloadAddress, m_uDownloadFileSize));
				//printf("downloadAddress : %x, downloadFileSize: %x\n", m_uDownloadAddress, m_uDownloadFileSize));
				//printf("*");
				
				#ifdef		__TEST
				RdPktEp3((U16 *)m_pDownPt, fifoCntByte-8); // The first 8-bytes are deleted.
				#else
				RdPktEp3((U8 *)m_pDownPt, fifoCntByte-8); // The first 8-bytes are deleted.
				#endif

				if (m_eOpMode == USB_CPU)
				{
				//printf("\n[0]\n");
					if (ep3csr & (0x2<<2))
					{
					//printf("\n[1]\n");
						Inp32(BYTE_READ_CNT_REG, fifoCnt);
						if (ep3csr&(0x1<<4))
							fifoCntByte = fifoCnt * 2 -1;
						else
							fifoCntByte = fifoCnt * 2;
						DbgUsb(("2'd BYTE_READ_CNT_REG : %x\n", fifoCntByte));
						#ifdef		__TEST
						RdPktEp3((U16 *)m_pDownPt, fifoCntByte);
						#else
						RdPktEp3((U8 *)m_pDownPt, fifoCntByte);
						#endif
					//	m_cpucnt++;
					}
				}
				
				else
				{
						uDownloadFileSize = m_uDownloadFileSize-fifoCntByte;
					if (uDownloadFileSize > m_uEp3MaxPktSize)
					{
						Outp32(FCON, DMA_ENABLE);				// USB Dma Enable in Core Outside
						Outp32(INDEX_REG, EP3);					// OUT Direction  Host -> Device
						Outp32(DMA_IF_CON_REG, MAX_BURST_INCR16);
						Outp32(MAX_PKT_REG, m_uEp3MaxPktSize);
						Outp32(DMA_FIFO_CNT_REG, m_uEp3MaxPktSize);
						Outp32(DMA_CNT_REG, m_uEp3MaxPktSize);
						Outp32(DMA_MEM_BASE_ADDR, m_uDownloadAddress+fifoCntByte-8);
						uDownloadFileSize = (uDownloadFileSize/m_uEp3MaxPktSize)*m_uEp3MaxPktSize;
						
						//printf("uDownloadFileSize = %d\n",uDownloadFileSize);
						//printf("fifoCntByte = %d\n",fifoCntByte);
							
						Outp32(DMA_TOTAL_CNT1_REG, (U16)(uDownloadFileSize));
						Outp32(DMA_TOTAL_CNT2_REG, (U16)(uDownloadFileSize>>16));
						DbgUsb(("Out Direction DMA RX Start\n"));
						Outp32(DMA_CON_REG, DMA_FLY_ENABLE|DMA_RX_START|USB_DMA_MODE);
					}
					else
						m_eOpMode = USB_CPU;
				}
			}
		}

		else
		{	
			//printf("\n22");			
			Inp32(EP_STATUS_REG, ep3csr);
			//printf("\n[rSRCPND : 0x%x]-[rINTPND : 0x%x],ep3csr=%x",rSRCPND, rINTPND,ep3csr);
		
			//if (m_eOpMode == USB_CPU)
			//printf("\nUSB_CPU");
			//else
			//printf("\nUSB_DMA");
			
			if (m_eOpMode == USB_CPU)
			//if (DMA_TOTAL_COUNT_ZERO & ep3csr)
			{
				Inp32(BYTE_READ_CNT_REG, fifoCnt);
				if (ep3csr&(0x1<<4))
					fifoCntByte = fifoCnt * 2 -1;
				else
					fifoCntByte = fifoCnt * 2;
				DbgUsb(("downloadFileSize!=0, 1'st BYTE_READ_CNT_REG : %x\n", fifoCntByte));
				#ifdef		__TEST
				RdPktEp3((U16 *)m_pDownPt, fifoCntByte);
				#else
				RdPktEp3((U8 *)m_pDownPt, fifoCntByte);
				#endif
				//printf("\n[1]\n");
				if ((ep3csr & (0x2<<2))||(ep3csr & (0x1<<2)))
				{
				//printf("\n[2]\n");
					Inp32(BYTE_READ_CNT_REG, fifoCnt);
					if (ep3csr&(0x1<<4))
						fifoCntByte = fifoCnt * 2 -1;
					else
						fifoCntByte = fifoCnt * 2;
					DbgUsb(("2'd BYTE_READ_CNT_REG : %x\n", fifoCntByte));
					#ifdef		__TEST
					RdPktEp3((U16 *)m_pDownPt, fifoCntByte);
					#else
					RdPktEp3((U8 *)m_pDownPt, fifoCntByte);
					#endif
				
				}
			}
		}
		return;
	}
/*
		if (DMA_TOTAL_COUNT_ZERO & ep3csr)
	{

		Delay(10);
		Outp32(DMA_CON_REG, DMA_RX_STOP|USB_INT_MODE);
		Outp32(FCON, DMA_DISABLE);
	//	Inp32(EP_STATUS_REG, ep3csr);
	//	printf("ep3csr = %x \n",ep3csr); 
		Inp32(EP_STATUS_REG, ep3csr);
		if (ep3csr & EP_RX_SUCCESS)									// short packet handled			
		{
			Inp32(BYTE_READ_CNT_REG, fifoCnt);
			if (ep3csr&(0x1<<4)) // RX packet success but remain Odd word 
				fifoCntByte = fifoCnt * 2 -1;
			else
				fifoCntByte = fifoCnt * 2;
		//	printf("downloadFileSize!=0, 1'st BYTE_READ_CNT_REG : %x\n", fifoCntByte);
			Inp32(DMA_MEM_CURRENT_ADDR, DmaCurrAddr);
			m_pDownPt = (U8 *)DmaCurrAddr;
			RdPktEp3((U8 *)m_pDownPt, fifoCntByte);

			if (ep3csr & (0x2<<2))
			{
				Inp32(BYTE_READ_CNT_REG, fifoCnt);
				if (ep3csr&(0x1<<4))
					fifoCntByte = fifoCnt * 2 -1;
				else
					fifoCntByte = fifoCnt * 2;
				DbgUsb(("2'd BYTE_READ_CNT_REG : %x\n", fifoCntByte));
				RdPktEp3((U8 *)m_pDownPt, fifoCntByte);
			}
			dma_done =1;	
		}
		else
		dma_done =1;
		
	}
//	printf("ep3csr = %x \n",ep3csr); 
*/	
}
