/******************************************************************************/
// 
// File Name   : Timer.c
// Description : S3C2443 PWM Timer test 
// Version     : 0.3 
// Date    	   : July 3, 2003
//   0.0 : Programming start (February 27,2002 in 2410) -> SOP
//   0.1 : Modified by Y. H. Lee
//   0.2 : edited by junon
//   0.3:  Modified by Jcs (2006.05.20)
/*******************************************************************************/

/* Include Files */
#include <string.h>
#include "def.h"
#include "option.h"
#include "2443addr.h"
#include "console.h"
#include "system.h"
#include "timer.h"

/* Structure Define for DMA Special Registers */
typedef struct tagDMA
{
    volatile U32 DISRC;	    //0x0
    volatile U32 DISRCC;    //0x4
    volatile U32 DIDST;	    //0x8
    volatile U32 DIDSTC;    //0xc
    volatile U32 DCON;	    //0x10
    volatile U32 DSTAT;	    //0x14
    volatile U32 DCSRC;	    //0x18
    volatile U32 DCDST;	    //0x1c
    volatile U32 DMASKTRIG; //0x20
    volatile U32 DMAREQSEL;	//0x24
}DMA;

static volatile int dmaDone; 
int TestMode; 

volatile int isWdtInt1;

//=========================================================================
//
//  Timer input clock frequency = PCLK/{prescaler value+1}/{divider value}
//  {prescaler value} = 0 ~ 255, {divider value} = 2,4,8,16
//
//=========================================================================

/******************** Selection Routine for Timer Test ********************/ 
void * func_timer_test[][2]=
{	
	(void *)Test_TimerNormal, 			"Timer Normal    ",
	(void *)Test_TimerInt, 				"Timer Interrupt ",
	(void *)Test_TimerDma,				"Timer DMA       ",

	0,0
};


void Test_Timer(void)
{
	int i;
	
	while(1)
	{
		printf("\n\n============== Timer Function Test ==============\n");

		i=0;
		printf("\n");
		while(1)
		{   //display menu
			printf("%2d:%s",i,func_timer_test[i][1]);
			i++;
			if((int)(func_timer_test[i][0])==0)
			{
				printf("\n");
				break;
			}
			if((i%2)==0)
			printf("\n");
		}

		printf("\n=================================================\n");
	
		printf("\nPress Enter key to exit : ");
		i = GetIntNum();
		if(i==-1) break;		// return.
		if(i>=0 && (i<((sizeof(func_timer_test)-1)/8)) )	// select and execute...
			( (void (*)(void)) (func_timer_test[i][0]) )();
	}
	
}



/******************** PWM Timer Normal Operation Test ********************/
void Test_TimerNormal(void)
{
    int save_B,save_H,save_PB,save_PH,save_MI;
    char key, toggle;
    
    /* Save Current Register related to PWM Port */
    //Save Current GPB, G, H Configure Registers 
    save_B  = rGPBCON;
    save_H  = rGPHCON;
    //Save Pull-Up Disable Register     
    save_PB = rGPBUDP;
    save_PH = rGPHUDP;
    //Save Miscellaneous Control Register 
    save_MI = rMISCCR;   
    
    /* Setting Port related to PWM */ 
    // PortB
    rGPBUDP = rGPBUDP & ~(0x3ff)|0x2aa; //GPB[4:0] Pull Up/Down disable
    rGPBCON = rGPBCON & ~(0x3ff) |0x2aa; //TCKL, TOUT[3:0]   

    // PortH
    rGPHUDP = rGPHUDP  & ~(0x3<<26)|(0x2<<26); //GPH[13] Pull Up/Down disable
    rGPHCON = rGPHCON & ~(0x3<<26) |(0x2<<26); //GPH13=>CLKOUT0  
    // Miscellaneous Control Register
    rMISCCR = rMISCCR & ~(0x7<<4) |(0x4<<4); //Select PCLK with CLKOUT0

    printf("[Select Timer Test Setting]\n");
    printf("a. Prescaler 1, 0: 0, Dead zone Disable, MUX 4~0: 1/2, (H/L)duty 50 \n");
    printf("b. Prescaler 1, 0: 8, Dead zone Enable, MUX 4~0: 1/16, (H/L)duty 50 \n");
    printf("c. (H/L)duty 0, TCNT =< TCMP, Inverter On\n");
    printf("d. TCLK Input Divider Test\n");
    printf("Select [a ~ d]: \n");
    
        
    key = getchar();    
    printf("\n");    
	
	/* Timer Configuration Register.0, 1(TCFG0, TCFG1) Setting */
	switch(key)
    {
        case 'a':
           	
    		rTCFG0 = rTCFG0 & ~(0xffffff) | 0x00000; //Dead zone=0, Prescaler1=0, Prescaler0=0
           	rTCFG1 = 0x0; //All Interrupt, MUX 4 ~ 0: 1/2            
              printf("Prescaler1=0, Prescaler0=0, Dead Zone Disable, MUX 4 ~ 0: 1/2, Duty 50% \n");
            break;
        
        case 'b':    
            
            rTCFG0 = rTCFG0 & ~(0xffffff) | (0x0)<<16 | (0x7)<<8 | (0x7); //Dead zone=0, Prescaler1=7, Prescaler0=7
	     	rTCFG1 = rTCFG1 & ~(0xffffff) | (0x3)<<16 | (0x3)<<12 | (0x3)<<8 | (0x3)<<4 | (0x3); //All Interrupt, MUX 4 ~ 0: 1/16
            printf("Prescaler1=8, Prescaler0=8, Dead zone=200 Enable, MUX 4 ~ 0: 1/16, duty = 50%, Consider Dead zone effect\n");        		
            break;
            
        case 'c':    
            
            rTCFG0 = rTCFG0 & ~(0xffffff) | 0x0;	//Dead zone=0, Prescaler1=0, Prescaler0=0
	     	rTCFG1 = rTCFG1 & ~(0xffffff) | 0x0;	//All Interrupt, MUX 4 ~ 0: 1/2
            printf("(H/L)Duty 0, TCNT =< TCMP, Inverter On\n");
            break;
        
        case 'd':
           	
    	    rTCFG0 = rTCFG0 & ~(0xffffff) | 0x0; //Dead zone=0, Prescaler1=0, Prescaler0=0
            rTCFG1 = rTCFG1 & ~(0xffffff) | 0x4 | 0x4 << 4 | 0x4 << 8 | 0x4 << 12 | 0x4 << 16;	//All Interrupt, MUX 4 ~ 0: 1/2            
            printf("TCLK Input Divider Test(if TCLK=25.4MHZ)\n");
            break;
         
        default:
            rGPBCON = save_B; 
            rGPHCON = save_H;           
            rGPBUDP = save_PB;
            rGPHUDP = save_PH;
            rMISCCR = save_MI;
            return;
    }
    
    //(1/(PCLK/(Prescaler+1)/divider) * count(Max 65535) = Timer clock (Frequency)
    //(1/(50.8MHz/1/2))  *  2000 = 0.0787 msec ( 12.7 KHz)
    //(1/(50.8MHz/1/2))  *  4000 = 0.1575 msec ( 6.35 KHz)
    //(1/(50.8MHz/1/2))  *  5000 = 0.1969 msec ( 5.080 KHz)
    //(1/(50.8MHz/1/2))  *  10000 = 0.3937 msec ( 2.54 KHz)
    
    //(1/(50.8MHz/8/16)) *  2000 = 5.039 msec ( 198.45 Hz)
    //(1/(50.8MHz/8/16)) *  4000 = 10.079 msec ( 99.216 KHz)
    //(1/(50.8MHz/8/16)) *  5000 = 12.598 msec ( 79.38 Hz) 
    //(1/(50.8MHz/8/16)) *  10000 = 25.197 msec ( 39.687 Hz)
                          
    rTCNTB0 = 2000;      
    rTCNTB1 = 4000;       
    rTCNTB2 = 5000;       
    rTCNTB3 = 10000;      
                                                
    rTCMPB0 =  2000 - 1000; //(H/L)duty 50%
    rTCMPB1 =  4000 - 2000;
    rTCMPB2 =  5000 - 2500;
    rTCMPB3 = 10000 - 5000;
	
	rTCON  = rTCON & ~(0xffffff) | 0x1<<1 | 0x1<<9 | 0x1<<13 | 0x1<<17 | 0x1<<21 ; //Manual update 
	
	switch(key)
	{
	case 'a':
		
     	 rTCON   = rTCON & ~(0xffffff) | 0x599909;  //Auto reload, Inverter off, No operation, Start, Dead zone Disable
	break;
		
	case 'b':
	
     	rTCNTB0 = rTCNTB0 & ~(0xffff) | 5; //(1/(50.8MHz/8/16)) *  500 = 1.273 msec ( 793.8 Hz)
     	rTCMPB0 =  5 - 2; //(H/L)duty 50%
     	
     	rTCON = rTCON & ~(0xffffff) | 0x6aaa0a;  //Auto reload, Inverter off, Manual update, Stop, Dead zone Disable
    	rTCON = rTCON & ~(0xffffff) | 0x599909| (0x1) << 4;  //Auto reload, Inverter off, No operation, Start, Dead zone Enable
		
    	break;
    	
    case 'c':
	
     	rTCNTB0 = rTCNTB0 & ~(0xffff) | 1000; 
     	rTCMPB0 =  300;      
     	
     	rTCNTB1 = rTCNTB1 & ~(0xffff) | 1000; 
     	rTCMPB1 =  1000; 
     	
     	rTCNTB2 = rTCNTB2 & ~(0xffff) | 1000; 
     	rTCMPB2 =  1000; 
     	
     	rTCNTB3 = rTCNTB3 & ~(0xffff) | 1000; 
     	rTCMPB3 =  1000; 
     		
     	rTCON = rTCON & ~(0xffffff) | 0x1 << 1 | 0x1 << 9 | 0x1 << 13 | 0x1 << 17 | 0x1 << 21;  //Auto reload disable, Inverter off, Manual update, Stop, Dead zone Disable
		rTCON = rTCON & ~(0xffffff) | 0x1 | 0x1 << 2 | 0x1 <<8 | 0x1 << 10| 0x1 << 12 | 0x1 << 14 | 0x1 << 16 | 0x1 << 18| 0x1 << 20;  //Auto reload enable, Inverter On, No Operation, Start, Dead zone Disable
		
		printf("rTCNTB0 = Dec:%d, rTCNTO0 = Dec:%d, rTCMPB0 = Dec:%d\n",rTCNTB0, rTCNTO0, rTCMPB0);
    	break;
    
    case 'd':
    
		rTCON   = rTCON & ~(0xffffff) | 0x599909;  //Auto reload, Inverter off, No operation, Start, Dead zone Disable
		break;
    
     	default:
        break;
    }
          
    if(key=='a' && (PCLK==(203200000/4)))  
    {
        
      	printf("PCLK 50.8MHz, Timer TOUT0 : 0.0787 msec ( 12.70 KHz)\n");
      	printf("PCLK 50.8MHz, Timer TOUT1 : 0.1575 msec (  6.35 KHz)\n");
        printf("PCLK 50.8MHz, Timer TOUT2 : 0.1969 msec (  5.08 KHz)\n");
        printf("PCLK 50.8MHz, Timer TOUT3 : 0.3937 msec (  2.54 KHz)\n");        
    }    
      
    else if(key=='b' && (PCLK==(203200000/4)))  
    {
        printf("PCLK 50.8MHz, Timer TOUT0 :   1.273 msec (793.8 KHz)\n");
        printf("PCLK 50.8MHz, Timer TOUT1 : /TOUT0 \n");
        printf("PCLK 50.8MHz, Timer TOUT2 : 13.6094 msec ( 73.478 Hz)\n");
        printf("PCLK 50.8MHz, Timer TOUT3 : 27.2189 msec ( 36.739 Hz)\n");                        
    }    
    
    else if(key=='d' && (PCLK==(203200000/4)))	 
    {
        printf("then Timer TOUT 0 : 0.0787 msec ( 12.70 KHz)\n");
        printf("then Timer TOUT 1 : 0.1575 msec (  6.35 KHz)\n");
        printf("then Timer TOUT 2 : 0.1969 msec (  5.08 KHz)\n");
        printf("then Timer TOUT 3 : 0.3937 msec (  2.54 KHz)\n");                        
    }
    
    printf("\nProbe PCLK.\n");
    printf("Probe TOUT0.\n");
    printf("Probe TOUT1.\n");
    printf("Probe TOUT2.\n");
    printf("Probe TOUT3.\n");
    
    printf("\nCheck PWM (Pulse Width Modulation) Output\n");
    printf("Press any key.\n");
    getchar();   
    
    /* Stop Timer0, 1, 2, 3, 4 */
    rTCON   = 0x0;	//One-shot, Inverter off, No operation, Dead zone disable, Stop Timer 
	
    rGPBCON = save_B; 
    rGPHCON = save_H;    
    rGPBUDP  = save_PB;
    rGPHUDP  = save_PH;
    rMISCCR = save_MI;

}

/******************** Timer Interrupt 0/1/2/3/4 test ********************/										
volatile int variable0,variable1,variable2,variable3,variable4;
int j;

void Test_TimerInt(void)
{
    variable0 = 0;variable1 = 0;variable2 = 0;variable3 = 0;variable4 = 0;
		
	/* Timer0,1,2,3,4 Interrupt service is available */ 
    rINTMSK = ~(BIT_TIMER4 | BIT_TIMER3 | BIT_TIMER2 | BIT_TIMER1 | BIT_TIMER0);
	// printf("rINTMSK (After)  = 0x%8x   <= Timer4,3,2,1 Bit[14:10]\n",rINTMSK);

    /* Timer0,1,2,3,4 Interrupt Service Routine Entry Point Determine */ 
    pISR_TIMER0 = (int)Timer0Done;
    pISR_TIMER1 = (int)Timer1Done;
    pISR_TIMER2 = (int)Timer2Done;
    pISR_TIMER3 = (int)Timer3Done;
    pISR_TIMER4 = (int)Timer4Done;

    printf("\n[Timer 0,1,2,3,4 Interrupt Test]\n\n");

    rTCFG0 = rTCFG0 & ~(0xffffff) | 0xff | 0xff<<8; //Dead zone=0,Prescaler1=255(0x0f),Prescaler0=255(0x0f)
    rTCFG1  =rTCFG1 & ~(0xffffff) | 0x001233; //All interrupt,Mux4=1/2,Mux3=1/4,Mux2=1/8,Mux1=1/16,Mux0=1/16
	
    rTCNTB0 = 30000;         
    rTCNTB1 = 30000;           
    rTCNTB2 = 30000;           
    rTCNTB3 = 30000;        
    rTCNTB4 = 30000;         
    
    rTCMPB0 = 15000;
    rTCMPB1 = 15000;
    rTCMPB2 = 15000;
    rTCMPB3 = 15000;
    
    rTCON  = rTCON & ~(0xffffff) | 0x1<<1 | 0x1<<9 | 0x1<<13 | 0x1<<17 | 0x1<<21 ; //Manual update 	
    
   
    rTCON  = rTCON & ~(0xffffff) | 0x1 | 0x1<<3 ; //Timer 0 Start, Auto-reload 
    rTCON  = rTCON | 0x1<<8 | 0x1<<11 ; //Timer 1 Start, Auto-reload
    rTCON  = rTCON | 0x1<<12 | 0x1<<15 ; //Timer 2 Start, Auto-reload
    rTCON  = rTCON | 0x1<<16 | 0x1<<19 ; //Timer 3 Start, Auto-reload
    rTCON  = rTCON | 0x1<<20 | 0x1<<22 ; //Timer 4 Start, Auto-reload   
    
    //Auto reload,Inverter off,No operation,Dead zone disable,Start
	
    while(1)
    {
     if(variable4 == 8)
     break;
    }
      
    //Delay(1); //To compensate timer error(<1 tick period)
    
    rTCON = 0x0; //Stop Timers
   
    if(variable4==8 && variable3==4 && variable2==2 && variable1==1 && variable0==1)
    {
        printf("Timer 0,1,2,3,4 Auto-reload Interrupt Test => OK!\n");
    }        
    else
    {
        printf("Timer 0,1,2,3,4 Auto-reload Interrupt Test => Fail............\n");
    }    
    printf("Timer0: %d (=1)\nTimer1: %d (=1)\nTimer2: %d (=2)\nTimer3: %d (=4)\nTimer4: %d (=8)\n",
        variable0,variable1,variable2,variable3,variable4);        
	
	/* OneShot Test */
  	variable0 = 0;variable1 = 0;variable2 = 0;variable3 = 0;variable4 = 0;
  	
  	rTCON   = rTCON & ~(0xffffff) | 0x6aaa0a;
             
  	//Auto reload, Inverter off, Manual update, Dead zone disable, Stop  
    rTCON   = rTCON & ~(0xffffff) | 0x1 |(0x1)<<8| (0x1)<<12 | (0x1)<<16 | (0x1)<<20 ;
     
    //T0,1,2,3,4= One-shot,Inverter off,No operation,Dead zone disable,Start

	while(variable0 == 0);

    
    Delay(1); //To compensate timer error(<1 tick period)
    rTCON = 0x0; //One-shot, Inverter off, No operation, Dead zone disable, Stop
   
    if(variable4==1 && variable3==1 && variable2==1 && variable1==1 && variable0==1)
    {
        printf("Timer 0,1,2,3,4 One-shot Interrupt Test => OK!\n");
    }        
    else
    {
        printf("Timer 0,1,2,3,4 One-shot Interrupt Test => Fail............\n");
    }    
    	printf("Timer0: %d (=1)\nTimer1: %d (=1)\nTimer2: %d (=1)\nTimer3: %d (=1)\nTimer4: %d (=1)\n",
        variable0,variable1,variable2,variable3,variable4);        

	/* Timer0,1,2,3,4 Interrupt Service is masked */
    rINTMSK |= (BIT_TIMER4 | BIT_TIMER3 | BIT_TIMER2 | BIT_TIMER1 | BIT_TIMER0); 
	printf("Press any key\n");      
	getchar();       //Key board press check      

}

/****** Timer Interrupt Request ******/ 
void __irq Timer0Done(void)
{
    rSRCPND = BIT_TIMER0;       //Clear pending bit
    rINTPND = BIT_TIMER0;
    rINTPND;                    //Prevent an double interrupt pending
    variable0++; 
   
}

void __irq Timer1Done(void)
{
    rSRCPND = BIT_TIMER1;       //Clear pending bit
    rINTPND = BIT_TIMER1;
    rINTPND;
    variable1++;    
}

void __irq Timer2Done(void)
{
    rSRCPND = BIT_TIMER2;       //Clear pending bit
    rINTPND = BIT_TIMER2;
    rINTPND;
    variable2++;    
}

void __irq Timer3Done(void)
{
    rSRCPND = BIT_TIMER3;       //Clear pending bit
    rINTPND = BIT_TIMER3;
    rINTPND;
    variable3++;    
}

void __irq Timer4Done(void)
{
    rSRCPND = BIT_TIMER4;       //Clear pending bit
    rINTPND = BIT_TIMER4;
    rINTPND;
    variable4++;    
}

/******************** Timer DMA Request Test ********************/										
void Test_TimerDma(void)
{
    int i;
    
    rTCFG0 = rTCFG0 & ~(0xffffff) | 0x0; //Dead zone=0, Prescaler0,1 = 0    
    rTCFG1 = rTCFG1 & ~(0xffffff) | 0x3 << 0 | 0x1 << 20; //Mux0 1/16, DMA Request Channel 0
    
    rTCNTB0 = 65500;   
  	rTCMPB0 = 30000;
  	
  	rTCNTB1 = 65500;   
  	rTCMPB1 = 30000;
  	
  	rTCNTB2 = 65500;   
  	rTCMPB2 = 30000;
  	
  	rTCNTB3 = 65500;   
  	rTCMPB3 = 30000;
  	
  	rTCNTB4 = 65500;   
  	
  	rTCON  = rTCON & ~(0xffffff) | 0x1<<1 | 0x1<<9 | 0x1<<13 | 0x1<<17 | 0x1<<21 ; //Manual update 
  	
  	printf("\nTimer 0 DMA Request Test\n\n");
  	
  	rTCON = 0x1 | 0x1<<3; //Auto reload, Start
   	for(i=0 ; i < 4 ; i++)
   	DMA_M2M(0,_NONCACHE_STARTADDRESS,_NONCACHE_STARTADDRESS+0x800000,0x80000,0,0); //byte,single
	rTCON  = 0x0; //Timer Stop

	printf("\nTimer 1 DMA Request Test\n\n");
  	
  	rTCFG1 = rTCFG1 & ~(0xffffff) | 0x3 << 4 | 0x2 << 20; //Mux0 1/16, DMA Request Channel 2
  	rTCON = 0x1<<8 | 0x1<<11; //Auto reload, Start
  	for(i=0 ; i < 4 ; i++) 
   	DMA_M2M(1,_NONCACHE_STARTADDRESS,_NONCACHE_STARTADDRESS+0x800000,0x80000,0,0); //byte,single
	rTCON  = 0x0; 
	
	printf("\nTimer 2 DMA Request Test\n\n");
  	
  	rTCFG1 = rTCFG1 & ~(0xffffff) | 0x3 << 8 | 0x3 << 20; //Mux0 1/16, DMA Request Channel 3
  	rTCON = 0x1<<12 | 0x1<<15; //Auto reload, Start
  	for(i=0 ; i < 4 ; i++) 
   	DMA_M2M(2,_NONCACHE_STARTADDRESS,_NONCACHE_STARTADDRESS+0x800000,0x80000,0,0); //byte,single
	rTCON  = 0x0; 

	printf("\nTimer 3 DMA Request Test\n\n");
  	
  	rTCFG1 = rTCFG1 & ~(0xffffff) | 0x3 << 12 | 0x4 << 20; //Mux0 1/16, DMA Request Channel 0
  	rTCON = 0x1<<16 | 0x1<<19; //Auto reload, Start
  	for(i=0 ; i < 4 ; i++) 
   	DMA_M2M(3,_NONCACHE_STARTADDRESS,_NONCACHE_STARTADDRESS+0x800000,0x80000,0,0); //byte,single
	rTCON  = 0x0; 

	printf("\nTimer 4 DMA Request Test\n\n");
  	
  	rTCFG1 = rTCFG1 & ~(0xffffff) | 0x3 << 16 | 0x5 << 20; //Mux0 1/16, DMA Request Channel 2
  	rTCON = 0x1<<20 | 0x1<<22; //Auto reload, Start
  	for(i=0 ; i < 4 ; i++) 
   	DMA_M2M(4,_NONCACHE_STARTADDRESS,_NONCACHE_STARTADDRESS+0x800000,0x80000,0,0); //byte,single
	rTCON  = 0x0;
	
	printf("Press any key\n");      
	while(!getchar());       //Key board press check      

}

/***** Memory to Memory DMA Service Routine *****/
void DMA_M2M(int ch,int srcAddr,int dstAddr,int tc,int dsz,int burst)
{
    int i,time;
    volatile U32 memSum0=0,memSum1=0;
    DMA *pDMA; //Pointer for DMA Special Register Structure 
    int length;
    
    length=tc*(burst ? 4:1)*((dsz==0)+(dsz==1)*2+(dsz==2)*4); //total data size(byte address)
    //     Counter value * burst size * data size     
    
    printf("[DMA%d MEM2MEM Test]\n",ch);

    switch(ch)
    {
    case 0:
    	pISR_DMA=(int)Dma0Done; //DMA0 interrupt Vector Table Initialzation
    	rINTSUBMSK=~(BIT_SUB_DMA0);
    	rINTMSK&=~(BIT_DMA);    //DMA0 interrupt Enable
    	pDMA=(void *)0x4b000000; //pDMA ponter indicate first addr. of DMA0 Special Register 
    	break;
    case 1:
    	pISR_DMA=(int)Dma1Done; //DMA1 interrupt Vector Table Initialzation
    	rINTSUBMSK=~(BIT_SUB_DMA1);
    	rINTMSK&=~(BIT_DMA);  
    	pDMA=(void *)0x4b000100;
    	break;
    case 2:
		pISR_DMA=(int)Dma2Done; //DMA2 interrupt Vector Table Initialzation
    	rINTSUBMSK=~(BIT_SUB_DMA2);
    	rINTMSK&=~(BIT_DMA);  
    	pDMA=(void *)0x4b000200;
		break;
    case 3:
        pISR_DMA=(int)Dma3Done; //DMA3 interrupt Vector Table Initialzation
       	rINTSUBMSK=~(BIT_SUB_DMA3);
       	rINTMSK&=~(BIT_DMA);  
       	pDMA=(void *)0x4b000300;
        break;
    case 4:
        pISR_DMA=(int)Dma4Done; //DMA3 interrupt Vector Table Initialzation
       	rINTSUBMSK=~(BIT_SUB_DMA4);
       	rINTMSK&=~(BIT_DMA);  
       	pDMA=(void *)0x4b000400;
        break;
    case 5:
        pISR_DMA=(int)Dma5Done; //DMA3 interrupt Vector Table Initialzation
       	rINTSUBMSK=~(BIT_SUB_DMA5);
       	rINTMSK&=~(BIT_DMA);  
       	pDMA=(void *)0x4b000500;
        break;
    }
                                                                                                                            
    printf("DMA%d %8xh->%8xh,size=%xh(tc=%xh),dsz=%d,burst=%d\n",ch,srcAddr,dstAddr,length,tc,dsz,burst);

    printf("Initialize the src.\n");
    
    for(i=srcAddr;i<(srcAddr+length);i+=4)
    {
    	*((U32 *)i)=i^0x55aa5aa5;
    	memSum0+=i^0x55aa5aa5;
    }

    
    printf("DMA%d start\n",ch);
    
    dmaDone=0;
    
    /* Value Setting of Current DMA Special Registers */
    pDMA->DISRC=srcAddr;
    pDMA->DISRCC=(0<<1)|(0<<0); //inc,AHB
    pDMA->DIDST=dstAddr;
    pDMA->DIDSTC=(0<<1)|(0<<0); //inc,AHB
    
    pDMA->DCON=(1<<31)|(1<<30)|(1<<29)|(burst<<28)|(1<<27)|(1<<22)|(dsz<<20)|(tc);
    //HS,AHB,TC interrupt,whole, HWSource request mode,relaod off
    pDMA->DMAREQSEL=(0x9<<1)|(1<<0);
    
    
    pDMA->DMASKTRIG= 0x1<<1; //DMA on,
	
	/* DMA Start */
//    Timer_Start(3);//128us resolution	      
    
    while(dmaDone==0);
    
//    time=Timer_Stop();
    
//    printf("DMA transfer done. time=%f, %fMB/S\n",(float)time/ONESEC3, length/((float)time/ONESEC3)/1000000.);
    
    rINTMSK=BIT_ALLMSK;
    
    /* Check DMA Result */
    for(i=dstAddr;i<dstAddr+length;i+=4)
    {
    	memSum1+=*((U32 *)i)=i^0x55aa5aa5;
    }
    
    printf("memSum0=%x,memSum1=%x\n",memSum0,memSum1);
    if(memSum0==memSum1)
    	printf("DMA test result--------------------------------------O.K.\n");
    else 
    	printf("DMA test result--------------------------------------ERROR!!!\n");
}

/***** DMA Interrupt Service Routine *****/ 
static void __irq Dma0Done(void)
{
    rSUBSRCPND|=BIT_SUB_DMA0;
    ClearPending(BIT_DMA);
    dmaDone=1;
}

static void __irq Dma1Done(void)
{
	rSUBSRCPND|=BIT_SUB_DMA1;
    ClearPending(BIT_DMA);
    dmaDone=1;
}

static void __irq Dma2Done(void)
{
	rSUBSRCPND|=BIT_SUB_DMA2;
    ClearPending(BIT_DMA);
    dmaDone=1;
}

static void __irq Dma3Done(void)
{
	rSUBSRCPND|=BIT_SUB_DMA3;
    ClearPending(BIT_DMA);
    dmaDone=1;
}

static void __irq Dma4Done(void)
{
	rSUBSRCPND|=BIT_SUB_DMA4;
    ClearPending(BIT_DMA);
    dmaDone=1;
}

static void __irq Dma5Done(void)
{
	rSUBSRCPND|=BIT_SUB_DMA5;
    ClearPending(BIT_DMA);
    dmaDone=1;
}



