/*********************************************************************
;* Project Name : S3C2443
;*
;* Copyright 2005 by Samsung Electronics, Inc.
;* All rights reserved.
;*
;* Project Description :
;* This software is only for verifying functions of the S3C2443
;* Anybody can use this code without our permission.
;**********************************************************************
;*
;*	Description	:	Camera
;*	
;*	History
;*			R0.0 (2006.8. ) : Y.C.Kwon draft
;*
;**********************************************************************/

#ifndef __CAMERA_H__
#define __CAMERA_H__

#ifdef __cplusplus
extern "C" {
#endif

#define CAMERA_ITU656				(0)
#define CAMERA_ITU601				(1)

#define UVOFFSET_0					(0)
#define UVOFFSET_128				(1)

#define ORDER_YCBYCR				(0)
#define ORDER_YCRYCB				(1)
#define ORDER_CBYCRY				(2)
#define ORDER_CRYCBY				(3)

#define InFormat_YCBCR420			(0)
#define InFormat_YCBCR422			(1)

#define OutFormat_RGB16B			(0)
#define OutFormat_RGB24B			(1)
#define OutFormat_YCBCR420			(2)
#define OutFormat_YCBCR422			(3)

#define	InvPolPCLK_NORMAL			(0)
#define	InvPolPCLK_INVERSE			(1)

#define	InvPolVSYNC_NORMAL			(0)
#define	InvPolVSYNC_INVERSE			(1)

#define	InvPolHREF_NORMAL			(0)
#define	InvPolHREF_INVERSE			(1)


#define	WINDOWOFFSET_DISABLE		(0)
#define	WINDOWOFFSET_ENABLE			(1)

#define	INTERLEAVE_OFF				(0)
#define	INTERLEAVE_ON				(1)

#define	PREVIEW_CAMERA				(0)
#define	PREVIEW_MSDMA				(1)


#define	TESTPATTERN_NORMAL			(0)
#define	TESTPATTERN_COLORBAR		(1)
#define	TESTPATTERN_HORIZONTAL		(2)
#define	TESTPATTERN_VERTICAL		(3)

#define FLIP_NORMAL					(0)
#define FLIP_XAXIS					(1)
#define FLIP_YAXIS					(2)
#define FLIP_180					(3)

#define	IE_BYPASS					(0)
#define	IE_ARBITRARY				(1)
#define	IE_NEGATIVE					(2)
#define	IE_ARTFREEZE				(3)
#define	IE_EMBOSSING				(4)
#define	IE_SILHOUETTE				(5)

#define MSDMA_InFormat_YCBCR422		(0)
#define MSDMA_InFormat_YCBCR420		(1)


void ResetCameraModule( void);
void EnableImageCapture( void);
void DisableImageCapture( void);
void ResetCameraInterface( void);

void SetCameraSourceFormat( unsigned int ITU601_656, unsigned int UVOffset, unsigned int Order422);
void SetCameraSourceSize( unsigned int SourceHsize, unsigned int SourceVsize);
void SetCameraInterfacePolarity( unsigned int InvPolPCLK, unsigned int InvPolVSYNC, unsigned int InvPolHREF);
void SetCameraWindowOffset( unsigned int WinOfsEn, unsigned int WinHorOfst, unsigned int WinVerOfst, unsigned int WinHorOfst2, unsigned int WinVerOfst2);
void SetCameraTestPattern( unsigned int TestPattern);
void SetImageEffector( unsigned int IE_FIN, unsigned int PAT_Cb, unsigned int PAT_Cr);
void EnableOverflowIRQ( void);
void DisableOverflowIRQ( void);

//*******************************************************************
//*		Functions for Camera Codec Path
//*******************************************************************

void EnableImageCaptureCodec( void);
void DisableImageCaptureCodec( void);
void EnableCodecDMA( void);
void DisableCodecDMA( void);
void SetCodecFormat( unsigned int InFormat, unsigned int OutFormat, unsigned int Interleave, unsigned int Order422);
void SetCodecFrameBuffer( unsigned int FrameBuffer, unsigned int TargetHsize, unsigned int TargetVsize, unsigned int OutFormat, unsigned int Interleave);
void SetCodecScalerBypass( void);
void EnableCodecScaler( void);
void DisableCodecScaler( void);
void SetCodecScaler( unsigned int SRC_Height, unsigned int SRC_Width, unsigned int DST_Height, unsigned int DST_Width);
void SetCodecDMA( unsigned int TargetHsize, unsigned int TargetVsize, unsigned int OutFormat, unsigned int Interleave);
void EnableCodecLastIRQ( void);
void SetCodecFlipMode( unsigned int FlipMd);
unsigned int GetCodecFrameCount( void);
unsigned int GetCodecFrameBuffer( unsigned int FrameNumber);

//*******************************************************************
//*		Functions for Camera Preview Path
//*******************************************************************

void EnableImageCapturePreview( void);
void DisableImageCapturePreview( void);
void SetPreviewSource( unsigned int SEL_DMA_CAM);
void SetPreviewFrameBuffer( unsigned int FrameBuffer, unsigned int TargetHsize, unsigned int TargetVsize, unsigned int OutFormat);
void EnablePreviewScaler( void);
void DisablePreviewScaler( void);
void SetPreviewPath( unsigned int SRC_Height, unsigned int SRC_Width, unsigned int TargetHsize, unsigned int TargetVsize, unsigned int OutFormat);
void EnablePreviewLastIRQ( void);
void SetPreviewCSCRange( unsigned int CSCRange);
void SetPreviewFlipMode( unsigned int FlipMode);
unsigned int GetPreviewFrameCount( void);
unsigned int GetPreviewFrameBuffer( unsigned int FrameNumber);

//*******************************************************************
//*		Functions for MSDMA
//*******************************************************************

void SetMSDMAFormat( unsigned int InFormat, unsigned int Interleave, unsigned int Order422);
void SetMSDMAFrameBuffer( unsigned int FrameBuffer, unsigned int SRC_Width, unsigned int SRC_Height, unsigned int SRC_Offset, unsigned int InFormat);
void StartMSDMA( void);
void WaitMSDMA( void);


#ifdef __cplusplus
}
#endif

#endif /*__CAMERA_H__*/

