/*======================================================================

 Project Name : S3C2443

 Copyright 2006 by Samsung Electronics, Inc.
 All rights reserved.

 Project Description :
 This software is only for verifying functions of the S3C2443. 
 Anybody can use this code without our permission.

 File Name    : OneNAND.c
 Description  : S3C2443 OneNAND access module
 Author       : Junon Jeon
 Dept         : AP
 Created Date : 2006.06.02
 Version      : 0.0
 History
   R0.0 (2006.06.02): Junon draft
		- This code is derived from S5I3000 OneNAND driver.
		
=======================================================================*/

#include <stdio.h>
#include <string.h>

#include "2443addr.h"
#include "Option.h"
#include "OneNand.h"
#include "System.h"
#include "Console.h"
#include "PLL.h"

// Setting address of OneNAND device
//****** Setting base address by using CSn ******
#define OND_BANK				0 	// 0:nCS0 ~ 5:nCS5
#define	__OND_BASEADDR			(OND_BANK * 0x08000000)	
#define __OND_BootRAM_BASE		(__OND_BASEADDR + 0x0)
#define __OND_DataRAM0_BASE		(__OND_BASEADDR + 0x0400)
#define __OND_DataRAM1_BASE		(__OND_BASEADDR + 0x0c00)

#define __OND_BootRAM_Spare		(__OND_BASEADDR + 0x10000)
#define __OND_DataRAM0_Spare	(__OND_BASEADDR + 0x10020)
#define __OND_DataRAM1_Spare	(__OND_BASEADDR + 0x10060)

// Register address of OneNAND device
#define	rOND_MANU				(*(volatile U16 *)(__OND_BASEADDR + 0x1e000))
#define	rOND_DEV				(*(volatile U16 *)(__OND_BASEADDR + 0x1e002))
#define	rOND_VER				(*(volatile U16 *)(__OND_BASEADDR + 0x1e004))
#define	rOND_DBUFSIZE			(*(volatile U16 *)(__OND_BASEADDR + 0x1e006))
#define	rOND_BBUFSIZE			(*(volatile U16 *)(__OND_BASEADDR + 0x1e008))
#define	rOND_BUFAMOUNT			(*(volatile U16 *)(__OND_BASEADDR + 0x1e00a))
#define	rOND_TECH				(*(volatile U16 *)(__OND_BASEADDR + 0x1e00c))

#define	rOND_START_ADDR1		(*(volatile U16 *)(__OND_BASEADDR + 0x1e200))
#define	rOND_START_ADDR2		(*(volatile U16 *)(__OND_BASEADDR + 0x1e202))
#define	rOND_START_ADDR3		(*(volatile U16 *)(__OND_BASEADDR + 0x1e204))
#define	rOND_START_ADDR4		(*(volatile U16 *)(__OND_BASEADDR + 0x1e206))
#define	rOND_START_ADDR5		(*(volatile U16 *)(__OND_BASEADDR + 0x1e208))
#define	rOND_START_ADDR6		(*(volatile U16 *)(__OND_BASEADDR + 0x1e20a))
#define	rOND_START_ADDR7		(*(volatile U16 *)(__OND_BASEADDR + 0x1e20c))
#define	rOND_START_ADDR8		(*(volatile U16 *)(__OND_BASEADDR + 0x1e20e))

#define	rOND_START_BUFFER		(*(volatile U16 *)(__OND_BASEADDR + 0x1e400))
#define	rOND_COMMAND			(*(volatile U16 *)(__OND_BASEADDR + 0x1e440))
#define	rOND_SYS_CONFIG1		(*(volatile U16 *)(__OND_BASEADDR + 0x1e442))
#define	rOND_SYS_CONFIG2		(*(volatile U16 *)(__OND_BASEADDR + 0x1e444))
#define	rOND_CON_STATUS			(*(volatile U16 *)(__OND_BASEADDR + 0x1e480))
#define	rOND_INT_STATUS			(*(volatile U16 *)(__OND_BASEADDR + 0x1e482))
#define	rOND_START_BADDR		(*(volatile U16 *)(__OND_BASEADDR + 0x1e498))
#define	rOND_WPROT_STATUS		(*(volatile U16 *)(__OND_BASEADDR + 0x1e49c))

#define rOND_ECC_STATUS			(*(volatile U16 *)(__OND_BASEADDR + 0x1fe00))
#define rOND_ECC_RES_M0			(*(volatile U16 *)(__OND_BASEADDR + 0x1fe02))
#define rOND_ECC_RES_S0			(*(volatile U16 *)(__OND_BASEADDR + 0x1fe04))
#define rOND_ECC_RES_M1			(*(volatile U16 *)(__OND_BASEADDR + 0x1fe06))
#define rOND_ECC_RES_S1			(*(volatile U16 *)(__OND_BASEADDR + 0x1fe08))
#define rOND_ECC_RES_M2			(*(volatile U16 *)(__OND_BASEADDR + 0x1fe0a))
#define rOND_ECC_RES_S2			(*(volatile U16 *)(__OND_BASEADDR + 0x1fe0c))
#define rOND_ECC_RES_M3			(*(volatile U16 *)(__OND_BASEADDR + 0x1fe0e))
#define rOND_ECC_RES_S3			(*(volatile U16 *)(__OND_BASEADDR + 0x1fe10))

// Values of Command Register - CMD 
#define OND_LD_MS_TO_BUF	0x0000
#define OND_LD_SS_TO_BUF	0x0013
#define OND_PG_MS_FR_BUF	0x0080
#define OND_PG_SS_FR_BUF	0x001a
#define OND_CP_BACK_PRG		0x001b
#define OND_UNLOCK			0x0023
#define OND_LOCK			0x002a
#define OND_LOCK_TIGHT		0x002c
#define OND_ERASE_VERIFY	0x0071
#define OND_BLK_ERASE		0x0094
#define OND_MULTY_ERASE		0x0095
#define	OND_ERASE_SUS		0x00b0
#define OND_ERASE_RESUME	0x0030
#define OND_RST_CORE		0x00f0
#define OND_RST_OND			0x00f3
#define	OND_OTP_ACS			0x0065

#define	DBS(b)		((b<<5)&0x8000)		// BufferRAM of DDP (Device BufferRAM Select)
#define	DFS(b)		((b<<5)&0x8000)		// flash core of DDP (Device Flash Core Select)
#define	FBA(b)		(b&0x03ff)			// NAND Flash Block Address	
#define	FPA(p)		((p<<2)&0xfc)		// NAND Flash Page Address
#define	FSA(s)		(s&0x3)				// NAND Flash Sector Address
#define	BSA(r)		((r<<8)&0x0f00)		// BufferRAM Sector Address
#define	BSC(n)		(n&0x3)				// BufferRAM Sector Count
#define	SBA(b)		(b&0x03ff)			// Start Block Address

// Others
#define	DATABUFFERSIZE	((rOND_DBUFSIZE<<1)/(rOND_BUFAMOUNT>>8))
#define USE_DMA	0


// OneNAND interface library
void OND_HotReset( void);
U32 OND_erase_block( U16);
U32 OND_erase_multi_block_verify( U16, U16, U16);
U32 OND_page_program( U16, U16);
U32 OND_page_load( U16, U16);
void OND_lock_block( U16);
void OND_lock_tight_block( U16);
void OND_unlock_block( U16);
void OND_read( U32 *, U32, U16);
void OND_write( U32 *, U32, U16);

void OneNAND_TimingCheck(void);
void OneNAND_Burst_Read(void);

// Temporary library
void set_dma_control(U8 burst, U8 tw);
void dma_memcpy(U32 s_addr, U32 d_addr, U32 N_LLI, U32 trans_size);
void SMC_Config4OneNAND(U32 nBank);

	
void * function_OND[][2]=
{	
	(void *)OneNAND_ReadID,					"OneNAND Read ID  ",
	(void *)OneNAND_Program,				"OneNAND program  ",
	(void *)OneNAND_Read_Page,				"Read page        ",
	(void *)OneNAND_Write_Page,				"Write page       ",
	(void *)OneNAND_Erase_All,				"Erase all blocks ",
	(void *)OneNAND_Erase_Block,			"Erase block      ",
	(void *)OneNAND_Unlock_Block,			"Unlock block     ",
	(void *)OneNAND_Lock_Block,				"Lock block       ",
	(void *)OneNAND_Performance_Test,		"Performance Test ",
	(void *)OneNAND_TimingCheck,			"Timing Check     ",
	(void *)OneNAND_Burst_Read,				"Read burst(page) ",
	0,0
};

void Test_OneNAND( void)
{
	int	i;

	OND_HotReset();
	SMC_Config4OneNAND(OND_BANK);

	while(1)
	{
		i=0;
		
		while(1)
		{
			printf("%2d:%s",i,function_OND[i][1]);
			i++;
			if((int)(function_OND[i][0])==0)
			{
				printf("\n");
				break;
			}
			if((i%4)==0)
			printf("\n");
		}

		printf("\nPress Enter key to exit : ");

		i = GetIntNum();
		if( i == (-1) ) break;
		if( (i>=0) && (i<sizeof(function_OND)/8-1) )
			( (void (*)(void)) (function_OND[i][0]) )();
	}	
}


//========================= [Drivers] ============================
void OND_HotReset( void)
{
	rOND_INT_STATUS	=	0x0;
	rOND_COMMAND	=	OND_RST_OND;
	
	while ((rOND_INT_STATUS & 0x8010) != 0x8010);	
}


U32 OND_erase_block( U16 block)
{

	rOND_START_ADDR1 = DFS(block) | FBA(block);
	
	rOND_INT_STATUS = 0;
	rOND_COMMAND = OND_BLK_ERASE;

	while (!(rOND_INT_STATUS & (1<<15)));
	
	if (rOND_CON_STATUS & (1<<10)) {
		printf("Fail to erase the block %d\n", block);
		return 0x3;
	}
	
	return 0x0;
}


U32 OND_erase_multi_block_verify( U16 start_block, U16 end_block, U16 do_verify)
{
	U16	block;

	if (end_block < start_block) return 0x1;

	for (block = start_block; block <= end_block; block++)
	{
		OND_unlock_block( block);
		
		rOND_START_ADDR1 = FBA(block);

		rOND_INT_STATUS = 0;
		rOND_COMMAND = OND_MULTY_ERASE;
		
		while (!(rOND_INT_STATUS & (1<<15)));
	}

	rOND_START_ADDR1 = FBA(end_block);
	
	rOND_INT_STATUS = 0;
	rOND_COMMAND = OND_BLK_ERASE;

	while (!(rOND_INT_STATUS & (1<<15)));

	if (do_verify)
	{
		for (block = start_block; block <= end_block; block++)
		{
		
			rOND_START_ADDR1 = FBA(block);
			
			rOND_INT_STATUS = 0;
			rOND_COMMAND = OND_ERASE_VERIFY;
			
			while (!(rOND_INT_STATUS & (1<<15)));

			if (rOND_CON_STATUS & (1<<10)) 
				printf(" Fail to erase-verify the block %d\n", block);
		}
	}
	
	return 0x0;
}


U32 OND_page_program( U16 block, U16 page)
{
		
	rOND_START_ADDR2 = DBS(block);

	rOND_START_ADDR1 = DFS(block) | FBA(block);

	rOND_START_ADDR8 = FPA(page) | FSA(0);

	rOND_START_BUFFER = BSA(0x8) | BSC(0);
	
	rOND_INT_STATUS = 0;
	rOND_COMMAND = OND_PG_MS_FR_BUF;
	
	while (!(rOND_INT_STATUS & (1<<15)));
	
	if (rOND_CON_STATUS & (1<<10)) {
		printf("Fail to program the page (B: %d, P: %d)\n", block, page);
		return 0x1;
	}
	
	return 0x0;
}


U32 OND_page_load( U16 block, U16 page)
{
	
	rOND_START_ADDR1 = DFS(block) | FBA(block);

	rOND_START_ADDR8 = FPA(page) | FSA(0);

	rOND_START_BUFFER = BSA(0x8) | BSC(0);

	rOND_START_ADDR2 = DBS(block);

	rOND_INT_STATUS = 0;
	rOND_COMMAND = OND_LD_MS_TO_BUF;

	while (!(rOND_INT_STATUS & (1<<15)));

	if (rOND_CON_STATUS & (1<<10)) {
		printf("Fail to load the page (B: %d, P: %d)\n", block, page);
		return 0x1;
	}
	
	return 0x0;
}


void OND_write_protection_block( U16 cmd, U16 block)
{

	rOND_START_ADDR1 = DFS(block) | FBA(block);

	rOND_START_BADDR = SBA(block);	
	
	rOND_INT_STATUS = 0;
	rOND_COMMAND = cmd;

	while (!(rOND_INT_STATUS & (1<<15)));
}


void OND_lock_block( U16 block)
{
	OND_write_protection_block( OND_LOCK, block);
}


void OND_lock_tight_block( U16 block)
{
	OND_write_protection_block( OND_LOCK_TIGHT, block);
}


void OND_unlock_block( U16 block)
{
	OND_write_protection_block( OND_UNLOCK, block);
}


//	Read-While-Load Operation
void OND_read( U32 *Address, U32 Size, U16 block)
{
	U32	size;
	U16	page;
	U16	b_temp;
	U32	data_load;
	U32	data_read;
//	U32	st_cnt = 1;
	

	b_temp = 0xffff;
	page = 0;
	data_load = 0;
	data_read = 0;
	
	if (!Size) {
		printf("The size is zero!! Retry this operation!! \n");	
		return;
	}

	while(Size > (data_read*DATABUFFERSIZE)) {  

		if (b_temp!=0xffff) {  // check int pending
			while (!(rOND_INT_STATUS & (1<<15)));

			if (rOND_CON_STATUS & (1<<10)) {
				printf("ECC Fail.\n");
			} else
				data_load++;
		}

		if (Size > (data_load*DATABUFFERSIZE)) {  // data load

			rOND_START_ADDR1 = DFS(block) | FBA(block);

			rOND_START_ADDR8 = FPA(page) | FSA(0);

			if (data_load&1)
				rOND_START_BUFFER = BSA(0xc) | BSC(0); // DataRAM1
			else
				rOND_START_BUFFER = BSA(0x8) | BSC(0); // DataRAM0

			rOND_START_ADDR2 = DBS(block);

			rOND_INT_STATUS = 0;

			rOND_COMMAND = OND_LD_MS_TO_BUF;

		}

		if (data_load > data_read) {  // data read. 105 us @50MHz 8,16,cont.-words

			rOND_START_ADDR2 = DBS(b_temp);
		
			if ((Size-(data_read*DATABUFFERSIZE)) >= DATABUFFERSIZE) // edit by junon
				size = DATABUFFERSIZE;
			else
				size = Size-(data_read*DATABUFFERSIZE);
			
// for memory copy time check			
//if (data_load == 3)
//	StartStopwatch();
//			printf("Address = 0x%x\n", (U32)Address);

#if USE_DMA
			if (data_read&0x1) // 87 us @50MHz 8,16,cont.-words
				dma_memcpy( __OND_DataRAM1_BASE, (U32)Address, 0, size );
			else
				dma_memcpy( __OND_DataRAM0_BASE, (U32)Address, 0, size );
#else
			if (data_read&0x1) // 87 us @50MHz 8,16,cont.-words
				memcpy( Address, (U32 *)__OND_DataRAM1_BASE, size );
			else
				memcpy( Address, (U32 *)__OND_DataRAM0_BASE, size );
#endif

// for memory copy time check			
//if (data_load == 3)
//{
//	st_cnt = EndStopwatch();
//	printf("\n load cycle time : %d us", st_cnt);
//}

			Address+=(DATABUFFERSIZE>>2);
			data_read++;
		}

		b_temp = block;		
		page = (page+1)&0x3f;
		if (page==0) block++;
	}
}


//	Write-While-Program Operation, unit : page
void OND_write( U32 *Address, U32 Size, U16 block)
{
	U16	page;
	U32	data_program;
	U32	data_write;

	page = 0;
	data_program = 0;
	data_write = 0;

	if (!Size) {
		printf("The size is zero!! Retry this operation!! \n");	
		return;
	}

	while( Size >= (data_program*DATABUFFERSIZE)) { // edit by junon

		if (data_write > 0) {

			if (page==0) {
				OND_unlock_block(block);
				OND_erase_block(block);
			}

			rOND_START_ADDR1 = DFS(block) | FBA(block);
			rOND_START_ADDR8 = FPA(page) | FSA(0);

			if (data_program&1)	
				rOND_START_BUFFER = BSA(0xc) | BSC(0);
			else				
				rOND_START_BUFFER = BSA(0x8) | BSC(0);

			rOND_INT_STATUS = 0;
			rOND_COMMAND = OND_PG_MS_FR_BUF;

			page = (page+1) & 0x3f;
			if (page==0) block++;
		}
		
		// write data to DataRAM0 or 1
		if (Size >= (data_write*DATABUFFERSIZE)) {  // edit by junon
			if (data_write <= (data_program+1)) {

				rOND_START_ADDR2 = DBS(block);
#if USE_DMA
				if (data_write&1)
					dma_memcpy( (U32)Address, __OND_DataRAM1_BASE, 0, DATABUFFERSIZE );
				else
					dma_memcpy( (U32)Address, __OND_DataRAM0_BASE, 0, DATABUFFERSIZE );
#else
				if (data_write&1)
					memcpy( (U32 *)__OND_DataRAM1_BASE, Address, DATABUFFERSIZE);
				else
					memcpy( (U32 *)__OND_DataRAM0_BASE, Address, DATABUFFERSIZE);
#endif

				Address+=(DATABUFFERSIZE>>2);
				data_write++;
			}
		}

		if (data_write > 1) {
			while (!(rOND_INT_STATUS & (1<<15)));

			if (rOND_CON_STATUS & (1<<10))
				printf("ECC Fail.\n");
			else			
				data_program++;
		}
	}
}


//======================== [ OneNAND test code ] ===========================
void OneNAND_ReadID( void)
{
	printf("\n\n< Read OneNAND information >\n");
	printf("Manufacturer ID       : 0x%02x\n",rOND_MANU);
	printf("Device ID             : 0x%02x\n",rOND_DEV);
	printf("Version ID            : 0x%02x\n",rOND_VER);
	printf("Data buffer size      : %d KB\n",((rOND_DBUFSIZE<<1)/(rOND_BUFAMOUNT>>8))>>10);
	printf("Boot buffer size      : %d KB\n",((rOND_BBUFSIZE<<1)/(rOND_BUFAMOUNT&0xff))>>10);
	printf("Amount of D/B buffers : %d, %d\n",rOND_BUFAMOUNT>>8,rOND_BUFAMOUNT&0xff);
	printf("Technology            : 0x%02x\n\n", rOND_TECH);
}


void OneNAND_Program( void)
{
	U16	block;
	U32	size;
	
	printf("\nOneNAND block program from [0x%08x]\n",(_NONCACHE_STARTADDRESS+0x00500000));

	printf("Enter block # to program : ");
	block = GetIntNum();

	printf("Enter size to program : ");
	size = GetIntNum();

	OND_write( (U32 *)(_NONCACHE_STARTADDRESS+0x00500000), size, block);

	printf("Program complete.\n");
}


void OneNAND_Read_Page( void)
{
	U16	block=0;
	U16	page=0;
	U32	i;
	U32	*Addr;

	printf("\nOneNAND block read\n");

	printf("Enter block # to read : ");
	block = GetIntNum();

	printf("Enter page # to read : ");
	page = GetIntNum();

	if (OND_page_load(block,page)==0) {

		Addr=(U32 *)__OND_DataRAM0_BASE;

		for(i=0;i<DATABUFFERSIZE;i+=sizeof(U32)) {

			if ((i&0x1f)==0)
				printf("\n%08x : ",i);

			printf("%08x ",*Addr++);
		
		}
		putchar('\n');
	}
}


void OneNAND_Write_Page( void)
{
	U16	block=0;
	U16	page=0;

	printf("\nOneNAND block write\n");

	printf("Enter block # to write : ");
	block = GetIntNum();

	printf("Enter page # to write : ");
	page = GetIntNum();

	if (OND_page_program(block,page)==0)
		printf("Block %d, page %d was writed.\n",block,page);

	printf("\n");
}


void OneNAND_Erase_All( void)
{
	U16	block;
	
	printf("\nOneNAND all block erase\n");
	
	for(block=0;block<1024;block++) {
		OND_unlock_block( block);
		OND_erase_block( block);
		if ((block&0x3f)==0)
			printf("[%d]",block);
	}

	printf("\nAll block was erased.\n");
	
	printf("\n");
}


void OneNAND_Erase_Block( void)
{
	U16	block=0;

	printf("\nOneNAND block erase\n");

	printf("Enter block # to erase : ");
	block = GetIntNum();
	
	OND_unlock_block( block);
	
	if (OND_erase_block( block)==0)
		printf("Block %d was erased.\n",block);
	
	printf("\n");
}


void OneNAND_Unlock_Block( void)
{
	U16	block=0;

	printf("\nOneNAND block unlock\n");

	printf("Enter block # to unlock : ");
	block = GetIntNum();
	
	OND_unlock_block( block);

	printf("\n");
}


void OneNAND_Lock_Block( void)
{
	U16	block=0;

	printf("\nOneNAND block lock\n");

	printf("Enter block # to lock : ");
	block = GetIntNum();
	
	OND_lock_block( block);
		
	printf("\n");
}


void SMC_Config4OneNAND(U32 nBank)
{
  	U32 sel_sync = 2;
  	U32 sel_burst = 4; 
  	U32 burst_mode = 1;
  	U32 mux_mode = 0;
	U16 sysconfig1;
	volatile SMC_REGS *pSmcRegs;
	
	pSmcRegs = (volatile SMC_REGS *)(SMC_REG_BASE+SMC_REG_OFFSET*nBank);

	printf("Select OneNAND interface mode, Demuxed mode(0-D)/Muxed mode(1) : ");
	mux_mode = GetIntNum();
	if (mux_mode == 1) 
		rSMBONETYPER |= (1<<nBank); // muxed mode
	else 
		rSMBONETYPER &= ~(1<<nBank); // demuxed mode

	printf("Select sync mode, Synchronous(1)/Asynchronous(2-D) : ");
	sel_sync = GetIntNum();
	if (sel_sync == 1)	
	{
		printf("Select burst mode, no burst(0-D)/4-words(1)/8-words(2)/16-words(3)/Continuous(4) : ");
		sel_burst = GetIntNum();
		burst_mode = 1;
		if ( (sel_burst == (U32)(-1)) || (sel_burst == 0) )
		{
			sel_burst = 4;			
			burst_mode = 0; // default no burst
		}
	}
		
	// defualt setting for reading correct rOND_SYS_CONFIG1
	pSmcRegs->rSmbWstRdr = 3;	// read wait state. [4:0] 1 ~ 10. only 4 @ 50MHz(Mux). sync burst only 4
	pSmcRegs->rSmbWstWrr = 3;	// write wait state. [4:0] 1 ~ 10. min 3 @ 50MHz
	pSmcRegs->rSmbWstOenr = 2;	// OE assertion delay. [3:0] 1 ~ 2. min 3 @ 50MHz
	pSmcRegs->rSmbWstWenr = 2;	// WE assertion delay. [3:0] 1 ~ 3. min 3 @ 50MHz

	sysconfig1=rOND_SYS_CONFIG1; // reading 
	printf("rOND_SYS_CONFIG1 = 0x%x \n", sysconfig1);

	//  ensure that ther are no memory transfers.
	rCLKDIV0 = (rCLKDIV0 & ~(1<<3)) | (1<<3); // select HALFHCLK : 0=HCLK, 1=HCLK/2
	rSMCCR = (rSMCCR & ~(3<<1)) | (1<<1)|(1); // SMC MemClkRatio setting should be equal HALFHCLK of CLKDIV0
	pSmcRegs->rSmbIdCyr = 0;
	printf("rCLKDIV0 = 0x%x\n", rCLKDIV0);
	printf("rSMBIDCYR%d = 0x%x\n", nBank, pSmcRegs->rSmbIdCyr);
	//  wait 4 SMC clocks

	switch(sel_sync) {
		case 1: // sync.
			printf("rSMBCR%d = 0x%x \n", nBank, pSmcRegs->rSmbCr);
			// rOND_SYS_CONFIG1
			// [15] : sync(1), async(0), 
			// [12:14] : Burst read latency. same as WSTRD value in demux. 
			// [9:11] : Continuous(0), 4word(1), 8word(2), 16word(3), 32word(4) - only burst
			rOND_SYS_CONFIG1 = (sysconfig1& ~(0x7f<<9))|(1<<15)|(3<<12)|((sel_burst%4)<<9);

			Delay(1);
			printf("sync = %d, sel burst = %d, burst mode = %d\n", sel_sync, sel_burst, burst_mode);
		
			// rSMBCR0
			// [10:11] : 4word(0), 8word(1), 16word(2), continuous(3) - only burst
			// [9] : sync(1), async(0)
			// [8] : burst(1), no burst(0)
			pSmcRegs->rSmbCr = (pSmcRegs->rSmbCr & ~((0xf<<8)|(0xf<<16)|(1<<14)|0x3)) 
					|((sel_burst-1)<<10)|(1<<9)|(burst_mode<<8) 	// set SSMC as read burst. 
					|(0<<18)|(0<<17)|(0<<16) 	// set SSMC write 
					|(0<<14)|(0<<1)|(1); // caution! SMWAIT polarity setting. now active low.
			pSmcRegs->rSmbWstBrdr = 0;
			printf("rSMBONETYPER = 0x%x\n",rSMBONETYPER);
			printf("\nSynchronous access mode \n");

			if (mux_mode)
			{   				
				pSmcRegs->rSmbWstRdr = 3;	// read wait state. [4:0] 1 ~ 10. only 4 @ 50MHz(Mux). sync burst only 4
				pSmcRegs->rSmbWstWrr = 3;	// write wait state. [4:0] 1 ~ 10. min 3 @ 50MHz
				pSmcRegs->rSmbWstOenr = 2;	// OE assertion delay. [3:0] 1 ~ 2. min 3 @ 50MHz
				pSmcRegs->rSmbWstWenr = 2;	// WE assertion delay. [3:0] 1 ~ 3. min 3 @ 50MHz
			}
			else // previous setting 3,3,1,2
			{
				pSmcRegs->rSmbWstRdr = 3;	// read wait state. [4:0] 1 ~ 10. only 4 @ 50MHz(Mux). sync burst only 4
				pSmcRegs->rSmbWstWrr = 3;	// write wait state. [4:0] 1 ~ 10. min 3 @ 50MHz
				pSmcRegs->rSmbWstOenr = 2;	// OE assertion delay. [3:0] 1 ~ 2. min 3 @ 50MHz
				pSmcRegs->rSmbWstWenr = 2;	// WE assertion delay. [3:0] 1 ~ 3. min 3 @ 50MHz
			}
		break;
		default: // async.
			rOND_SYS_CONFIG1 = (sysconfig1& ~(0xf<<12))|(0<<15)|(3<<12);
			pSmcRegs->rSmbCr = (pSmcRegs->rSmbCr & ~(0xf<<8))|(0<<10)|(0<<9)|(0<<8); 	// set SSMC as async read. 
			printf("Asynchronous access mode \n");

			// defualt setting 3,3,2,3
			pSmcRegs->rSmbWstRdr = 3;	// read wait state. [4:0] 1 ~ 10. min 3 @ 50MHz(Mux)
			pSmcRegs->rSmbWstWrr = 3;	// write wait state. [4:0] 1 ~ 10. min 3 @ 50MHz
			pSmcRegs->rSmbWstOenr = 2;	// OE assertion delay. [3:0] 1 ~ 2. min 2 @ 50MHz
			pSmcRegs->rSmbWstWenr = 2;	// WE assertion delay. [3:0] 1 ~ 3. min 3 @ 50MHz
		break;
	}

	sysconfig1=rOND_SYS_CONFIG1;
	printf("rOND_SYS_CONFIG1 = 0x%x \n", sysconfig1);
	printf("rSMBCR%d = 0x%x \n", nBank, pSmcRegs->rSmbCr);
	printf("rSMBWSTRDR%d = 0x%x\n", nBank, pSmcRegs->rSmbWstRdr);	
	printf("rSMBWSTWRR%d = 0x%x\n", nBank, pSmcRegs->rSmbWstWrr);	
	printf("rSMBWSTOENR%d = 0x%x\n", nBank, pSmcRegs->rSmbWstOenr);	
	printf("rSMBWSTWENR%d = 0x%x\n\n", nBank, pSmcRegs->rSmbWstWenr);	
//	rSMCCR = (1<<1)|(1<<0);
}


void OneNAND_Performance_Test(void)
{
	U16	block=0;
	U32	size=0;
	U32	i;
	U32	*SrcAddr, *DstAddr;
	U32	t_cnt=1;
	U8	d_correct=0;
	
#if USE_DMA
	U8	dma_burst;
	U8	dma_tw;
#endif
	
	SetResStopwatch(1000); // stopwatch 
	
//	rGPDCON = rGPDCON & ~(3<<22) | (1<<22); // GPD11 output setn - B'd TPD01
//	rGPDDAT |= (1<<11); // high

	// SSMC parameter value setting 
	SMC_Config4OneNAND(OND_BANK);
	
	printf("\nrOND_SYS_CONFIG1 = 0x%x \n", rOND_SYS_CONFIG1);
	printf("rOND_DBUFSIZE = 0x%x \n", rOND_DBUFSIZE);
	printf("rOND_BBUFSIZE = 0x%x \n", rOND_BBUFSIZE);
	printf("rOND_BUFAMOUNT = 0x%x \n", rOND_BUFAMOUNT);
	
	printf("\n Enter start block # to write [D:700]: ");
	block = GetIntNum();
	printf(" Enter data size # to write [D:2MB, MAX:10MB]: ");
	size = GetIntNum();
	
#if USE_DMA
	printf(" Enter dma transfer width [0]1byte, [1]2byte, [2]4byte : ");
	dma_tw = (U8)GetIntNum();
	printf("\n Enter dma burst size [0]1, [1]4, [2]8, [3]16, [4]32, [5]64, [6]128, [7]256 : ");
	dma_burst = (U8)GetIntNum();
	set_dma_control(dma_burst, dma_tw);
#endif

	// Default test block variable setting
	if ( (block == (U16)(-1)) || (block<1) )
		block 	= 700; // start block
	if ( (size == (U32)(-1)) || (size<2048) )	
		size 	= 0x200000; // 16 blocks = 2MB
	SrcAddr	= (U32 *)(_NONCACHE_STARTADDRESS+0x00500000);
	printf(" Start block : %d, Data size : %d Bytes\n", block, size);

	// fill buffer data
	for (i=0; i<size ;i+=4)
		*SrcAddr++ = i;
	
//	rGPDDAT &= ~(1<<11); // low
//	Delay(1);
//	rGPDDAT |= (1<<11); // high
	// write test by block unit(internally page unit), Write-While-Program Operation
	StartStopwatch();
	OND_write( (U32 *)(_NONCACHE_STARTADDRESS+0x00500000), size, block);
	t_cnt = EndStopwatch();
//	rGPDDAT &= ~(1<<11); // low
//	Delay(2);
//	rGPDDAT |= (1<<11); // high
	printf("\n %d Bytes WRITE time by block unit : %d us", size, t_cnt);
	printf("\n %.3f Byte/sec\n", (float)size/(float)t_cnt*1000000 ); // for us order

	printf("Press any key to continue\n");
	getchar();
	
	// read test by block unit(internally page unit), Read-While-Load Operation
	StartStopwatch();
	OND_read( (U32 *)(_NONCACHE_STARTADDRESS+0x01000000), size, block);
	t_cnt = EndStopwatch();
	printf("\n %d Bytes READ time by block unit : %d us", size, t_cnt);
	printf("\n %.3f Byte/sec\n", (float)size/(float)t_cnt*1000000 );

	// verify data	
	SrcAddr = (U32 *)(_NONCACHE_STARTADDRESS+0x00500000);
	DstAddr = (U32 *)(_NONCACHE_STARTADDRESS+0x01000000);
	for (i=0; i<size ;i+=4)
		if (*SrcAddr++ != *DstAddr++)
			d_correct = 1;
	if(d_correct == 1)
		printf("\n Copied data error!! \n\n");
	else printf("\n Copied data OK!! \n\n");

}


void OneNAND_TimingCheck( void)
{
	U32 R1, R2, R3, R4, W1, W2, W3, W4;
	volatile SMC_REGS *pSmcRegs;
	
	pSmcRegs = (volatile SMC_REGS *)(SMC_REG_BASE+SMC_REG_OFFSET*OND_BANK);
	
	printf("Read, write test \n");

	pSmcRegs->rSmbWstRdr = 4;	// read wait state. [4:0] 1 ~ 10. only 4 @ 50MHz(Mux). sync burst only 4
	pSmcRegs->rSmbWstWrr = 3;	// write wait state. [4:0] 1 ~ 10. min 3 @ 50MHz
	pSmcRegs->rSmbWstOenr = 1;	// OE assertion delay. [3:0] 1 ~ 2. min 3 @ 50MHz
	pSmcRegs->rSmbWstWenr = 3;	// WE assertion delay. [3:0] 1 ~ 3. min 3 @ 50MHz
	R1 = rOND_DEV; // read
	rOND_START_ADDR1 = 0x8355; // write, addr1 is effective about 0~8,15bit
	W1 = rOND_START_ADDR1; // read
	
	pSmcRegs->rSmbWstRdr = 10;	// read wait state. [4:0] 1 ~ 10. only 4 @ 50MHz(Mux). sync burst only 4
	pSmcRegs->rSmbWstWrr = 10;	// write wait state. [4:0] 1 ~ 10. min 3 @ 50MHz
	pSmcRegs->rSmbWstOenr = 5;	// OE assertion delay. [3:0] 1 ~ 2. min 3 @ 50MHz
	pSmcRegs->rSmbWstWenr = 5;	// WE assertion delay. [3:0] 1 ~ 3. min 3 @ 50MHz
	R2 = rOND_DEV; // read
	rOND_START_ADDR1 = 0x8355; // write
	W2 = rOND_START_ADDR1; // read

	pSmcRegs->rSmbWstRdr = 4;	// read wait state. [4:0] 1 ~ 10. only 4 @ 50MHz(Mux). sync burst only 4
	pSmcRegs->rSmbWstWrr = 3;	// write wait state. [4:0] 1 ~ 10. min 3 @ 50MHz
	pSmcRegs->rSmbWstOenr = 2;	// OE assertion delay. [3:0] 1 ~ 2. min 3 @ 50MHz
	pSmcRegs->rSmbWstWenr = 2;	// WE assertion delay. [3:0] 1 ~ 3. min 3 @ 50MHz
	R3 = rOND_DEV; // read
	rOND_START_ADDR1 = 0x8355; // write
	W3 = rOND_START_ADDR1; // read

	pSmcRegs->rSmbWstRdr = 7;	// read wait state. [4:0] 1 ~ 10. only 4 @ 50MHz(Mux). sync burst only 4
	pSmcRegs->rSmbWstWrr = 7;	// write wait state. [4:0] 1 ~ 10. min 3 @ 50MHz
	pSmcRegs->rSmbWstOenr = 1;	// OE assertion delay. [3:0] 1 ~ 2. min 3 @ 50MHz
	pSmcRegs->rSmbWstWenr = 3;	// WE assertion delay. [3:0] 1 ~ 3. min 3 @ 50MHz
	R4 = rOND_DEV; // read
	rOND_START_ADDR1 = 0x8355; // write
	W4 = rOND_START_ADDR1; // read

	printf("R1 = 0x%x, R2 = 0x%x, R3 = 0x%x, R4 = 0x%x\n",R1,R2,R3,R4);
	printf("W1 = 0x%x, W2 = 0x%x, W3 = 0x%x, W4 = 0x%x\n",W1,W2,W3,W4);

	printf("\n\n< Read OneNAND information >\n");
	printf("Manufacturer ID       : 0x%02x\n",rOND_MANU);
	printf("Device ID             : 0x%02x\n",rOND_DEV);
	printf("Version ID            : 0x%02x\n",rOND_VER);
	printf("Data buffer size      : %d KB\n",((rOND_DBUFSIZE<<1)/(rOND_BUFAMOUNT>>8))>>10);
	printf("Boot buffer size      : %d KB\n",((rOND_BBUFSIZE<<1)/(rOND_BUFAMOUNT&0xff))>>10);
	printf("Amount of D/B buffers : %d, %d\n",rOND_BUFAMOUNT>>8,rOND_BUFAMOUNT&0xff);
	printf("Technology            : 0x%02x\n\n", rOND_TECH);
}


void OneNAND_Burst_Read( void)
{
	U16	block=0;
	U16	page=0;
	U32	i;
	U32	*Addr;

	printf("\nOneNAND block read\n");

	printf("Enter block # to read : ");
	block = GetIntNum();

	printf("Enter page # to read : ");
	page = GetIntNum();

	if (OND_page_load(block,page)==0) {

		memcpy( (U32 *)(_NONCACHE_STARTADDRESS+0x01000000), (U32 *)__OND_DataRAM0_BASE, DATABUFFERSIZE);

		Addr=(U32 *)(_NONCACHE_STARTADDRESS+0x01000000);
	
		for(i=0;i<DATABUFFERSIZE;i+=sizeof(U32)) {

			if ((i&0x1f)==0)
				printf("\n%08x : ",i);

			printf("%08x ",*Addr++);
		
		}
		putchar('\n');
	}
}


// temporary
void set_dma_control(U8 burst, U8 tw)
{
	return;
}


void dma_memcpy(U32 s_addr, U32 d_addr, U32 N_LLI, U32 trans_size)
{
	return;	
}
